/*
 * api/brow_sav.cpp
 *
 */

#include "brow_sav.h"

#include "../level/level.h" // fuer die haeufig benutzte/beispielhafte Box
#include "../other/globals.h"

namespace Api {

const int SaveBrowser::window_yl        (380);
const int SaveBrowser::frame_offset     ( 20);
const int SaveBrowser::info_file_name_y ( 40);
const int SaveBrowser::info_level_name_y( 60);
const int SaveBrowser::dir_list_y       (100);
const int SaveBrowser::dir_list_xl      (100);
const int SaveBrowser::dir_list_yl      (220);
const int SaveBrowser::file_list_xl     (360);

SaveBrowser::SaveBrowser(const std::string& bdir,
                         const std::string& ext,
                         const std::string& dir,
                               std::string file,
                               ListFile::SearchCrit crit,
                               BoxMessage* (*cmb)(const std::string&),
                         const bool replay_style)
:
    Window(*osd, 60, 20, frame_offset*3
     + dir_list_xl + file_list_xl,
     window_yl, Language::save_browser_title),

    dir_list  (get_torbit(), frame_offset,
     dir_list_y, dir_list_xl,
     dir_list_yl, bdir, dir),

    level_list(get_torbit(), 2 * frame_offset + dir_list_xl,
     dir_list_y, file_list_xl, dir_list_yl),

    file_name(get_torbit(), 2*frame_offset + dir_list_xl,
     dir_list_y + dir_list_yl + frame_offset, 140),

    ok    (get_torbit(), file_name.get_x() + 160, file_name.get_y(),  80),
    cancel(get_torbit(), file_name.get_x() + 260, file_name.get_y(), 100),

    exit_with(false),
    extension(ext),
    box_overwrite(0),
    new_box_overwrite(cmb)
{
    level_list.set_search_criterion(crit);
    level_list.set_write_file_names();
    if (replay_style) level_list.set_replay_style();
    level_list.load_dir(dir);
    set_subtitle(dir);

    draw_text(Language::editor_file_name,  frame_offset, info_file_name_y );
    draw_text(Language::editor_level_name, frame_offset, info_level_name_y);
    draw_text(Language::save_filename,     frame_offset, file_name.get_y());

    ok    .set_hotkey(KEY_ENTER);
    ok    .set_text  (Language::ok);
    cancel.set_text  (Language::cancel);
    // cancel.set_hotkey(KEY_ESC); // Wird nicht gemacht, weil es manuell
    // kontrolliert wird. Beim Texteintippen soll ESC nur das Eintippen abbr.

    Help::string_remove_dir(file);
    Help::string_remove_extension(file);
    file_name.set_text(file);
    file_name.set_scroll();
}

SaveBrowser::~SaveBrowser()
{
    if (box_overwrite) delete box_overwrite;
}









void SaveBrowser::undraw_self()
{
    if (box_overwrite) {
        box_overwrite->undraw();
    }
}



void SaveBrowser::calculate()
{
    // Unterfenster
    if (box_overwrite) {
        box_overwrite->calculate();
        box_overwrite->set_draw_required(); // Damit Buttons eingedrueckt werd.
        switch (box_overwrite->get_button_clicked()) {
        case 1:
            exit_with = true; // Ja, speichern
            set_exit();
            // faellt durch
        case 2:
            delete box_overwrite;
            box_overwrite = 0;
        default:
            ;
        }
    }

    // Dieses Fenster
    else {
        // Das Texteintippen vereinnahmt alle Benutzereingaben mit Ausnahme
        // der OK- und Abbrechen-Buttons, die werden dennoch ausgefuehrt.
        bool typing = false;
        if (file_name.get_on()) {
            file_name.calculate();
            typing = true;
        }

        if (!typing) {
            file_name.calculate(get_x(), get_y());

            if (hardware.key_once(KEY_ESC)) {
                set_exit();
            }
            else if (dir_list.clicked(get_x(), get_y())) {
                level_list.load_dir(dir_list.get_current_dir());
                set_subtitle(dir_list.get_current_dir());
            }
            else if (level_list.clicked(get_x(), get_y())) {
                std::string s = level_list.get_current_file();
                Help::string_remove_extension(s);
                Help::string_remove_dir(s);
                file_name.set_text(s);
                file_name.set_on();
            }
        }

        // Etwas aus editor/bit_brow.cpp kopierter Code
        if (hardware.get_mr()
         &&  (dir_list.is_mouse_here(get_x(), get_y())
         || level_list.is_mouse_here(get_x(), get_y()))) {
            if (dir_list.get_current_dir() == dir_list.get_base_dir()) {
                if (!typing) set_exit(true);
            }
            else {
                dir_list  .set_current_dir_to_parent_dir();
                level_list.load_dir(dir_list.get_current_dir());
            }
        }
        else if (button_clicked(ok)) {
            if (file_name.get_text().empty()) return; // Nichts passiert

            std::string complete_file_name = dir_list.get_current_dir()
             + file_name.get_text() + extension;

            // Testen, ob die Datei bereits exisitiert; ggf. box_overwrite new
            if (exists(complete_file_name.c_str())) {
                box_overwrite = new_box_overwrite(complete_file_name);
            }
            // Datei exisitiert noch nicht
            else {
                exit_with = true; // Lass den Editor den Level speichern
                set_exit();
            }
        }
        if (button_clicked(cancel)) {
            set_exit();
        }
    } // Ende von diesem Fenster
}



void SaveBrowser::draw()
{
    if (box_overwrite) {
        box_overwrite->draw();
    }
    else Window::draw();
}

void SaveBrowser::draw_self()
{
    dir_list  .draw();
    level_list.draw();
    file_name .draw();

    ok        .draw();
    cancel    .draw();
}



void SaveBrowser::set_info_file_name(const std::string& s)
{
    draw_text(s, level_list.get_x(), info_file_name_y);
}
void SaveBrowser::set_info_level_name(const std::string& s)
{
    draw_text(s, level_list.get_x(), info_level_name_y);
}



std::string SaveBrowser::get_current_file()
{
    if (file_name.get_text().empty()) return Globals::empty_string;
    else return file_name.get_text() + Globals::ext_level;
}

std::string SaveBrowser::get_current_dir_and_file()
{
    if (file_name.get_text().empty()) return Globals::empty_string;
    else {
        std::string str = dir_list.get_current_dir();
        str += file_name.get_text();
        str += Globals::ext_level;
        return str;
    }
}



// Hauefig benoetigt und ein gutes Beispiel:
// BoxMessage im Speicherdialog, und Suchkriterium
bool SaveBrowser::search_criterion_level(const std::string& s)
{
    return Help::string_ends_with(s, Globals::ext_level)
     ||    Help::string_ends_with(s, Globals::ext_level_orig);
}



BoxMessage* SaveBrowser::new_box_overwrite_level(const std::string&file)
{
    std::string s1 = Language::editor_file_name  + ' ' + file;
    std::string s2 = Language::editor_level_name + ' ' + Level::get_name(file);

    Api::BoxMessage* box_overwrite = new Api::BoxMessage(500, 3,
                                     Language::save_box_overwrite_title);

    box_overwrite->add_text(Language::save_box_overwrite_question);
    box_overwrite->add_text(s1);
    box_overwrite->add_text(s2);
    box_overwrite->add_button(Language::yes, KEY_ENTER);
    box_overwrite->add_button(Language::no,  KEY_ESC);

    return box_overwrite;
}

}
// Ende Namensraum Api
