/*
 * api_butt.cpp
 *
 */

#include "button.h"

#include "../other/hardware.h"
#include "../other/log.h"

namespace Api {

Button::Button(Torbit& b, int nx, int ny, int xl, int yl)
:
    Element(b, nx, ny, xl, yl),
    warm  (false),
    hot   (false),
    hotkey(0),
    down  (false),
    on    (false)
{
    // Farben werden ohnehin beim Zeichnen gewaehlt
}



Button::Button(const Button& b)
:
    Element(b)
{
    warm   = b.warm;
    hot    = b.hot;
    hotkey = b.hotkey;
    down   = b.down;
    on     = b.on;
    // Farben werden beim Zeichnen gewaehlt
}



Button::~Button()
{
    undraw();
}



// Liefert nur true, wenn der Button gerade fertig angeklickt wurde
bool Button::clicked(const int x_offset, const int y_offset)
{
    bool mouse_here = is_mouse_here(x_offset, y_offset);

    // Eingedrueckt erscheinen? Nur im kalten Modus oder bei ausgeschalteten
    // Buttons auch im warmen Modus kontrollieren, nie im heissen Modus.
    if (!hot) {
        if (mouse_here && hardware.get_mlh() && (!warm || !on)) {
            if (!down) set_draw_required();
            down = true;
        }
        else {
            if (down) set_draw_required();
            down = false;
        }
    }
    // Aktives Anklicken pruefen
    // KEY_ENTER als Hotkey laesst auch KEY_ENTER_PAD zu!
    bool ret =
        !warm && !hot && mouse_here && hardware.get_mlr()
     ||  warm && !hot && mouse_here && hardware.get_ml ()
     ||           hot && mouse_here && hardware.get_mlh();
    // See hardware.h/cpp for why Enter is separated
    if (hotkey == KEY_ENTER) ret = ret || hardware.key_enter_once();
    else                     ret = ret || hardware.key_once(hotkey);
    return ret;
}



void Button::draw_self() {
    // Diese hier machen das Zeichnen bersichtlicher.
    BITMAP* g    = get_ground().get_al_bitmap();
    const int x1 = get_x();
    const int y1 = get_y();
    const int x2 = get_x() + get_xl() - 1;
    const int y2 = get_y() + get_yl() - 1;

    if (down) {
        color_1    = color[COL_API_DOWN_D];
        color_2    = color[COL_API_DOWN_M];
        color_3    = color[COL_API_DOWN_L];
        color_text = color[COL_TEXT  ];
    } else if (on) {
        color_1    = color[COL_API_ON_D];
        color_2    = color[COL_API_ON_M];
        color_3    = color[COL_API_ON_L];
        color_text = color[COL_TEXT_ON   ];
    } else {
        color_1    = color[COL_API_L   ];
        color_2    = color[COL_API_M   ];
        color_3    = color[COL_API_D   ];
        color_text = color[COL_TEXT];
    }

    // Jetzt wird's ernst
    acquire_bitmap(g);

    // Jetzt malen!
    putpixel(g, x1,   y2,               color_2); // Unten links auen
    putpixel(g, x1+1, y2-1,             color_2); // Unten links innen
    putpixel(g, x2-1, y1+1,             color_2); // Oben rechts auen
    putpixel(g, x2,   y1,               color_2); // Oben rechts innen
    rectfill(g, x1+2, y1+2, x2-2, y2-2, color_2); // Mittelflche
    vline   (g, x1,   y1,         y2-1, color_1); // Links  auen
    vline   (g, x1+1, y1,         y2-2, color_1); // Links  innen
    hline   (g, x1+2, y1,   x2-1,       color_1); // Oben   auen
    hline   (g, x1+2, y1+1, x2-2,       color_1); // Oben   innen
    vline   (g, x2,   y1+1,       y2,   color_3); // Rechts auen
    vline   (g, x2-1, y1+2,       y2,   color_3); // Rechts innen
    hline   (g, x1+1, y2,   x2-2,       color_3); // Unten  auen
    hline   (g, x1+2, y2-1, x2-2,       color_3); // Unten  innen

    release_bitmap(g);
}

} // Api
