/*
 * ./api/button_s.cpp
 *
 */

#include "button_s.h"

#include "../graphic/gra_lib.h"
#include "../other/language.h" // Lemming-Bildname

namespace Api {

SkillButton::SkillButton(Torbit& b, const unsigned nx, const unsigned ny,
 const Lem::Ac ac, const int nr, const Lem::Style style)
:
    Button(b, nx, ny, 40, 60),
    lem(GraLib::get_lemming(style), b, get_x() + 3, get_y() + 23)
{
    set_skill(ac);
    set_number(nr);
}

SkillButton::~SkillButton()
{
}



void SkillButton::set_x(const int i)
{
    lem.set_x(i + 3);
    Element::set_x(i);
}

void SkillButton::set_y(const int i)
{
    lem.set_x(i + 23);
    Element::set_x(i);
}



void SkillButton::set_skill(const Lem::Ac ac)
{
    set_draw_required();
    skill = ac;
    lem.set_y_frame(skill - 1);
    if (skill == Lem::NOTHING) set_number(0);
}

void SkillButton::set_number(const unsigned nr)
{
    set_draw_required();
    number = (skill != Lem::NOTHING ? nr : 0);
    if (number > 0) {
        lem.set_x_frame(0);
    }
    else {
        set_off();
        lem.set_x_frame(1);
    }
}

void SkillButton::draw_self()
{
    Button::draw_self();
    if (skill != Lem::NOTHING) {
        lem.draw();
        if (number > 0) {
            std::string s;
            s += number;
            if (number >= Lem::infinity) s = "*";
            Help::draw_shadow_centered_text(get_ground(), font_big, s.c_str(),
             get_x()+20, get_y()+4, color[COL_TEXT_ON], color[COL_API_SHADOW]);
        }
    }
}

}  // Ende Namensraum

