/*
 * api_elem.cpp
 *
 */

#include "element.h"

namespace Api {

Element::Element(Torbit& gr, int nx, int ny, int nxl, int nyl)
:
    ground(gr),
    x     (nx),
    y     (ny),
    xl    (nxl),
    yl    (nyl),
    drawn (false),
    draw_required(true),
    undraw_color (0)
{
}



Element::Element(const Element& e)
:
    ground       (e.ground),
    x            (e.x),
    y            (e.y),
    xl           (e.xl),
    yl           (e.yl),
    drawn        (e.drawn),
    draw_required(e.draw_required),
    undraw_color (e.undraw_color)
{
}



Element& Element::operator = (const Element& e)
{
    if (this == &e || &ground != &e.ground) return *this;
    x             = e.x;
    y             = e.y;
    xl            = e.xl;
    yl            = e.yl;
    drawn         = e.drawn;
    draw_required = e.draw_required;
    undraw_color  = e.undraw_color;
    return *this;
}



Element::~Element()
{
}



void Element::set_x(const int i)
{
    x = i;
    draw_required = true;
}

void Element::set_y(const int i)
{
    y = i;
    draw_required = true;
}



void Element::set_xl(const unsigned i)
{
    xl = i;
    draw_required = true;
}
void Element::set_yl(const unsigned i)
{
    yl = i;
    draw_required = true;
}



bool Element::is_mouse_here(const int x_offset, const int y_offset) const {
    if (hardware.get_mx() >= x + x_offset && hardware.get_mx() < x + x_offset + xl
     && hardware.get_my() >= y + y_offset && hardware.get_my() < y + y_offset + yl) {
        return true;
    } else  return false;
}



void Element::draw() {
    if (draw_required) {
        draw_required = false;
        drawn = true;
        draw_self();
    }
}

void Element::undraw() {
    // Zuvor war undraw_self() ganz normal ein Teil von if (drawn).
    // Dort gab es aber den Fehler, dass die Speichernachfrage im Editor
    // nicht geloescht wurde.
    undraw_self();
    if (drawn) {
        if (undraw_color) {
            rectfill(ground.get_al_bitmap(),
             x, y, x+xl-1, y+yl-1, undraw_color);
            draw_required = true;
        }
        drawn = false;
    }
}

} // Api
