/*
 * element.h
 *
 * class Element
 *
 *   Grundlegende Eigenschaften eines Fensterelements sind gesammelt in dieser
 *   Basisklasse. Alle anderen Elemente sind hiervon abgeleitet mit
 *   "class SpeziellesElement : public Element". Zur Beachtung beim Schreiben
 *   neuer abgeleiteter Klassen ist auch, dass sie draw() und undraw() von
 *   Element aufrufen - um sicher zu gehen, dass drawn und draw_required
 *   korrekt gesetzt werden.
 *
 * bool is_mouse_here(int offset_x = 0, int offset_y = 0)
 *
 *   Diese Funktion braucht nur dann ein Offset, wenn ein Fenster abfragt und
 *   die Mauskoordinaten (0|0) nicht den Fensterkoordinaten (0|0) entsprechen.
 *   In diesem Fall sollte das Fenster seine eigenen Koordinaten mitteilen.
 *
 * int undraw_color
 *
 *   Wenn ein Element entzeichnet wird, so malt es einen rechteckigen Kasten
 *   auf seine Stelle im Bitmap "ground". Wenn undraw_color == 0 ist, wird
 *   dieser Schritt uebersprungen und undraw() setzt nur ein paar wenige
 *   Variablen zurueck. 0 ist Standardeinstellung, nicht color[COL_PINK]!
 *
 */

#pragma once

#include "../other/myalleg.h" // Die Api zeichnet eigenstndig

#include "../graphic/glob_gfx.h" // osd als Std-Bitmap fr Element::ground/Farben
#include "../other/hardware.h" // Maussteuerung
#include "../other/help.h"  // Schattentextzeichnen mit Allegro

namespace Api {
class Element {

private:

    Torbit& ground;

    int x;
    int y;

    int xl;
    int yl;

    bool drawn;
    bool draw_required;

    int  undraw_color;

public:

    //               Worauf?        x        y        Breite   Lnge
             Element(Torbit& = *osd, int = 0, int = 0, int = 0, int = 0);
             Element(const Element&);
    virtual ~Element();

    Element& operator = (const Element&);

    Torbit&      get_ground() const               { return ground;        }

    inline int   get_x() const                    { return x;             }
    inline int   get_y() const                    { return y;             }
    virtual void set_x(const int);
    virtual void set_y(const int);

    inline int   get_xl() const             { return xl;      }
    inline int   get_yl() const             { return yl;      }
    virtual void set_xl(const unsigned);
    virtual void set_yl(const unsigned);

    inline bool  is_drawn() const                 { return drawn;         }

    inline bool  get_draw_required() const        { return draw_required; }
    inline void  set_draw_required(bool b = true) { draw_required = b;    }

    // Hintergrundfarbe beim Entzeichnen gewuenscht? (siehe Headerkommentar)
    inline int   get_undraw_color() const { return undraw_color; }
    inline void  set_undraw_color(int i)  { undraw_color = i;    }

    bool        is_mouse_here(const int = 0, const int = 0) const; // Offset

           virtual void draw();
           virtual void undraw();

protected:

    inline virtual void draw_self()   {}
    inline virtual void undraw_self() {}

};
}
