/*
 * api_fram.cpp
 *
 */

#include "frame.h"

namespace Api {

Frame::Frame(Torbit& b, int nx, int ny, int xl, int yl)
:
    Element(b, nx, ny, xl, yl)
{
}

Frame::~Frame()
{
}

void Frame::draw_self()
{
    // Diese hier machen das Zeichnen bersichtlicher.
    BITMAP* g    = get_ground().get_al_bitmap();
    const int x1 = get_x() - 2;
    const int y1 = get_y() - 2;
    const int x2 = get_x() + 1 + get_xl();
    const int y2 = get_y() + 1 + get_yl();

    // Farben whlen
    int col_1 = color[COL_API_D];
    int col_3 = color[COL_API_L];

    // Acquiren, malen, freilassen
    acquire_bitmap(g);
    vline   (g, x1,   y1,         y2-1, col_1); // Links  auen
    vline   (g, x1+1, y1,         y2-2, col_1); // Links  innen
    hline   (g, x1+2, y1,   x2-1,       col_1); // Oben   auen
    hline   (g, x1+2, y1+1, x2-2,       col_1); // Oben   innen
    vline   (g, x2,   y1+1,       y2,   col_3); // Rechts auen
    vline   (g, x2-1, y1+2,       y2,   col_3); // Rechts innen
    hline   (g, x1+1, y2,   x2-2,       col_3); // Unten  auen
    hline   (g, x1+2, y2-1, x2-2,       col_3); // Unten  innen
    release_bitmap(g);
}

void Frame::undraw_self()
{
    if (!get_undraw_color()) return;

    BITMAP* g    = get_ground().get_al_bitmap();
    const int x1 = get_x() - 2;
    const int y1 = get_y() - 2;
    const int x2 = get_x() + 1 + get_xl();
    const int y2 = get_y() + 1 + get_yl();

    // Acquiren, malen, freilassen
    acquire_bitmap(g);
    vline   (g, x1,   y1,         y2-1, get_undraw_color()); // Links  auen
    vline   (g, x1+1, y1,         y2-2, get_undraw_color()); // Links  innen
    hline   (g, x1+2, y1,   x2-1,       get_undraw_color()); // Oben   auen
    hline   (g, x1+2, y1+1, x2-2,       get_undraw_color()); // Oben   innen
    vline   (g, x2,   y1+1,       y2,   get_undraw_color()); // Rechts auen
    vline   (g, x2-1, y1+2,       y2,   get_undraw_color()); // Rechts innen
    hline   (g, x1+1, y2,   x2-2,       get_undraw_color()); // Unten  auen
    hline   (g, x1+2, y2-1, x2-2,       get_undraw_color()); // Unten  innen
    release_bitmap(g);
}

} // Api
