/*
 * api/texttype.cpp
 *
 */

#include "texttype.h"

#include "../other/log.h"

namespace Api{

Texttype* Texttype::selection = 0;

// Konstruktoren und Destruktoren
Texttype::Texttype(Torbit& b, const int x,  const int y, const int xl)
:
    Button(b, x, y, xl, 20), // 20 ist generell bei Textelementen richtig
    invisible(false),
    scroll   (false)
{
}

Texttype::Texttype(Torbit& b, const int x,  const int y,
                              const int xl, std::string& t)
:
    Button(b, x, y, xl, 20),
    invisible(false),
    scroll   (false)
{
    set_text(t);
}

Texttype::Texttype(Texttype& tt)
:
    Button   (tt),
    invisible(tt.invisible),
    scroll   (tt.scroll)
{
    set_text(tt.text);
}

Texttype::~Texttype() {
    if (selection == this) selection = 0;
}








bool Texttype::get_too_long(const std::string t)
{
    return text_length(font_med, t.c_str()) > get_xl()-10;
}



void Texttype::set_text(const std::string& t)
{
    text = t;
    while (!scroll && get_too_long(text))          text.resize(text.size()-1);
    while (text.size()>0 && *text.rbegin() == ' ') text.resize(text.size()-1);
    set_draw_required();
}



void Texttype::set_on()
{
    // Schreibmodus beginnen
    Button::set_on();
    selection   = this;
    text_backup = text; // Fuer ESC-Druck/Rechtsklick
    clear_keybuf();
}



void Texttype::set_off()
{
    Button::set_off();
    selection = 0;
    set_text(text); // Leerzeichen am Ende abschneiden
}



void Texttype::calculate(const int x, const int y)
{
    if (!get_on()) {
        if (clicked(x, y)) set_on();
    }
    // Schreibmodus
    else {
        // Schreibmodus beenden? Einmal ohne, einmal mit Backup-Wiederherst.
        if (hardware.get_ml() || hardware.get_mr()
                              || hardware.key_enter_once()) {
            set_off();
        }
        else if (hardware.key_once(KEY_ESC)) {
            set_off();
            text = text_backup;
        }
        // Oder doch noch im Schreibmodus bleiben? Dann Tastatur lesen.
        else {
            int  k      = hardware.get_key();
            char kascii = hardware.get_key_ascii();

            // Zeichen verarbeiten
            if (!k) return; // Aus Zeitgruenden
            else if (k == KEY_BACKSPACE && text.size() > 0) {
                text.resize(text.size()-1);
            }
            else if ((k >= KEY_A     && k <= KEY_9    )  || k == KEY_SPACE
             ||      (k >= KEY_0_PAD && k <= KEY_9_PAD)  || k == KEY_DEL_PAD
             || k == KEY_STOP      || k == KEY_COMMA     || k == KEY_COLON
             || k == KEY_MINUS     || k == KEY_PLUS_PAD  || k == KEY_EQUALS
             || k == KEY_QUOTE     || k == KEY_SLASH     || k == KEY_CLOSEBRACE
             || k == KEY_OPENBRACE || k == KEY_SEMICOLON || k == KEY_ASTERISK)
            {
                text += kascii;
                if (!scroll && get_too_long(text)) text.resize(text.size()-1);
            }
            // Ende Tastenverarbeitung
        }
        // Ende Schreibmodus
    }
    // Ende Abfrage, ob man im Schreibmodus ist
}








void Texttype::draw_self()
{
    // Zeichnen vorbereiten
    // td ist "text to display";
    std::string td;
    int         text_color    = color[COL_TEXT];
    bool        align_right   = false;
    bool        caret_visible = false;

    if (scroll && get_too_long(text)) {
        align_right = true;
        int i = text.size() - 1;
        while (!get_too_long(td) && i >= 0) td = text.substr(i--);
        td.erase(0, 1);
    }
    else td = text;

    if (get_on() && (!get_down() || invisible)) {
        set_draw_required();
        text_color = color[COL_TEXT_ON];
        if (Help::timer_ticks % 60 < 30) {
            td += '^';
            caret_visible = true;
        }
    }

    // Gut. Jetzt zeichnen!
    if (!invisible) Button::draw_self();
    Help::draw_shadow_text(get_ground(), font_med,
     td.c_str(), align_right ? get_x() + get_xl() - 7
     - text_length(font_med, td.c_str())
     + text_length(font_med, "^") * caret_visible
     : get_x() + 3, get_y(), text_color, color[COL_API_SHADOW]);
}






} // Ende Namensraum Api
