/*
 * api/texttype.h
 *
 * Ein Button mit Texteingabezeile. Sobald das Feld angeklickt wird, nimmt es
 * Zeichen von der Tastatur auf. Damit das Ganze nicht zu kompliziert zu
 * programmieren wird, kann der Benutzer nur am Ende der Eingabezeile
 * den Text bearbeiten. [Backspace] loescht jedoch wie gewohnt :-)
 *
 * Ein Mausklick, ein Druck auf [Enter] oder [ESC] beendet den Schreibmodus.
 *
 * static Texttype* selection
 *
 *   Zeigt dies auf 0, so durchlaufen alle Fenster die normalen Schleifen.
 *   Ein Fenster mit Texteingabezeilen sollte jedoch pruefen, ob dieser
 *   Zeiger irgendwohin zeigt: In diesem Fall sollte nur die Hauptschleife
 *   des Zieles ausgefuehrt werden, damit Klicks und Tastendruecke nicht mehr
 *   als eine Aktion verursachen. Wird das Zeigerziel geloescht, so wird der
 *   Zeiger per Destruktor auf 0 gesetzt.
 *
 * void set_text(std::string& t)
 *
 *   Setzt den Text der Eingabezeile auf den String t, aber schneidet aehnlich
 *   wie bei der manuellen Eingabe ab, wenn der Text zu lang ist.
 *
 */

#pragma once

#include <string>

#include "button.h"

namespace Api {
class Texttype : public Button {

private:

    static Texttype*   selection;

    bool        invisible;   // Keinen Button zeichnen, nur den Eingabetext
    bool        scroll;      // Mehr Text eingebbar, als das Feld lang ist

    std::string text;
    std::string text_backup; // Bei ESC/Rechtsklick wird Zurueckgeschrieben

    bool        get_too_long(const std::string);
    bool        get_too_long() { return get_too_long(text); }

public:

    Texttype(Torbit&, const int, const int, const int);
    Texttype(Torbit&, const int, const int, const int, std::string&);
    Texttype(Texttype&);
    virtual ~Texttype();

    inline static Texttype* get_selection() { return selection; }

    inline void         set_invisible(const bool b = true) { invisible = b;   }
    inline void         set_scroll   (const bool b = true) { scroll    = b;   }
    inline std::string& get_text     ()                    { return text;     }
           void         set_text     (const std::string& t);

    inline bool         get_on() { return selection == this; }
           void         set_on();
           void         set_off();

           void         calculate(const int = 0, const int = 0);

protected:

    virtual void        draw_self();

}; // Ende Texttype
}  // Ende Namensraum
