/*
 * api_wind.cpp
 *
 */

#include "window.h"

namespace Api {

const int Window::title_bar_height(20);

Window::Window(Torbit& b, const int nx, const int ny,
                          const unsigned xl, const unsigned yl,
                          const std::string& tit)
:
    Element(b, nx, ny, xl, yl),
    torbit (xl, yl),
    draw_title_required(true),
    title(tit),
    subtitle()
{
    exit = false;
    x_drawn = 0;
    y_drawn = 0;

    // Loeschen beim Entzeichnen
    set_undraw_color(color[COL_PINK]);

    // Minimalbreite und -lnge bercksichtigen
    if (get_xl() < 10)                 set_xl(10);
    if (get_yl() <title_bar_height+10) set_yl(title_bar_height+10);
    torbit.resize(get_xl(), get_yl());

    prepare_torbit();
}



Window::Window(const Window& w)
:
    Element(w),
    torbit (w.get_torbit())
{
    x_drawn  = w.x_drawn;
    y_drawn  = w.y_drawn;
    exit     = w.exit;
    title    = w.title;
    subtitle = w.subtitle;

    draw_title_required = true;
}



Window::~Window()
{
}





// Private Funktionen

// Titel wird hier nicht gemalt, sondern beim ersten Zeichnen!
void Window::prepare_torbit()
{
    BITMAP*   g     = torbit.get_al_bitmap();
    const int col_1 = color[COL_API_L];
    const int col_2 = color[COL_API_M];
    const int col_3 = color[COL_API_D];
    const int x1    = 0;
    const int y1    = title_bar_height;
    const int x2    = get_xl() - 1; // -1, denn die Lnge zhlt
    const int y2    = get_yl() - 1; // Pixel an Position 0 mit

    // Fenster malen
    acquire_bitmap(g);

    putpixel(g, x1,   y2,               col_2); // Unten links auen
    putpixel(g, x1+1, y2-1,             col_2); // Unten links innen
    rectfill(g, x1+2, y1,   x2-2, y2-2, col_2); // Mittelflche
    vline   (g, x1,   y1,         y2-1, col_1); // Links  auen
    vline   (g, x1+1, y1,         y2-2, col_1); // Links  innen
    vline   (g, x2,   y1,         y2,   col_3); // Rechts auen
    vline   (g, x2-1, y1,         y2,   col_3); // Rechts innen
    hline   (g, x1+1, y2,   x2-2,       col_3); // Unten  auen
    hline   (g, x1+2, y2-1, x2-2,       col_3); // Unten  innen

    release_bitmap(g);
}






void Window::draw_text(const std::string& s, const int x, const int y,
                       const int col)
{
    Help::draw_shadow_text(torbit, font_med, s.c_str(), x, y,
     col, color[COL_API_SHADOW]);
}

void Window::draw_centered_text(const std::string& s, const int x, const int y,
                                const int col)
{
    Help::draw_shadow_centered_text(torbit, font_med, s.c_str(), x, y,
     col, color[COL_API_SHADOW]);
}



void Window::set_title(const std::string& s)
{
    title = s;
    draw_title_required = true;
}

void Window::set_subtitle(const std::string& s)
{
    subtitle = s;
    draw_title_required = true;
}



void Window::draw()
{
    x_drawn = get_x();
    y_drawn = get_y();
    // Erst einmal Variablen richtig setzen und, da Element::draw() auch
    // die virtuelle Zeichenmethode aufruft, Fenster aufs eigene Bitmap zeichn.
    Element::draw();

    if (draw_title_required) {
        draw_title_required = false;
        draw_title();
    }
    // Eigenes Bitmap auf die Unterlage zeichnen
    torbit.draw(get_ground(), get_x(), get_y());
}



void Window::draw_title()
{
    // Variablen fuer die Titelflaeche
    BITMAP *g       = torbit.get_al_bitmap();
    const int col_1 = color[COL_API_ON_L];
    const int col_2 = color[COL_API_ON_M];
    const int col_3 = color[COL_API_ON_D];
    const int x1    = 0;
    const int y1    = 0;
    const int x2    = get_xl() - 1; // -1, denn Lnge zhlt
    const int y2    = title_bar_height - 1;

    // Jetzt wird's ernst
    acquire_bitmap(g);

    // Titel malen
    putpixel(g, x1,   y2,               col_2); // Unten links auen
    putpixel(g, x1+1, y2-1,             col_2); // Unten links innen
    putpixel(g, x2-1, y1+1,             col_2); // Oben rechts auen
    putpixel(g, x2,   y1,               col_2); // Oben rechts innen
    rectfill(g, x1+2, y1+2, x2-2, y2-2, col_2); // Mittelflche
    vline   (g, x1,   y1,         y2-1, col_1); // Links  auen
    vline   (g, x1+1, y1,         y2-2, col_1); // Links  innen
    hline   (g, x1+2, y1,   x2-1,       col_1); // Oben   auen
    hline   (g, x1+2, y1+1, x2-2,       col_1); // Oben   innen
    vline   (g, x2,   y1+1,       y2,   col_3); // Rechts auen
    vline   (g, x2-1, y1+2,       y2,   col_3); // Rechts innen
    hline   (g, x1+1, y2,   x2-2,       col_3); // Unten  auen
    hline   (g, x1+2, y2-1, x2-2,       col_3); // Unten  innen

    std::string real_title = title;
    if (!subtitle.empty()) {
        real_title += " - ";
        real_title += subtitle;
    }
    Help::draw_shadow_centered_text(torbit, font_med, real_title.c_str(),
     x1+get_xl()/2, y1, color[COL_TEXT_ON], color[COL_API_SHADOW]);

    release_bitmap(g);
}

}
// Ende von namespace Api
