/*
 * editor/win_scro.cpp
 *
 */

#include "win_scro.h"

#include "../other/language.h"

namespace Api {

const unsigned WindowScroll::this_xl(420);
const unsigned WindowScroll::this_yl(200);
const unsigned WindowScroll::nrxl(180); // Number X-Length
const unsigned WindowScroll::rgxl(180); // Regular X-Length
const unsigned WindowScroll::step_small(10); // Fuer Rasterbestimmung

WindowScroll::WindowScroll(Level& l, Map& m)
:
    Window(*osd, 60, 60, this_xl,this_yl,Language::win_scroll_title),

    level(l),
    map  (m),

    x(get_torbit(), this_xl-20-nrxl, 40, nrxl, 4, 0,
     level.torus_x ? level.size_x : level.size_x - Level::view_xl,
     level.start_x, true),
    y(get_torbit(), this_xl-20-nrxl, 80, nrxl, 4, 0,
     level.torus_y ? level.size_y : level.size_y - Level::view_yl,
     level.start_y, true),
    jump   (get_torbit(), 20,              120, rgxl),
    current(get_torbit(), 20,              160, rgxl),
    ok     (get_torbit(), this_xl-20-nrxl, 120, nrxl),
    cancel (get_torbit(), this_xl-20-nrxl, 160, nrxl)
{
    x.set_step_sml  ( 10);
    y.set_step_sml  ( 10);
    x.set_step_med  (100);
    y.set_step_med  (100);
    x.set_step_big  (Level::view_xl);
    y.set_step_big  (Level::view_yl);
    x.set_white_zero();
    y.set_white_zero();
    draw_text(Language::win_scroll_x, 20, 40);
    draw_text(Language::win_scroll_y, 20, 80);
    jump   .set_text(Language::win_scroll_jump   );
    current.set_text(Language::win_scroll_current);
    ok     .set_text(Language::ok                );
    cancel .set_text(Language::cancel            );
    ok     .set_hotkey(KEY_ENTER);
    cancel .set_hotkey(KEY_ESC);
}

WindowScroll::~WindowScroll()
{
}





void WindowScroll::calculate()
{
    x.calculate(get_x(), get_y());
    y.calculate(get_x(), get_y());

    if (button_clicked(jump)) {
        map.set_screen_x(x.get_number());
        map.set_screen_y(y.get_number());
    }
    else if (button_clicked(current)) {
        x.set_number((map.get_screen_x()+step_small/2)/step_small*step_small);
        y.set_number((map.get_screen_y()+step_small/2)/step_small*step_small);
        map.set_screen_x(x.get_number());
        map.set_screen_y(y.get_number());
    }
    else if (button_clicked(ok) || hardware.get_mr()) {
        level.start_x = Help::mod(x.get_number(), level.size_x);
        level.start_y = Help::mod(y.get_number(), level.size_y);
        set_exit();
    }
    else if (button_clicked(cancel)) set_exit();
}



void WindowScroll::draw_self()
{
    x.draw();
    y.draw();
    jump.draw();
    current.draw();
    ok.draw();
    cancel.draw();
}

} // Ende Namensraum Api
