/*
 * editor/win_size.cpp
 *
 */

#include "win_size.h"

#include "../other/language.h"

namespace Api {

const unsigned WindowSize::this_xl (300);
const unsigned WindowSize::this_yl (320);
const unsigned WindowSize::nrxl    (200); // Number X-Length
const unsigned WindowSize::x_offset(this_xl - 20 - nrxl);

const unsigned WindowSize::y_l      (                        40);
const unsigned WindowSize::y_r      (WindowSize::y_l       + 20);
const unsigned WindowSize::y_u      (WindowSize::y_r       + 40);
const unsigned WindowSize::y_d      (WindowSize::y_u       + 20);
const unsigned WindowSize::y_x      (WindowSize::y_d       + 40);
const unsigned WindowSize::y_y      (WindowSize::y_x       + 20);
const unsigned WindowSize::y_torus_x(WindowSize::y_y       + 40);
const unsigned WindowSize::y_torus_y(WindowSize::y_torus_x + 20);

const int WindowSize::min_x(Level::min_xl);
const int WindowSize::min_y(Level::min_yl);
const int WindowSize::max_x(Level::view_xl * 5);
const int WindowSize::max_y(Level::view_yl * 5);

WindowSize::WindowSize(Level& lv, Map& mp)
:
    Window(*osd, 60, 60, this_xl,
     this_yl, Language::win_size_title),

    level(lv),
    map  (mp),

    exit_with_x_to_adjust(0),
    exit_with_y_to_adjust(0),

    // Minimum und Maximum dieser vier Einstell-Zahlen werden in
    // calculate() staendig neu gesetzt.
    l(get_torbit(), x_offset, y_l, nrxl, 5, -max_x, max_x, 0, true),
    r(get_torbit(), x_offset, y_r, nrxl, 5, -max_x, max_x, 0, true),
    u(get_torbit(), x_offset, y_u, nrxl, 5, -max_y, max_y, 0, true),
    d(get_torbit(), x_offset, y_d, nrxl, 5, -max_y, max_y, 0, true),

    torus_x(get_torbit(), 20, y_torus_x),
    torus_y(get_torbit(), 20, y_torus_y),

    ok    (get_torbit(),  20, this_yl - 40, 120),
    cancel(get_torbit(), 160, this_yl - 40, 120)
{
    l.set_step_sml(2); l.set_step_med(20); l.set_step_big(min_x / 4);
    r.set_step_sml(2); r.set_step_med(20); r.set_step_big(min_x / 4);
    u.set_step_sml(2); u.set_step_med(20); u.set_step_big(140);
    d.set_step_sml(2); d.set_step_med(20); d.set_step_big(140);

    torus_x.set_checked(level.torus_x);
    torus_y.set_checked(level.torus_y);

    ok    .set_text(Language::ok);
    cancel.set_text(Language::cancel);
    ok    .set_hotkey(KEY_ENTER);
    cancel.set_hotkey(KEY_ESC);

    draw_text(Language::win_size_l, 20, l.get_y());
    draw_text(Language::win_size_r, 20, r.get_y());
    draw_text(Language::win_size_u, 20, u.get_y());
    draw_text(Language::win_size_d, 20, d.get_y());

    draw_text(Language::win_size_x, 20, y_x);
    draw_text(Language::win_size_y, 20, y_y);

    draw_text(Language::win_size_torus_x, 60, y_torus_x);
    draw_text(Language::win_size_torus_y, 60, y_torus_y);
}

WindowSize::~WindowSize()
{
}

void WindowSize::calculate()
{
    l.calculate(get_x(), get_y());
    if      (map.get_xl() + l.get_number() + r.get_number() < (int) min_x)
     l.set_number(min_x - map.get_xl() - r.get_number());
    else if (map.get_xl() + l.get_number() + r.get_number() > (int) max_x)
     l.set_number(max_x - map.get_xl() - r.get_number());

    r.calculate(get_x(), get_y());
    if      (map.get_xl() + l.get_number() + r.get_number() < (int) min_x)
     r.set_number(min_x - map.get_xl() - l.get_number());
    else if (map.get_xl() + l.get_number() + r.get_number() > (int) max_x)
     r.set_number(max_x - map.get_xl() - l.get_number());

    u.calculate(get_x(), get_y());
    if      (map.get_yl() + u.get_number() + d.get_number() < (int) min_y)
     u.set_number(min_y - map.get_yl() - d.get_number());
    else if (map.get_yl() + u.get_number() + d.get_number() > (int) max_y)
     u.set_number(max_y - map.get_yl() - d.get_number());

    d.calculate(get_x(), get_y());
    if      (map.get_yl() + u.get_number() + d.get_number() < (int) min_y)
     d.set_number(min_y - map.get_yl() - u.get_number());
    else if (map.get_yl() + u.get_number() + d.get_number() > (int) max_y)
     d.set_number(max_y - map.get_yl() - u.get_number());

    torus_x.clicked(get_x(), get_y());
    torus_y.clicked(get_x(), get_y());

    if (button_clicked(ok) || hardware.get_mr()) {
        if (l.get_number() == 0 && r.get_number() == 0
         && u.get_number() == 0 && d.get_number() == 0
         && level.torus_x == torus_x
         && level.torus_y == torus_y) set_exit();
        else {
            exit_with_x_to_adjust = l.get_number();
            exit_with_y_to_adjust = u.get_number();
            map.set_torus_x(torus_x);
            map.set_torus_y(torus_y); // Hier ok: resize callt zoom()
            map.resize(map.get_xl() + l.get_number() + r.get_number(),
                       map.get_yl() + u.get_number() + d.get_number());
            map.set_screen_x(map.get_screen_x() + exit_with_x_to_adjust);
            map.set_screen_y(map.get_screen_y() + exit_with_y_to_adjust);
            level.size_x = map.get_xl();
            level.size_y = map.get_yl();
            level.start_x += exit_with_x_to_adjust;
            level.start_y += exit_with_y_to_adjust;
            level.torus_x = torus_x;
            level.torus_y = torus_y;
            if (torus_x) level.start_x = Help::mod(level.start_x,level.size_x);
            if (torus_y) level.start_y = Help::mod(level.start_y,level.size_y);
            if (!torus_x && level.start_x > level.size_x - Level::view_xl)
                            level.start_x = level.size_x - Level::view_xl;
            if (!torus_y && level.start_y > level.size_y - Level::view_yl)
                            level.start_y = level.size_y - Level::view_yl;
            set_exit();
        }
    }
    else if (button_clicked(cancel)) set_exit();
}





void WindowSize::draw_self()
{
    l.draw();
    r.draw();
    u.draw();
    d.draw();

    torus_x.draw();
    torus_y.draw();

    ok    .draw();
    cancel.draw();

    draw_calculation(y_x + 1, map.get_xl(), l.get_number() + r.get_number());
    draw_calculation(y_y + 1, map.get_yl(), u.get_number() + d.get_number());
}

// old, plus
void WindowSize::draw_calculation(const int y, const int o, const int p)
{
    std::string s_old(4, ' ');
    std::string s_add(4, ' ');
    std::string s_new(4, ' ');
    std::string s_pls(p < 0 ? "-" : "+");
    std::string s_eql("=");

    Torbit& tb = get_torbit();
    BITMAP* bp = tb.get_al_bitmap();

    // Alte Zahl wegputzen
    rectfill(bp, x_offset, y, this_xl-2,y+19,color[COL_API_M]);

    for (int i = 3, temp = o; i >= 0 && temp > 0; --i) {
        s_old[i] = '0' + temp%10;
        temp /= 10;
    }
    for (int i = 3, temp = o+p; i >= 0 && temp > 0; --i) {
        s_new[i] = '0' + temp%10;
        temp /= 10;
    }
    s_add[3] = '0';
    for (int i = 3, temp = (p < 0) ? -p : p; i >= 0 && temp > 0; --i) {
        s_add[i] = '0' + temp%10;
        temp /= 10;
    }
    Help::draw_shadow_fixed_text(tb, font_med, s_old, x_offset+  0, y, color[COL_TEXT]);
    Help::draw_shadow_fixed_text(tb, font_med, s_pls, x_offset+ 55, y, color[COL_TEXT]);
    Help::draw_shadow_fixed_text(tb, font_med, s_add, x_offset+ 85, y, color[COL_TEXT]);
    Help::draw_shadow_fixed_text(tb, font_med, s_eql, x_offset+135, y, color[COL_TEXT]);
    Help::draw_shadow_fixed_text(tb, font_med, s_new, x_offset+160, y, color[COL_TEXT_ON]);
}

} // Ende Namensraum Api
