/*
 * ./editor/win_skil.cpp
 *
 */

#include "win_skil.h"

#include "../graphic/gra_lib.h"
#include "../other/language.h"

namespace Api {

const int WindowSkill::y_sbwb(140);
const int WindowSkill::this_xl(430);
const int WindowSkill::this_yl(y_sbwb + 190);

WindowSkill::WindowSkill(Level& l)
:
    Window(*osd, 105, 50, this_xl, this_yl,
     Language::win_skill_title),

    level(&l),

    cur_pos (0),
    cur_sbwb(0),

    clear   (get_torbit(),  20, y_sbwb + 120, 110),
    standard(get_torbit(),  20, y_sbwb + 150, 110),
    all_to  (get_torbit(), 150, y_sbwb + 120, 130),
    all_to_n(get_torbit(), 150, y_sbwb + 150, 130, 2, 0, 100, 0),
    ok      (get_torbit(), 300, y_sbwb + 120, 110),
    cancel  (get_torbit(), 300, y_sbwb + 150, 110)
{
    // Moegliche Faehigkeiten
    unsigned int i = 0;
    Torbit&      b = get_torbit();
    typedef PossibleSkill PosSk;

    possible.push_back(PosSk(b, px(i), py(i), Lem::WALKER));     ++i;
    possible.push_back(PosSk(b, px(i), py(i), Lem::JUMPER));     ++i;
    possible.push_back(PosSk(b, px(i), py(i), Lem::EXPLODER));   ++i;
    possible.push_back(PosSk(b, px(i), py(i), Lem::NOTHING));    ++i;
    possible.push_back(PosSk(b, px(i), py(i), Lem::PLATFORMER)); ++i;
    // possible.push_back(PosSk(b, px(i), py(i), Lem::ROPER));      ++i;
    possible.push_back(PosSk(b, px(i), py(i), Lem::NOTHING));    ++i;
    possible.push_back(PosSk(b, px(i), py(i), Lem::NOTHING));    ++i;
    possible.push_back(PosSk(b, px(i), py(i), Lem::NOTHING));    ++i;

    possible.push_back(PosSk(b, px(i), py(i), Lem::CLIMBER));    ++i;
    possible.push_back(PosSk(b, px(i), py(i), Lem::FLOATER));    ++i;
    possible.push_back(PosSk(b, px(i), py(i), Lem::EXPLODER2));  ++i;
    possible.push_back(PosSk(b, px(i), py(i), Lem::BLOCKER));    ++i;
    possible.push_back(PosSk(b, px(i), py(i), Lem::BUILDER));    ++i;
    possible.push_back(PosSk(b, px(i), py(i), Lem::BASHER));     ++i;
    possible.push_back(PosSk(b, px(i), py(i), Lem::MINER));      ++i;
    possible.push_back(PosSk(b, px(i), py(i), Lem::DIGGER));     ++i;

    // Tatsaechliche Faehigkeiten
    for (i = 0; i < 8; ++i) {            // *50 = Abstand von 10, da 40 breit
        sbwb.push_back(SkillButtonWithButtons(b, 20 + i*50, y_sbwb,
         level->skill[i].ac, level->skill[i].nr));
    }

    // Eventuell bei den moeglichen was ausgrauen?
    for (PosIt i = possible.begin(); i != possible.end(); ++i)
     for (SbwbIt j = sbwb.begin();   j != sbwb    .end(); ++j)
      if (i->ac == j->skill.get_skill()) i->set_color(false);

    clear   .set_text(Language::win_skill_clear);
    standard.set_text(Language::win_skill_standard);
    all_to  .set_text(Language::win_skill_all_to);
    ok      .set_text(Language::ok);
    cancel  .set_text(Language::cancel);

    ok      .set_hotkey(KEY_ENTER);
    cancel  .set_hotkey(KEY_ESC);
}

WindowSkill::~WindowSkill()
{
}



void WindowSkill::swap_sbwbs(SkillButtonWithButtons* a,
                             SkillButtonWithButtons* b)
{
    if (a == b) return;
    Lem::Ac tempa = a->skill.get_skill ();
    unsigned    tempi = a->skill.get_number();
    a->skill.set_skill (b->skill.get_skill ());
    a->skill.set_number(b->skill.get_number());
    b->skill.set_skill (tempa);
    b->skill.set_number(tempi);
}

void WindowSkill::assign_sbwb_pos(SkillButtonWithButtons* b,
                                  PossibleSkill* p)
{
    Lem::Ac bac = b->skill.get_skill();
    if (bac == p->ac || p->ac == Lem::NOTHING) {
        for (PosIt i = possible.begin(); i != possible.end(); ++i)
         if (i->ac == bac) {
            i->set_color(true);
            break;
        }
        b->skill.set_skill(Lem::NOTHING);
    }
    else {
        bool swapped = false;
        for (SbwbIt i = sbwb.begin(); i != sbwb.end(); ++i)
         if (&*i != b && i->skill.get_skill() == p->ac) {
            swapped = true;
            swap_sbwbs(b, &*i);
            break;
        }
        if (!swapped) {
            // Bisherigen Inhalt des Sbwbs oben wieder farbig machen
            for (PosIt i = possible.begin(); i != possible.end(); ++i)
             if (i->ac == bac) {
                i->set_color(true);
                break;
            }
            b->skill.set_skill(p->ac);
            p->set_color(false);
        }
    }
}



void WindowSkill::calculate()
{
    for (PosIt  i = possible.begin(); i != possible.end(); ++i) {
        if (button_clicked(*i)) {
            if (cur_sbwb != 0) {
                assign_sbwb_pos(cur_sbwb, &*i);
                cur_sbwb->skill.set_off();
                cur_sbwb = 0;
            }
            else if (cur_pos != 0) {
                cur_pos->set_off();
                cur_pos = (&*i != cur_pos ? &*i : 0);
                if (cur_pos) cur_pos->set_on();
            }
            else {
                cur_pos = &*i;
                cur_pos->set_on();
            }
        }
    }
    for (SbwbIt i = sbwb.begin(); i != sbwb.end(); ++i) {
        i->calculate(get_x(), get_y());
        if (button_clicked(i->skill)) {
            if (cur_pos != 0) {
                assign_sbwb_pos(&*i, cur_pos);
                cur_pos->set_off();
                cur_pos = 0;
            }
            else if (cur_sbwb != 0
             && cur_sbwb->skill.get_skill() == Lem::NOTHING
             && i       ->skill.get_skill() == Lem::NOTHING) {
                cur_sbwb->skill.set_off();
                cur_sbwb = &*i;
                cur_sbwb->skill.set_on();
            }
            else if (cur_sbwb != 0) {
                swap_sbwbs(&*i, cur_sbwb);
                cur_sbwb->skill.set_off();
                cur_sbwb = 0;
            }
            else {
                cur_sbwb = &*i;
                cur_sbwb->skill.set_on();
            }
        }
    }
    // Ende vom Faehigkeitenzuweisen



    all_to_n.calculate(get_x(), get_y());

    if (button_clicked(ok) || hardware.get_mr()) {
        // Daten in das Levelobjekt des Editors schreiben
        for (unsigned i = 0; i < 8; ++i) {
            level->skill[i].ac = sbwb[i].skill.get_skill();
            level->skill[i].nr = sbwb[i].skill.get_number();
        }
        set_exit();
    }
    else if (button_clicked(cancel)) {
        set_exit();
    }
    else if (button_clicked(clear)) {
        for (SbwbIt i = sbwb.begin(); i != sbwb.end(); ++i)
         i->skill.set_skill(Lem::NOTHING);
        for (PosIt i = possible.begin(); i != possible.end(); ++i)
         i->set_color(true);
        if (cur_sbwb) {
            cur_sbwb->skill.set_off();
            cur_sbwb = 0;
        }
        if (cur_pos) {
            cur_pos->set_off();
            cur_pos = 0;
        }
    }
    else if (button_clicked(standard)) {
        sbwb[0].skill.set_skill(Lem::CLIMBER);
        sbwb[1].skill.set_skill(Lem::FLOATER);
        sbwb[2].skill.set_skill(Lem::EXPLODER2);
        sbwb[3].skill.set_skill(Lem::BLOCKER);
        sbwb[4].skill.set_skill(Lem::BUILDER);
        sbwb[5].skill.set_skill(Lem::BASHER);
        sbwb[6].skill.set_skill(Lem::MINER);
        sbwb[7].skill.set_skill(Lem::DIGGER);
        for (PosIt i = possible.begin(); i != possible.end(); ++i) {
            if (i->ac == Lem::CLIMBER
             || i->ac == Lem::FLOATER
             || i->ac == Lem::EXPLODER2
             || i->ac == Lem::BLOCKER
             || i->ac == Lem::BUILDER
             || i->ac == Lem::BASHER
             || i->ac == Lem::MINER
             || i->ac == Lem::DIGGER) i->set_color(false);
            else                      i->set_color(true);
        }
    }
    else if (button_clicked(all_to)) {
        for (unsigned i = 0; i < 8; ++i)
         sbwb[i].skill.set_number(all_to_n.get_number());
    }
}



void WindowSkill::draw_self()
{
    for (std::vector <PossibleSkill> ::iterator i = possible.begin();
     i != possible.end(); ++i) i->draw();
    for (std::vector <SkillButtonWithButtons> ::iterator i = sbwb.begin();
     i != sbwb.end();     ++i) i->draw();

    clear   .draw();
    standard.draw();
    all_to  .draw();
    all_to_n.draw();
    ok      .draw();
    cancel  .draw();
}



// ############################################################################
// ############################################################################
// ############################################################################



const unsigned PossibleSkill::this_length(40);

PossibleSkill::PossibleSkill(Torbit& b,
 const int nx, const int ny, const Lem::Ac nac, const Lem::Style sk)
:
    Button(b, nx, ny, this_length, this_length),
    lem   (GraLib::get_lemming(sk), b, get_x() + 2, get_y() + 3),
    ac    (nac)
{
    if (ac != Lem::NOTHING) lem.set_y_frame(ac - 1);
    else                        lem.set_y_frame(0);
}

PossibleSkill::~PossibleSkill()
{
}



bool PossibleSkill::get_color()
{
    set_draw_required();
    return !lem.get_x_frame();
}

void PossibleSkill::set_color(const bool b)
{
    lem.set_x_frame(b ? 0 : 1);
    set_draw_required();
}



void PossibleSkill::draw_self()
{
    Button::draw_self();
    lem.draw();
}



// ############################################################################
// ############################################################################
// ############################################################################



SkillButtonWithButtons::SkillButtonWithButtons(Torbit& b, const unsigned nx,
 const unsigned ny, const Lem::Ac ac, const int nr)
:
    Element   (b,         nx,    ny,      40, 60+40),
    skill     (b,         nx,    ny + 40, ac, nr),
    up_big    (b, GraLib::get(Globals::file_bitmap_api_number), nx   , ny   ),
    up_small  (b, GraLib::get(Globals::file_bitmap_api_number), nx+20, ny   ),
    down_big  (b, GraLib::get(Globals::file_bitmap_api_number), nx   , ny+20),
    down_small(b, GraLib::get(Globals::file_bitmap_api_number), nx+20, ny+20)
{
    up_big    .set_x_frame(6);
    up_small  .set_x_frame(7);
    down_big  .set_x_frame(8);
    down_small.set_x_frame(9);
}

SkillButtonWithButtons::~SkillButtonWithButtons()
{
}






void SkillButtonWithButtons::calculate(const int cx, const int cy)
{
    // Wenn gesetzt, haben Klicks keine Wirkung.
    bool t = skill.get_skill() != Lem::NOTHING;
    const unsigned& infty = Lem::infinity;

    if (up_big    .clicked(cx, cy) && t) {
        skill.set_number(skill.get_number() + 10);
        if (skill.get_number() > infty) {
            if (skill.get_number() == infty + 10) skill.set_number(    0);
            else                                  skill.set_number(infty);
        }
    }
    else if (up_small  .clicked(cx, cy) && t) {
        skill.set_number(skill.get_number() + 1);
        if (skill.get_number() > infty) skill.set_number(0);
    }
    else if (down_big  .clicked(cx, cy) && t) {
        if      (skill.get_number() == 0) skill.set_number( infty);
        else if (skill.get_number() < 10) skill.set_number(     0);
        else     skill.set_number(        skill.get_number() - 10);
    }
    else if (down_small.clicked(cx, cy) && t) {
        if (skill.get_number() == 0) skill.set_number(infty);
        else                         skill.set_number(skill.get_number() - 1);
    }
}

void SkillButtonWithButtons::draw()
{
    Element::draw();
    skill     .draw();
    up_big    .draw();
    up_small  .draw();
    down_big  .draw();
    down_small.draw();
}

void SkillButtonWithButtons::draw_self()
{
}

} // Ende Namensraum
