/*
 * ./editor/win_skill.h
 *
 * Fenster, in dem die Belegung der acht Faehigkeits-Tasten belegt weden.
 * Speziell fuer diese Klasse wird auch die Klasse SkillButton erweitert zu
 * SkillButtonWithButtons, diese Definition steht weiter unten in dieser Datei.
 *
 */

#pragma once

#include <vector>

#include "../api/window.h"
#include "../other/globals.h"
#include "../level/level.h"

namespace Api {

struct PossibleSkill;
struct SkillButtonWithButtons;

class WindowSkill : public Window {

private:

    static const int y_sbwb;
    static const int this_xl;
    static const int this_yl;

    Level* level;

    std::vector <PossibleSkill>          possible;
    std::vector <SkillButtonWithButtons> sbwb;

    typedef std::vector <PossibleSkill>          ::iterator PosIt;
    typedef std::vector <SkillButtonWithButtons> ::iterator SbwbIt;

    PossibleSkill*          cur_pos;
    SkillButtonWithButtons* cur_sbwb;

    TextButton clear;
    TextButton standard;
    TextButton all_to;
    Number     all_to_n;
    TextButton ok;
    TextButton cancel;

    // Kopierverbot
    WindowSkill(const WindowSkill&);
    void operator = (const WindowSkill&);

    // Helfer fuer die schnelle Positionierung der Possible-Buttons
    inline unsigned int px(unsigned int i) { return 20 + i%8 * 50; }
    inline unsigned int py(unsigned int i) { return 40 + i/8 * 50; }

    void swap_sbwbs(SkillButtonWithButtons*, SkillButtonWithButtons*);
    void assign_sbwb_pos(SkillButtonWithButtons*, PossibleSkill*);

public:

             WindowSkill(Level&);
    virtual ~WindowSkill();

    void calculate();

protected:

    virtual void draw_self();

};



// ############################################################################
// ############################################################################
// ############################################################################



struct PossibleSkill : public Button {

    static const unsigned this_length;

    Graphic     lem;
    Lem::Ac ac;

    PossibleSkill (Torbit&, const int = 0, const int = 0,
                   const Lem::Ac    = Lem::NOTHING,
                   const Lem::Style = Lem::GARDEN);
    virtual ~PossibleSkill();

    bool get_color();
    void set_color(const bool b);

protected:

    virtual void draw_self();

};



struct SkillButtonWithButtons : public Element {

    SkillButton  skill;

    BitmapButton   up_big;
    BitmapButton   up_small;
    BitmapButton down_big;
    BitmapButton down_small;

             SkillButtonWithButtons(Torbit&, const unsigned, const unsigned,
                                    const Lem::Ac = Lem::NOTHING,
                                    const int = 0);
    virtual ~SkillButtonWithButtons();

            void calculate(const int cx, const int cy);
    virtual void draw();

protected:

    virtual void draw_self();

}; // Ende Klasse
}  // Ende Namensraum
