/*
 * varwin.cpp
 *
 */

#include "win_var.h"

#include "../other/language.h"

namespace Api{

const int WindowVariables::this_xl(440);
const int WindowVariables::this_yl(260);

WindowVariables::WindowVariables(Level& l)
:
    Window(*osd, 100, 100, this_xl, this_yl, Language::win_var_title),
    author      (get_torbit(), 120,  40, 300,   l.author),
    name_german (get_torbit(), 120,  70, 300,   l.name_german),
    name_english(get_torbit(), 120, 100, 300,   l.name_english),

    lemmings    (get_torbit(), 140, 130, 140, 2, 1, 99,    l.lemmings),
    required    (get_torbit(), 140, 160, 140, 2, 1, 99,    l.required),
    rate        (get_torbit(), 140, 190, 140, 2, 1, 99,    l.rate),
    seconds     (get_torbit(), 120, 220, 180, 5, 0, 30*60, l.seconds,  true),

    ok          (get_torbit(), 320, this_yl - 70, 100, 20),
    cancel      (get_torbit(), 320, this_yl - 40, 100, 20)
{
    level = &l;

    ok     .set_hotkey(KEY_ENTER);
    cancel .set_hotkey(KEY_ESC);
    ok     .set_text  (Language::ok);
    cancel .set_text  (Language::cancel);

    seconds.set_format_time();
    seconds.set_step_big(60);
    seconds.set_step_med(10);
    seconds.set_step_sml( 1);

    // Beschriftungen vor den Texteingabezeilen
    draw_text(Language::win_var_author,       20, author      .get_y());
    draw_text(Language::win_var_name_german,  20, name_german .get_y());
    draw_text(Language::win_var_name_english, 20, name_english.get_y());

    draw_text(Language::win_var_lemmings,     20, lemmings.get_y());
    draw_text(Language::win_var_required,     20, required.get_y());
    draw_text(Language::win_var_rate,         20, rate    .get_y());
    draw_text(Language::win_var_clock,        20, seconds .get_y());
}

WindowVariables::~WindowVariables()
{
}



void WindowVariables::calculate()
{
    // Ist bereits eine Zeile im Schreibmodus?
    if (Texttype::get_selection()) Texttype::get_selection()->calculate();
    // Nichts im Schreibmodus? Alles anklickbar
    else {
        // Alles prueft sich selber auf Anklicken uns so weiter
        author      .calculate(get_x(), get_y());
        name_german .calculate(get_x(), get_y());
        name_english.calculate(get_x(), get_y());

        lemmings    .calculate(get_x(), get_y());
        required    .calculate(get_x(), get_y());
        rate        .calculate(get_x(), get_y());
        seconds     .calculate(get_x(), get_y());

        // Ausstieg ohne Speichern und mit Speichern
        if      (button_clicked(cancel)) set_exit();
        else if (button_clicked(ok) || hardware.get_mr()) {
            // Gegen bloede Werte
            if (required.get_number() > lemmings.get_number()) {
                required.set_number(lemmings.get_number());
            }
            level->author       = author      .get_text();
            level->name_german  = name_german .get_text();
            level->name_english = name_english.get_text();

            level->lemmings     = lemmings    .get_number();
            level->required     = required    .get_number();
            level->rate         = rate        .get_number();
            level->seconds      = seconds     .get_number();
            set_exit();
        }
        // Ende vom Ausstieg
    }
    // Ende von "Alles anklickbar"
}















void WindowVariables::draw_self()
{
    author      .draw();
    name_german .draw();
    name_english.draw();

    lemmings    .draw();
    required    .draw();
    rate        .draw();
    seconds     .draw();

    ok          .draw();
    cancel      .draw();
}









} // Ende Namensraum
