/*
 * gameplay/gamepl_c.cpp
 *
 */

#include "gameplay.h"

void Gameplay::calculate()
{
    // Wenn nicht gestartet, macht dies nix
    Network::calculate();

    if (window_gameplay) {
        calculate_window();
        // This is a bit kludgy, but it prevents opening the window
        // immediately again during a network game on an ESC press
        if (window_gameplay && Network::get_started()
         && !window_gameplay->get_game_end()) calculate_self();
    }
    else calculate_self();
}



void Gameplay::calculate_window()
{
    mouse_cursor.set_x_frame(0);
    mouse_cursor.set_y_frame(0);

    window_gameplay->calculate();

    Api::WindowGameplay::ExitWith exit_with = window_gameplay->get_exit_with();
    if (exit_with != Api::WindowGameplay::NOTHING) {
        switch (exit_with)
        {
        case Api::WindowGameplay::RESUME:
            break;

        case Api::WindowGameplay::MENU:
            save_result();
            exit = true;
            break;

        case Api::WindowGameplay::RESTART:
            save_result();
            load_state(state_manager.load_zero());
            panel[PAUSE]      .set_off();
            panel[SPEED_SLOW] .set_off();
            panel[SPEED_FAST] .set_off();
            panel[SPEED_TURBO].set_off();
            break;

        default:
            break;
        }
        delete window_gameplay;
        window_gameplay = 0;
    }
}



// ############################################################################
// ############################################################################
// ############################################################################



void Gameplay::calculate_self()
{
    // Berechnungen, die in jedem Tick, nicht nur in jedem Update
    // ausgefhrt werden mssen
    ++local_ticks;

    // Aendert die Mauskoordinaten ggf., also frueh aufrufen!
    map.calculate_scrolling();

    mouse_cursor.set_x_frame(0);
    mouse_cursor.set_y_frame(0);

    if (malo && malo->aiming == 2) {
        mouse_cursor.set_x_frame(1);
        mouse_cursor.set_y_frame(2);
    }
    else if (Globals::user.scroll_right && hardware.get_mrh()
     && map.get_scrollable()) mouse_cursor.set_x_frame(3);



    // ec kurz fuer exit_count
    unsigned ec = effect.get_effects();
    for (Tribe::CIt t = cs.tribes.begin(); t != cs.tribes.end(); ++t)
     ec += t->lemmings_hatch + t->get_lemmings_out();
    for (IacIt i = cs.trap.begin(); i != cs.trap.end(); ++i)
     ec += i->get_x_frame();
    // Check also for !window_gameplay, as in a networked game,
    // the action keeps running in the background
    if (ec == 0 && !window_gameplay) {
        console_tt.set_off();
        if (cs.tribes.size() == 1) {
            // Ergebnisfenster anzeigen
            window_gameplay = new Api::WindowGameplay(&replay,
             trlo->lemmings_saved, level.required, level.lemmings,
             trlo->skills_used,
             // Augenzucker bei null Geretteten, wird ohnehin nicht gespeichert
             (trlo->lemmings_saved > 0 ? update_last_exiter : cs.update),
              level.get_name());
        }
        else {
            window_gameplay = new Api::WindowGameplay(&replay, cs.tribes, trlo,
             cs.tribes.size(), level.get_name(), &level);
        }
        return;
    }
    // Beachte das return im vorhergehenden if.



    // Wenn die Konsoleneingabe aktiv ist, werden die zahlreichen Panels
    // nicht abgearbeitet! Dazu muss erst die Konsoleneingabe beendet werden.
    if (console_tt.get_on()) {
        if (!console_tt.get_text().empty() && hardware.key_enter_once()) {
            Network::send_chat_message(console_tt.get_text());
            console_tt.set_text(Globals::empty_string);
        }
        else if (hardware.key_once(KEY_ESC)) {
            // Ganz loeschen, nicht nur zum vorher weggeklickten Text
            console_tt.set_text(Globals::empty_string);
            console_tt.set_off();
        }
        console_tt.calculate();
    }

    // Konsole deaktiviert, normale Buttons ansehen und,
    // wenn kein Replay stattfindet, auch den aktiven Main-Loop abarbeiten
    else {
        // Speichern
        if (!Network::get_started() && panel[STATE_SAVE].clicked()) {
            state_manager.save_user(cs);
        }

        // Laden
        if (!Network::get_started() && panel[STATE_LOAD].clicked()) {
            load_state(state_manager.load_user());
        }

        // Pause
        if (!Network::get_started() && (panel[PAUSE].clicked())) {
            panel[PAUSE].set_on(!panel[PAUSE].get_on());
        }
        // Zoom
        if (panel[ZOOM].clicked()) {
            panel[ZOOM].set_on(!panel[ZOOM].get_on());
            map.set_zoom(panel[ZOOM].get_on());
        }
        // Geschwindigkeit
        for (unsigned i = SPEED_SLOW; i <= SPEED_TURBO; ++i)
         if (!Network::get_started() && panel[i].clicked()) {
            bool b = panel[i] .get_on();
            panel[SPEED_SLOW] .set_off();
            panel[SPEED_FAST] .set_off();
            panel[SPEED_TURBO].set_off();
            panel[i]          .set_on(!b);
        }
        // Neustart
        if (!Network::get_started() && panel[RESTART].clicked()) {
            load_state(state_manager.load_zero());
        }

        // Konsoleneingabe aktivieren
        if (Network::get_started()
         && hardware.key_once(Globals::user.key_chat)) {
            console_tt.set_on();
        }

        // Beenden bzw. Menue?
        if (hardware.key_once(KEY_ESC) && !window_gameplay) {
            window_gameplay = new Api::WindowGameplay(&replay,
                               Network::get_started() ? &level : 0);
            return;
        }

    }
    // Ende von deaktivierter Konsole





    // Replay oder normales Spiel: Hauptsaechlichkeiten abarbeiten
    // Wenn ein Geschwindigkeitsbutton angeklickt oder per Hotkey aktiviert
    // wurde, dann ueberspringen wir den Replay-Abbruch doch noch.
    if (replaying && (hardware.get_ml()
     || cs.update >= replay.get_max_updates()
     || replay.get_max_updates() == 0)) {
        bool b = true; // Replay abbrechen?
        if (hardware.get_ml() &&
           (panel[STATE_SAVE] .is_mouse_here()
         || panel[STATE_LOAD] .is_mouse_here()
         || panel[PAUSE]      .is_mouse_here()
         || panel[ZOOM]       .is_mouse_here()
         || panel[SPEED_SLOW] .is_mouse_here()
         || panel[SPEED_FAST] .is_mouse_here()
         || panel[SPEED_TURBO].is_mouse_here()
         || panel[RESTART]    .is_mouse_here())) b = false;
        if (b) {
            replaying = false;
            replay.erase_data_after_update(cs.update);
            panel[SPEED_FAST] .set_off();
            panel[SPEED_TURBO].set_off();
        }
    }

    // Grafikeffekte muessen noch vor dem Update berechnet werden,
    // aber geschwindigkeitsmaessig unabhaengig von den cs.update
    // Trotzdem "cs.update" uebergeben, damit zu alte Effektspeicherungen
    // wieder geloescht werden koennen.
    effect.calculate(cs.update);





    // Steuerung der Lemminge
    if (!replaying && !console_tt.get_on()) calculate_active();

    // Jetzt die Schleife fr ein Update, also eine Gameplay-Zeiteinheit
    // Dies prueft nicht die lokalen Ticks, sondern richtet sich nur nach
    // den vom Timer gezaehlten Ticks!
    if (!Network::get_started()) {
        if (!panel[PAUSE].get_on()) {
            if (panel[SPEED_TURBO].get_on()) {
                if (Help::timer_ticks >= timer_tick_last_update
                                       + timer_ticks_for_update_fast)
                 for (unsigned i = 0; i < turbo_times_faster_than_fast; ++i) {
                    update();
                }
            }
            else if (panel[SPEED_FAST].get_on()) {
                if (Help::timer_ticks >= timer_tick_last_update
                                       + timer_ticks_for_update_fast) {
                    update();
                }
            }
            else if (panel[SPEED_SLOW].get_on()) {
                if (Help::timer_ticks >= timer_tick_last_update
                                       + timer_ticks_for_update_slow) {
                    update();
                }
            }
            else if (Help::timer_ticks >= timer_tick_last_update
                                        + timer_ticks_for_update_normal) {
                update();
            }
        }
    }
    // Im Netzwerk die Zeit anhand der Server-Informationen synchronisieren
    else {
        unsigned u = Network::get_updates();
        if (u != 0) {
            u += Network::get_ping_millis()
               * Help::timer_ticks_per_second / 1000 / 2 // /2: ping=Roundtrip
               / timer_ticks_for_update_normal;          // u = Ziel-Update
            unsigned t = timer_ticks_for_update_normal;
            if      (u > cs.update + 10) t -= 2;
            else if (u > cs.update     ) t -= 1;
            else if (u < cs.update - 10) t += 1;
            else if (u < cs.update     ) t += 2;
            timer_ticks_for_update_client = t;
        }
        // In jedem Fall das Update normal rechnen, wie oben ohne Netz
        if (Help::timer_ticks >= timer_tick_last_update
                               + timer_ticks_for_update_client) update();
    }

}
// Ende von main_loop_calculate()



void Gameplay::load_state(const State& state)
{
    if (state) {
        cs = state;
        // Replay nicht laden, sondern das derzeitige ab dort abspielen
        replaying = cs.update < replay.get_max_updates();
        // Panel aktualisieren
        rate_displayed  =  trlo->rate;
        panel[NUKE].set_on(trlo->nuke);
        if (trlo) for (unsigned int a = 0; a < skill.size(); ++a) {
            skill[a].set_off();
            skill[a].set_number(trlo->skill[a].nr);
        }
        if (malo && skill[malo->skill_sel].get_number() > 0) {
            skill[malo->skill_sel].set_on();
        }
        // Klappen bei Bedarf schliessen
        if (cs.update <= updates_hatch_open)
         for (IacIt itr =  special[Object::HATCH].begin();
                    itr != special[Object::HATCH].end(); ++itr)
         itr->set_x_frame(0);
        // Sauberputzen
        effect.delete_after(cs.update);
    }
}
