/*
 * gameplay/gamepl_d.cpp
 *
 * Entzeichnen und Zeichnen inklusive Subfunktionen
 *
 */

#include <cmath> // std::sin fuer Replay-Zeichen

#include "gameplay.h"

void Gameplay::undraw() {
    mouse_cursor.undraw();

    if (window_gameplay) window_gameplay->undraw();

    console_tt.undraw();
    console_bg.undraw();

    // Rest vom OSD
    replay_sign .undraw();
    for (unsigned i = 0; i < skill.size(); ++i) skill[i].undraw();
    for (unsigned i = 0; i < panel.size(); ++i) panel[i].undraw();
}



// ############################################################################
// ############################################################################
// ############################################################################



void Gameplay::draw() {
    map.clear_screen_rectangle();

    // Zuerst die interaktiven Objekte. Wir zeichnen Listen mit allen Indizies,
    // auch wenn die Listen von TERRAIN oder GOAL natuerlich immer leer sind.
    // Debugging: We draw gloals below deco if we're in singleplayer, and
    // vice-versa in multiplayer. This is because we want the ownership
    // to show on top of the decoration, but Frapela's windmill wings (deco)
    // top of the windmill goals. Should probably be done better, with the
    // ownerships as decoration as well (topmost of all decos) maybe?
    if (cs.tribes.size() == 1)
     for (Goal::It i = goal.begin(); i != goal.end(); ++i) i->draw(false);
    for (int type = Object::TERRAIN; type != Object::MAX; ++type)
     for (IacIt i =  special[Object::perm(type)].begin();
                i != special[Object::perm(type)].end(); ++i) i->draw();
    if (cs.tribes.size() > 1)
     for (Goal::It i = goal.begin(); i != goal.end();     ++i) i->draw(true);
    for (IacIt i = cs.trap.begin();  i != cs.trap.end();  ++i) i->draw();
    for (IacIt i = cs.fling.begin(); i != cs.fling.end(); ++i) i->draw();

    map.load_masked_screen_rectangle(cs.land);

    effect.draw();

    // Erst die entfernten Spieler zeichnen, dann den lokalen.
    // Erst die Unmarkierten zeichnen, dann die Arbeitstiere
    // Markierungen bleiben erhalten bis zum naechsten Update.
    for (Tribe::It t = --cs.tribes.end(); t != --cs.tribes.begin(); --t) {
        if (&*t == trlo) continue;
        for (LemIt i = --t->lemming.end(); i != --t->lemming.begin(); --i)
         if (!i->get_mark()) i->draw();
        for (LemIt i = --t->lemming.end(); i != --t->lemming.begin(); --i)
         if ( i->get_mark()) i->draw();
    }
    for (LemIt i = --trlo->lemming.end(); i != --trlo->lemming.begin(); --i)
     if (!i->get_mark()) i->draw();
    for (LemIt i = --trlo->lemming.end(); i != --trlo->lemming.begin(); --i)
     if ( i->get_mark()) i->draw();



    // OSD: Replay-Zeichen
    if (replaying) {
        replay_sign.set_y((int) (5 + 5 * -std::sin(local_ticks * 0.07)));
        replay_sign.draw();
    }
    // Alle Faehigkeiten
    position_panels();
    for (unsigned int i = 0; i < skill.size(); ++i) skill[i].draw();

    // Hilfsbuttons nicht nach der Spielerzahl, sonder nach der Netzwerk-
    // aktivitaet zeichnen, da man im Replay volle Kontrolle haben soll
    if (!Network::get_started()) {
        // Plus und Minus werden zwar gezeichnet, wenn ein Mehrspielerspiel
        // abgespielt wird, sind aber nicht anklickbar in gamepl_a.cpp.
        for (unsigned int i = 0; i < panel.size(); ++i)
         panel[i].draw();
        // Wir machen zweimal string_plus_int, damit die fuehrende Null erscheint.
        std::string str;
        (str += trlo->rate_min   / 10) += trlo->rate_min   % 10;
        Help::draw_shadow_centered_text(*osd, font_med, str.c_str(),
         panel[RATE_MINUS].get_x() + 27, panel[RATE_MINUS].get_y() + 5);
        str.clear();
        (str += rate_displayed / 10) += rate_displayed % 10;
        Help::draw_shadow_centered_text(*osd, font_med, str.c_str(),
         panel[RATE_PLUS] .get_x() + 27, panel[RATE_PLUS] .get_y() + 5);
    }
    // In laufenden Netzwerkpartien nur diese beiden Buttons zeichnen
    else {
        panel[ZOOM].draw();
        panel[NUKE].draw();
    }



    // Statusanzeige
    if (cs.tribes.size() > 1) {
        int lemmings_saved_leader = 0;
        for (Tribe::CIt itr = cs.tribes.begin(); itr != cs.tribes.end();++itr)
         if (itr->lemmings_saved > lemmings_saved_leader)
         lemmings_saved_leader = itr->lemmings_saved;
        stats.set_required(lemmings_saved_leader);
    }
    if (cs.clock > 0) stats.set_show_clock();
    stats.set_clock(cs.clock);
    stats.draw();



    // Konsole
    if (Network::get_started()) {
        unsigned console_count = 0;
        const std::list <Console::Line>& li = Console::get_lines_recent();
        for (Console::LiIt i = li.begin(); i != li.end(); ++i)
         Help::draw_shadow_text(*osd, font_sml, i->text.c_str(), 2,
         12*console_count++ - 1, i->white ? color[COL_TEXT_ON]
         : color[COL_TEXT], color[COL_API_SHADOW]);
        console_bg.set_yl(12 * console_count+console_tt.get_yl());
        console_tt.set_y (12 * console_count - 2);
        console_bg.draw(); // Dies macht nichts ausser undraw() zu ermoeglichen
        if (console_tt.get_on()) {
            // Diese Abfrage ermoeglicht es, zwischendurch zu klicken und doch
            // den Chat-Text nicht zu verlieren
            console_tt.draw();
            std::string s = Globals::user.get_name() + ":";
            Help::draw_shadow_text(*osd, font_med, s.c_str(),
             1, console_tt.get_y(), color[COL_TEXT], color[COL_API_SHADOW]);
        }
    }



    // Fenster
    if (window_gameplay) window_gameplay->draw();

    // Mauszeiger ganz obenauf
    mouse_cursor.set_x(hardware.get_mx()-mouse_cursor_offset);
    mouse_cursor.set_y(hardware.get_my()-mouse_cursor_offset);
    mouse_cursor.draw();

    map. draw(*pre_screen, false);
    osd->draw(*pre_screen);

    blit_to_screen(pre_screen->get_al_bitmap());
}
// Ende draw()



// ############################################################################
// ############################################################################
// ############################################################################



void Gameplay::position_panels()
{
    if (!Network::get_started()) {
        for (unsigned i = 0; i < MAX; ++i) {
            if      (i  < PAUSE) panel[i].set_x(i/2 * 40);
            else if (i == PAUSE) panel[i].set_x(360);
            else                 panel[i].set_x((i+1)/2 * 40 + 320);
        }
        for (unsigned i = 0; i < skill.size(); ++i) skill[i].set_x((i+1) * 40);
        stats.set_wide(false);
    }
    else {
        panel[ZOOM].set_x(8 * 40);
        panel[NUKE].set_x(8 * 40);
        for (unsigned i = 0; i < skill.size(); ++i) skill[i].set_x(i * 40);
        stats.set_wide(true);
    }
}
// Ende position_panels()
