/*
 * gameplay/gameplay.h
 *
 * Gameplay ist die Klasse, die das eigentliche Spiel beschreibt. Es werden
 * einige grosse Codestuecke in separate CPP-Dateien ausgelagert.
 *
 */

#pragma once

#include "effect.h"
#include "goal.h"
#include "panel.h"
#include "replay.h"
#include "state_ma.h"
#include "win_game.h"

#include "../editor/graph_ed.h"

#include "../graphic/graph_bg.h"
#include "../graphic/map.h"
#include "../lemming/ac.h"
#include "../level/level.h"
#include "../network/network.h"
#include "../other/console.h"

class Gameplay {

public:

    static const int block_s; // Blocker-Abstand zur Seite
    static const int block_m; // zur Seite, wenn der andere ein Miner ist
    static const int block_u; // nach oben
    static const int block_d; // nach unten

    static const int updates_per_second;

private:

    static const unsigned turbo_times_faster_than_fast;
    static const unsigned timer_ticks_for_update_fast;
    static const unsigned timer_ticks_for_update_normal;
    static const unsigned timer_ticks_for_update_slow;

    static const int mouse_cursor_offset;
    static const int mouse_max_lemming_distance_x;
    static const int mouse_max_lemming_distance_u;
    static const int mouse_max_lemming_distance_d;
    static const int distance_safe_fall;
    static const int distance_float;

    static const unsigned updates_lets_go;
    static const unsigned updates_hatch_open;

    bool              exit;
    const std::string filename;
    const Level       level;

    PlNr           player_local;
    Tribe*         trlo; // Abkuerzung fuer cs.tribes[tribe_local]
    Tribe::Master* malo; // The master of cs.tribes[tribe_local] that
                          // has player_local as its player number

    unsigned long local_ticks;
    unsigned long update_last_exiter; // fuer Einspieler-Rekord
    unsigned long timer_tick_last_update;
    unsigned long timer_tick_last_F1;
    unsigned long timer_tick_last_F2;
    unsigned long timer_tick_last_F12;
    int           timer_ticks_for_update_client;

    Map    map;
    Torbit steel_mask;

    State         cs; // current state
    StateManager  state_manager;
    EffectManager effect;

    GraphicBG mouse_cursor;
    GraphicBG replay_sign;

    std::vector <Api::BitmapButton> panel;
    std::vector <Api::SkillButton>  skill;
    Api::Element       console_bg; // Um OSD nur teilweise zu schwaerzen
    Api::Texttype      console_tt; // Texttype, das Eingabefeld der Konsole
    Api::GameplayPanel stats;

    // Alles fuers Replay
    bool              replaying;
    Replay            replay;
    bool              replay_recalculate_need;
    unsigned long     replay_recalculate_from;
    Api::SkillButton* old_selected_skill;

    int               rate_displayed;

    Api::WindowGameplay* window_gameplay;

    enum PanNr {
        RATE_MINUS,
        RATE_PLUS,
        PAUSE,
        ZOOM,
        SPEED_SLOW,
        SPEED_FAST,
        SPEED_TURBO,
        STATE_SAVE,
        STATE_LOAD,
        RESTART,
        NUKE,
        MAX
    };

    typedef std::vector <Lemming>   ::iterator         LemIt;

    std::vector <std::vector <EdGraphic> >             special;
    std::vector         <Goal>                         goal;
    typedef std::vector <EdGraphic> ::iterator         IacIt;
    typedef std::vector <EdGraphic> ::reverse_iterator IacRIt;

    ////////////////
    // Funktionen //
    ////////////////

    Replay::Data new_replay_data(); // Replay::Data() mit guten Standardinfos

    std::string  determine_filename(Replay* = 0);
    void         prepare_players   (Replay*);
    void         prepare_level     ();
    void         save_result       ();

    // Calculate-Subroutinen
    void calculate_window(); // Fuers Unterfenster, ansonsten:
    void calculate_self  (); // Wird sonst in jedem Tick ausgefhrt
    void calculate_active(); // "Aktiv" == kein Replay abzuspielen
    void load_state      (const State&);

    // Update-Subroutinen
    void update();               // Normalerweise 15x pro Sek., ruft auf:
    void update_cs_once          ();
    void update_cs_one_data      (Tribe&, Tribe::Master*, Replay::It);
    void update_lemming          (Lemming&, const UpdateArgs&);
    void update_lemming_blocker  (Lemming&);
    void update_lemming_goals    (Lemming&, const UpdateArgs&);
    void make_knockback_explosion(const unsigned long, const Tribe&,
                                  const unsigned,      const int, const int);
    // Draw-Subroutinen
    void position_panels();  // 1x pro draw(), ist wichtig bei Serverabbruch

    // Kopierverbot
    Gameplay        (const Gameplay&);
    void operator = (const Gameplay&);

public:

    // Der Konstruktor klaut sich seine paar Infos zusammen
    // aus Network und Globals - Spielerzahl, -daten und den Level.
    // Einzig ein Replay kann optional uebergeben werden.
    Gameplay(Replay* = 0);
    ~Gameplay();

    inline const bool         get_exit    () { return exit;             }
    inline const std::string& get_filename() { return filename;         }
    inline Replay&            get_replay  () { return replay;           }
    inline int                get_tribes  () { return cs.tribes.size(); }

    void undraw();
    void calculate();
    void draw();

};
