/*
 * gameplay/panel.cpp
 *
 */

#include "panel.h"
#include "gameplay.h" // Konstanten, wie viele Updates pro Sekunde

#include "../graphic/gra_lib.h"

namespace Api {

const int GameplayPanel::PanelTribe::frame_walker = Lem::WALKER  - 1;
const int GameplayPanel::PanelTribe::frame_exiter = Lem::EXITER  - 1;
const int GameplayPanel::PanelTribe::frame_clock  = Lem::DROWNER - 1;
const int GameplayPanel::PanelTribe::frame_grey   = 15;
const int GameplayPanel::PanelTribe::frame_normal = 16;
const int GameplayPanel::PanelTribe::frame_green  = 17;

const int GameplayPanel::PanelTribe::name_short_width_sum = 180;
const int GameplayPanel::PanelTribe::tarinf_max_length    =  75;



GameplayPanel::PanelTribe::PanelTribe(Torbit& b)
:
    ground  (&b),
    tr      (0),
    white   (false)
{
}



GameplayPanel::PanelTribe::~PanelTribe()
{
}



// Breite: 80 Pixel
void GameplayPanel::PanelTribe::draw_big(
    const int x,
    const int y,
    const int initial,
    const int required,
    const int*     clock,
    const Lemming* tarinf,
    const int      tarcnt
) {
    if (!tr) return;
    const int  in    = tr->lemmings_saved;
    const int  out   = tr->get_lemmings_out();
    const bool green = required && in >= required;
    GraLib::get_lemming(tr->style).draw(*ground, x - 37, y,
     out == 0 ? frame_grey :                       frame_normal, frame_walker);
    GraLib::get_lemming(tr->style).draw(*ground, x - 37, y + 20,
     in  == 0 ? frame_grey : green ? frame_green : frame_normal, frame_exiter);
    draw_nr_big(out, initial,  x - 15, y +  0);
    draw_nr_big(in,  required, x - 15, y + 20, green);

    // Tarinf
    if (tarinf) {
        std::string cnt;
        std::string str;
        std::string ab;
        cnt += tarcnt;
        cnt += ' ';
        str += Lem::ac_to_string(tarinf->get_ac());
        Help::string_to_nice_case(str);
        if (tarcnt > 1 && Language::get() != Language::GERMAN) str += 's';
        str = cnt + str;
        if (tarinf->get_climber() || tarinf->get_floater()) {
            ab += " (";
            if (tarinf->get_climber()) ab += 'C';
            if (tarinf->get_floater()) ab += 'F';
            ab += ')';
        }
        while (text_length(font_med, (str + ab).c_str()) > tarinf_max_length
         && str.size() > 1) {
            str.resize(str.size() - 2);
            str += '.';
        }
        Help::draw_shadow_text(*ground, font_med,
         (str + ab).c_str(), x - 37, y + 40);
    }
    // Uhr, falls nicht Tarinf
    else if (clock) {
        GraLib::get_lemming(tr->style)
         .draw(*ground, x-37, y+40, frame_grey, frame_clock);
        std::string str;
        const int  c = *clock + Gameplay::updates_per_second - 1;
        const int& u = Gameplay::updates_per_second;
        str += c / (u * 60);
        str += ':';
        str += c % (u * 60) / (u * 10);
        str += c % (u * 10) / (u     );
        Help::draw_shadow_centered_text(*ground, font_med, str.c_str(), x + 10,
         y + 40, color[COL_TEXT]);
    }



}



// Breite: 50 Pixel
void GameplayPanel::PanelTribe::draw_med
 (const int x, const int y, const int required)
{
    if (!tr) return;
    const int  in    = tr->lemmings_saved;
    const int  out   = tr->get_lemmings_out();
    const bool green = required && in >= required;
    GraLib::get_lemming(tr->style).draw(*ground, x - 22, y,
     out == 0 ? frame_grey :                       frame_normal, frame_walker);
    GraLib::get_lemming(tr->style).draw(*ground, x - 22, y + 20,
     in  == 0 ? frame_grey : green ? frame_green : frame_normal, frame_exiter);
    draw_nr_sml(out, x, y +  0);
    draw_nr_sml(in,  x, y + 20, green);
    Help::draw_shadow_centered_text(*ground, font_med,
     name_short.c_str(), x - 2, y + 40); // - 2 sieht iwie besser aus als 0
}



// Breite: 20 Pixel
void GameplayPanel::PanelTribe::draw_sml
 (const int x, const int y, const int required)
{
    if (!tr) return;
    const int  in    = tr->lemmings_saved;
    const int  out   = tr->get_lemmings_out();
    // y + 38, weil die oberen zwei Pixel ohnehin leer sind, damit bei der
    // ueblichen Anordnung (Lemming-Symbol in der ersten Zeile) nicht in den
    // Button-Rand gezeichnet wird.
    GraLib::get_lemming(tr->style).draw(*ground, x - 10, y + 38,
     out == 0 ? frame_grey : frame_normal, frame_walker);
    draw_nr_sml(out, x - 10, y);
    draw_nr_sml(in,  x - 10, y + 20, required && in >= required);
}



void GameplayPanel::PanelTribe::draw_nr_big(const int first,
 const int second, const int x, const int y, const bool green)
{
    std::string s;
    std::string slash  = "/";
    const int col_zero =         color[COL_TEXT];
    const int col_full = green ? color[COL_TEXT_GREEN]
                       : white ? color[COL_WHITE]      : col_zero;
    s += first;
    Help::draw_shadow_centered_text(*ground, font_med,
     s.c_str(), x + 10, y, first > 0 ? col_full : col_zero);
    Help::draw_shadow_fixed_text(*ground, font_med, slash, x + 20, y,
     green ? col_full : col_zero);
    s.clear();
    s += second;
    Help::draw_shadow_centered_text(*ground, font_med,
     s.c_str(), x + 40, y, green ? col_full : col_zero);
}



void GameplayPanel::PanelTribe::draw_nr_sml(const int first,
 const int x, const int y, const bool green)
{
    std::string s;
    const int col_zero =         color[COL_TEXT];
    const int col_full = green ? color[COL_TEXT_GREEN]
                       : white ? color[COL_WHITE]      : col_zero;
    s += first;
    Help::draw_shadow_centered_text(*ground, font_med,
     s.c_str(), x + 10, y, first > 0 ? col_full : col_zero);
}




// ############################################################################
// ############################################################################
// ############################################################################



GameplayPanel::GameplayPanel(Torbit& b)
:
    Button(b, 0, LEMSCR_Y - 60, 0, 60),
    initial   (0),
    required  (0),
    wide      (false),
    show_clock(false),
    tarcnt    (0),
    tarinf    (0),
    clock     (0)
{
}

GameplayPanel::~GameplayPanel()
{
}



void GameplayPanel::add_tribe(const Tribe& t)
{
    tribes.push_back(PanelTribe(get_ground()));
    tribes.back().tr         = &t;
    tribes.back().name_short = t.get_name();

    // Shorten names whenever someone new is added
    if (tribes.size() > 1) // Against division by 0
     for (int i = 0; i < (int) tribes.size(); ++i) {
        std::string& str = tribes[i].name_short;
        while (text_length(font_med, str.c_str())
         > PanelTribe::name_short_width_sum / (int) (tribes.size() - 1)
         && str.size() > 1) {
            str.resize(str.size() - 2);
            str += '.';
        }
    }
    set_draw_required();
}



void GameplayPanel::set_tribe_local(const Tribe& t)
{
    for (std::vector <PanelTribe> ::iterator
     itr = tribes.begin(); itr != tribes.end(); ++itr)
     if (itr->tr == &t) {
        itr->white = true;
        break;
    }
}



void GameplayPanel::draw_self()
{
    // Zunaechst alle alten Aufschriften loeschen: Das Setzen der X-Koordinate
    // und der Laenge setzt draw_required per Element-Basisfunktionen.
    set_x(LEMSCR_X - 40 * (wide ? 7 : 2));
    set_xl(LEMSCR_X - get_x());

    Button::draw_self();
    if (tribes.empty()) return;

    // Zunaechst den Spieler mit weisser Schrift nach links malen
    for (int nr = 0; nr < (int) tribes.size(); ++nr)
     if (tribes[nr].white) {
        tribes[nr].draw_big(get_x() + 40, get_y(), initial, required,
                            show_clock ? &clock : 0, tarinf, tarcnt);
        break;
    }

    if (wide) {
        // Das Restpanel machen wir ganz blatant um 5-10 Pixel kuerzer, um mehr
        // Abstand zwischen dem eigenen und den anderen Spielern zu haben!
        // Das Restpanel in 2 * (cs.tribes.size() - 1) Stuecke der X-Laenge nach
        // teilen und zwischen das 1. und 2., zwischen das 3. und 4., zwischen
        // das 5. und 6., ... Stueck jeweils das Zentrum einer PanelTribe-
        // -Zeichenausgabe legen. Die Zeichenfunktion betrachtet die ueber-
        // gebene X-Koordinate als Zentrum.
        const int    extra = 6;
        const double piece = (get_xl() - 80 - extra) / 2.0 / (tribes.size()-1);
        int          pos   = 0;

        for (int nr = 0; nr < (int) tribes.size(); ++nr) {
            if (!tribes[nr].tr || tribes[nr].white) continue;
            const int x = get_x() + 80 + extra + (2 * pos+1) * (int)piece;
            if (tribes.size() <= 5) tribes[nr].draw_med(x, get_y(), required);
            else                    tribes[nr].draw_sml(x, get_y(), required);
            ++pos;
        }
    }
}

}
// Ende Namensraum Api
