/*
 * gameplay/panel.h
 *
 * Das Panel am unteren Bildschirmrand, welches Informationen ueber die
 * Lemminge sowie geretteten Lemminge gibt und ggf. die Target Information
 * anzeigt.
 *
 * Im Achtspielermodus brauchen wir mindestens 160 Pixel in der Breite, das
 * ist so viel, wie vier normale Skill-Buttons nebeneinander lang sind.
 *
 * Ein GameplayPanel verwaltet mehrere Spielerdaten, also Objekte der Klasse
 * PanelTribe. Diese Objekte koennen auf verschieden Arten gezeichnet werden:
 *
 *  draw_big()        draw_med()        draw_sml()
 *
 * [Lm] 56 / 80        [Lm]  56            [Lm]
 * [Ex] 23 / 78        [Ex]  23             56
 * <-- frei -->        < frei >             23
 *
 *   80 Pixel          50 Pixel          20 Pixel, man verplane aber mehr
 *
 * Die der draw_*()-Funktion uebergebene X-Koordinate gibt das Zentrum der
 * PanelTribe-Zeichenausgabe an, nicht den linken Rand!
 *
 * Im Falle von draw_big() wird die untere Zahlenzeile gruen gezeichnet, wenn
 * mindestens so viele Lemminge gerettet worden sind wie benoetigt. In beiden
 * anderen Faellen haben die Spieler mit den meisten geretteten Lemmingen > 0
 * eine gruene untere Zahl.
 *
 * X- und Y-Koordinate sind hartkodiert, sie richten sich je nach der Breit-
 * heit des Panels, einstellbar ueber set_wide(bool). Gleiches gilt fuer
 * die Laenge.
 *
 */

#pragma once

#include "tribe.h"

#include "../api/button.h"
#include "../lemming/lem_enum.h" // Farben und Tarinf

namespace Api {

class GameplayPanel : public Api::Button {

private:

    struct PanelTribe {

        static const int frame_walker;
        static const int frame_exiter;
        static const int frame_clock;
        static const int frame_grey;
        static const int frame_normal;
        static const int frame_green;

        static const int name_short_width_sum;
        static const int tarinf_max_length;

        Torbit*      ground;
        const Tribe* tr;
        std::string  name_short;
        bool         white;

        PanelTribe(Torbit&);
        ~PanelTribe();

        // Die der draw_*()-Funktion uebergebene X-Koordinate gibt das Zentrum
        // der PanelTribe-Zeichenausgabe an, nicht den linken Rand!
        void draw_big(int, int, int = 0, int = 0, const int*     = 0, // clock
                                                  const Lemming* = 0, int = 0);
        void draw_med(int, int, int = 0);
        void draw_sml(int, int, int = 0);

    private:

        //               1    2    X,   Y,   gruen? (X = links)
        void draw_nr_big(int, int, int, int, const bool = false);
        void draw_nr_sml(int,      int, int, const bool = false);
    };
    // Ende Klasse PanelTribe

    std::vector <PanelTribe> tribes;
    int  initial;        // Wie viele Lemminge am Anfang hatte jeder
    int  required;       // Im Netzwerk ist das max(Gerett. eines Splr.)
    bool wide;           // Mehrere Spieler oder nur einer?
    bool show_clock;     // Wenn unbefristet, ist die Uhr unnoetig.

    int            tarcnt; // Gesamtzahl der Lemmings unterm Cursor
    const Lemming* tarinf;
    int            clock;

public:

    //            wohin
    GameplayPanel(Torbit&);
    ~GameplayPanel();

    void add_tribe(const Tribe&);

    inline void sdr()                                { set_draw_required();  }
           void set_tribe_local(const Tribe&);
    inline void set_initial    (const int u)         { initial    = u; sdr();}
    inline void set_required   (const int u)         { required   = u; sdr();}
    inline void set_wide       (const bool b = true) { wide       = b; sdr();}
    inline void set_show_clock (const bool b = true) { show_clock = b; sdr();}
    inline void set_tarinf     (const Lemming*    l) { tarinf     = l; sdr();}
    inline void set_tarcnt     (const int u)         { tarcnt     = u; sdr();}
    inline void set_clock      (const int u)         { clock      = u; sdr();}

protected:

    virtual void draw_self();

};
// Ende Klasse GameplayPanel

}
// Ende Namensraum Api
