/*
 * gameplay/tribe.h
 *
 * Informationen ueber einen Spieler: Eine Liste mit der eigenen
 * Lemminghorde, die ausgewaehlte Farbe, Lemminge in der Klappe, im Ziel usw.
 *
 * Die Klasse Gameplay verwaltet die korrekte Anzahl Spieler in entsprechend
 * vielen Player-Objekten. Eines dieser Objekte ist als lokaler Spieler
 * markiert, ihm kommen alle Klicks usw. zugute.
 *
 * char position
 *
 *   Nicht Spielernummer, sondern Startplatz. Wird im Netzwerk durch eine
 *   zufaellige Permutation ermittelt.
 *
 * bool active
 *
 *   Wenn dies falsch ist, so ist der Spieler nur ein Platzhalter im Spieler-
 *   Vektor, damit die Vektornummern der Spieler mit den Peer-Nummern von
 *   Network uebereinstimmen.
 *
 */

#pragma once

#include <list>

#include "../network/net_t.h" // net types: PlNr
#include "../lemming/lemming.h"
#include "../other/globals.h"

struct Tribe {

    struct Skill {
        Lem::Ac  ac;
        unsigned nr;
        inline Skill() : ac(Lem::NOTHING), nr(0) {}
    };

    struct Master {
        PlNr        number;
        std::string name;
        int         skill_sel;
        int         aiming; // 0 = no, 2 = yes, 1 = no, earlier this update
        int         aiming_lemming;
        inline Master()
         : number(0), skill_sel(0), aiming(0), aiming_lemming(0) {}
    };

    typedef std::vector <Tribe>  ::iterator       It;
    typedef std::vector <Tribe>  ::const_iterator CIt;
    typedef std::list   <Master> ::iterator       MIt;

    int         lemmings_hatch;
    int         lemmings_saved;
    int         rate_min;
    int         rate;
    bool        nuke;

    unsigned long update_hatch;
    int           hatch_next;

    Skill       skill[Globals::skill_max];
    int         skills_used;

    Lem::Style            style;
    std::vector <Lemming> lemming;
    std::list   <Master>  masters;

    Tribe();
    ~Tribe();

    void        return_skills(const Lem::Ac, const int);
    std::string get_name() const;

    int         get_lemmings_out() const;

};
