/*
 * gameplay/win_game.cpp
 *
 */

#include <algorithm>

#include "win_game.h"

#include "../other/globals.h"
#include "../level/level.h" // Minimalabmasse fuer Level fuer Fensterposition

namespace Api {

const unsigned WindowGameplay::this_xl_network_results(300);
const unsigned WindowGameplay::this_yl_network_results(170);
const unsigned WindowGameplay::this_xl_single_results(300);
const unsigned WindowGameplay::this_yl_single_results(300);
const unsigned WindowGameplay::this_xl_no_results(200);
const unsigned WindowGameplay::this_yl_no_results(170);
const unsigned WindowGameplay::this_yl_net_no_results(140);
const unsigned WindowGameplay::top_item  (40);
const unsigned WindowGameplay::y_saved   (70);
const unsigned WindowGameplay::y_comment(100);
const unsigned WindowGameplay::y_button (200);

WindowGameplay::WindowGameplay(Replay* rep, const Level* const lev)
:
    Window(*osd,
     Level::view_xl/2 - this_xl_no_results/2,
     Level::view_yl/2 - (lev ? this_yl_net_no_results : this_yl_no_results)/2,
     this_xl_no_results, lev ? this_yl_net_no_results : this_yl_no_results,
     Language::win_game_title),

    game_end(false),
    game_net(lev ? true : false),
    exit_with    (NOTHING),
    resume       (get_torbit(), 20, top_item     , this_xl_no_results-40),
    restart      (get_torbit(), 20, top_item + 30, this_xl_no_results-40),
    save_replay  (get_torbit(), 20, top_item + 60, this_xl_no_results-40),
    menu         (get_torbit(), 20, top_item + 90, this_xl_no_results-40),
    replay       (rep),
    level        (lev),
    browser_save (0)
{
    common_constructor();
    // Resume reagiert bedingt durch calculate() auf jede Taste, die keinem
    // anderen Button im Fenster zugewiesen ist.

    // No hotkey during a network game, ESC should resume!
    if (game_net) {
        save_replay.set_y(top_item + 30);
        menu       .set_y(top_item + 60);
        resume .set_hotkey(KEY_ESC);
        restart.set_hotkey();
        menu   .set_hotkey(KEY_F1);
    }
}



WindowGameplay::WindowGameplay(
    Replay* rep,
    int lemmings_saved,
    int lemmings_required,
    int lemmings_at_start,
    int skills_used,
    int updates_used,
    const std::string& tit)
:
    Window(*osd,
     Level::view_xl/2 - this_xl_single_results/2,
     Level::view_yl/2 - this_yl_single_results/2,
     this_xl_single_results, this_yl_single_results, tit),

    game_end(true),
    game_net(false),
    exit_with    (NOTHING),
    resume       (get_torbit(), 20, y_button     , this_xl_single_results-40),
    restart      (get_torbit(), 20, y_button     , this_xl_single_results-40),
    save_replay  (get_torbit(), 20, y_button + 30, this_xl_single_results-40),
    menu         (get_torbit(), 20, y_button + 60, this_xl_single_results-40),
    replay       (rep),
    level        (0),
    browser_save (0)
{
    int col_a = color[COL_TEXT];
    int col_b = color[COL_TEXT];
    if (lemmings_saved > 0)                  col_a = color[COL_TEXT_ON];
    if (lemmings_saved >= lemmings_required) col_b = color[COL_TEXT_ON];

    std::string c                                   = Language::win_game_comment_none;
    if      (lemmings_saved == lemmings_at_start) c = Language::win_game_comment_perfect;
    else if (lemmings_saved >  lemmings_required) c = Language::win_game_comment_more;
    else if (lemmings_saved == lemmings_required) c = Language::win_game_comment_exactly;
    else if (lemmings_saved >  0)                 c = Language::win_game_comment_less;

    std::string s1; s1 += lemmings_saved;
    std::string s2      = "/";
    std::string s3; s3 += lemmings_required;

    dct(font_med, Language::win_game_lemmings_saved, 0, top_item);
    dct(font_big, s1, -35, y_saved,   col_a);
    dct(font_big, s2,   0, y_saved,   col_b);
    dct(font_big, s3,  35, y_saved,   col_b);
    dct(font_med, c,    0, y_comment, col_b);

    // Anzahl Gerettete ist gemalt. Jetzt Zusatzinfos.
    draw_text(Language::win_game_result_skills, 20, y_comment + 40);
    draw_text(Language::win_game_result_time,   20, y_comment + 60);

    // true: Schreibe rechtsbuendig
    Help::draw_shadow_fixed_number(      get_torbit(), font_med,
     skills_used,  get_xl() - 20,  y_comment + 40, col_b, true);
    Help::draw_shadow_fixed_updates_used(get_torbit(), font_med,
     updates_used, get_xl() - 20,  y_comment + 60, col_b, true);

    common_constructor();
}



WindowGameplay::WindowGameplay(
    Replay*                    rep,
    const std::vector <Tribe>& pv,
    const Tribe*               pl,
    const unsigned             pls_act,
    const std::string&         tit,
    const Level* const         lev)
:
    Window(*osd,
     Level::view_xl/2 - this_xl_network_results/2,
     Level::view_yl/2 - this_yl_network_results/2 - pls_act * 10,
                      this_xl_network_results,
                      this_yl_network_results   + pls_act * 20, tit),
    game_end(true),
    game_net(true),
    exit_with    (NOTHING),
    resume       (get_torbit()),
    restart      (get_torbit()),
    save_replay  (get_torbit(), 20, get_yl() - 70, get_xl() - 40),
    menu         (get_torbit(), 20, get_yl() - 40, get_xl() - 40),
    replay       (rep),
    level        (lev),
    browser_save (0)
{
    // Ergebnisse auswerten, sortieren und Listenerstellung vorbereiten
    std::vector <SortablePlayer> sortvec;
    for (Tribe::CIt i = pv.begin(); i != pv.end(); ++i)
     sortvec.push_back(SortablePlayer(&*i == pl, i->get_name(),
                                                 i->lemmings_saved));
    std::sort(sortvec.begin(), sortvec.end());
    // Komische Abkuerzungen: o = own, s = score, p = place
    unsigned op = 0;
    unsigned os = 0;

    // Liste schreiben
    for (unsigned place = 0; place < sortvec.size(); ++place) {
        std::string str;
        for (unsigned shown = 0; shown <= place; ++shown)
         if (sortvec[place].score == sortvec[shown].score) {
            str += shown + 1;
            break;
        }
        str += ". ";
        str += sortvec[place].name;
        const int col = sortvec[place].pl ? color[COL_WHITE] : color[COL_TEXT];
        Help::draw_shadow_text(get_torbit(), font_med,
         str.c_str(), 60, 40 + place*20, col);
        Help::draw_shadow_fixed_number(get_torbit(), font_med,
         sortvec[place].score,  get_xl() - 60, 40 + place*20, col, true);
        // Dies nur fuer den Kommentar unten.
        if (sortvec[place].pl) {
            op = place;
            os = sortvec[place].score;
        }
    }

    // Kommentar schreiben
    std::string    c;
    const unsigned fs =    sortvec. begin() ->score;
    const unsigned ss = (++sortvec. begin())->score;
    const unsigned ls =    sortvec.rbegin() ->score;
    const unsigned lp =    sortvec.size()   - 1;

    if      (fs == 0)             c = Language::win_game_net_all_zero;
    else if (os == 0)             c = Language::win_game_net_zero;
    else if (fs == ls)            c = Language::win_game_net_all_tie;
    else if (op == 0 && os == ss) c = Language::win_game_net_first_tie;
    else if (op < lp && os == ls) c = Language::win_game_net_last_tie;
    else if (op == 0)             c = Language::win_game_net_first;
    else if (op == lp)            c = Language::win_game_net_last;
    else if (op == 1)             c = Language::win_game_net_second;
    else                          c = Language::win_game_net_middle;

    dct(font_med, c, 0, get_yl() - 70 - 40, color[COL_WHITE]);

    if (c == Language::win_game_net_first
     || c == Language::win_game_net_first_tie) {
        Sound::play_loud(Sound::AWARD_1);
    }
    else if (c == Language::win_game_net_second
     ||      c == Language::win_game_net_first_tie) {
        Sound::play_loud(Sound::AWARD_2);
    }
    else if (c == Language::win_game_net_middle
     ||      c == Language::win_game_net_all_tie) {
        Sound::play_loud(Sound::AWARD_3);
    }
    else Sound::play_loud(Sound::AWARD_4);

    common_constructor();
}



WindowGameplay::~WindowGameplay()
{
    if (browser_save) delete browser_save;
    undraw();
}

void WindowGameplay::common_constructor()
{
    resume     .set_text(Language::win_game_resume);
    restart    .set_text(Language::win_game_restart);
    save_replay.set_text(Language::win_game_save_replay);
    menu       .set_text(Language::win_game_menu);
    restart    .set_hotkey(KEY_F1);
    save_replay.set_hotkey(KEY_F2);
    menu       .set_hotkey(KEY_ESC); // also triggered by space
}

// draw centered text
void WindowGameplay::dct(FONT* f, std::string& s,
 int x, int y, int c)
{
    Help::draw_shadow_centered_text(get_torbit(), f, s.c_str(),
     get_xl()/2 + x, y, c, color[COL_API_SHADOW]);
}





void WindowGameplay::calculate()
{
    if (browser_save) {
        browser_save->calculate();
        if (browser_save->get_exit()) {
            if (browser_save->get_exit_with()) {
                replay->save_to_file(
                 browser_save->get_current_dir_and_file(), level);
                Globals::user.replay_last_dir = browser_save->get_current_dir();
                if (game_end) exit_with = MENU;
                else          exit_with = RESUME;
            }
            delete browser_save;
            browser_save = 0;
        }
    }
    else {
        if (!game_end) {
            const int k = hardware.get_key();
            if (button_clicked(resume) || hardware.get_mr()
             || (k != -1 && k != menu       .get_hotkey()
                         && k != restart    .get_hotkey()
                         && k != save_replay.get_hotkey())) {
                exit_with = RESUME;
            }
        }
        if (!game_net && button_clicked(restart)) {
             exit_with = RESTART;
        }
        if (button_clicked(menu) || hardware.key_once(KEY_SPACE))
         exit_with = MENU;
        if (button_clicked(save_replay)) {
            browser_save = new SaveBrowser(
             Globals::dir_replay,
             Globals::ext_replay,
             Globals::user.replay_last_dir,
             Globals::empty_string,
             search_criterion_save_replay,
             new_box_overwrite, true); // true = benutze Replay-Style
            browser_save->set_info_file_name(replay->get_level_filename());
            browser_save->set_info_level_name(
             Level::get_name(replay->get_level_filename()));
        }
    }
}

void WindowGameplay::undraw()
{
    if (browser_save) browser_save->undraw();
    else Window::undraw();
}

void WindowGameplay::draw()
{
    if (browser_save) browser_save->draw();
    else Window::draw();
}

void WindowGameplay::draw_self()
{
    if (!game_end) resume .draw();
    if (!game_net) restart.draw();
    save_replay.draw();
    menu       .draw();
}



bool WindowGameplay::search_criterion_save_replay(const std::string& filename)
{
    return Help::string_ends_with(filename, Globals::ext_replay);
}



Api::BoxMessage* WindowGameplay::new_box_overwrite(const std::string& file)
{
    Replay r;
    r.load_from_file(file);
    std::string s1 = Language::editor_file_name + ' ' + file;
    std::string s2 = Language::editor_level_name
                   + ' ' + Level::get_name(r.get_level_filename());
    Api::BoxMessage* box_overwrite = new Api::BoxMessage(500, 3,
                                     Language::win_game_overwrite_title);
    box_overwrite->add_text(Language::win_game_overwrite_question);
    box_overwrite->add_text(s1);
    box_overwrite->add_text(s2);
    box_overwrite->add_button(Language::yes, KEY_ENTER);
    box_overwrite->add_button(Language::no,  KEY_ESC);
    return box_overwrite;
}

} // Ende Namensraum Api
