/*
 * level/gra_lib.cpp
 *
 */

#include "gra_lib.h"

#include "../lemming/lemming.h" // create countdown matrix
#include "../other/help.h" // remove_extension, Dateisuche

GraLib* GraLib::singleton = 0;



GraLib::GraLib()
{
    // Die Verzeichnisse nach Bilddateien durchsuchen
    // Abk.-Deklarationen, um die Funktionsaufrufe in einer Zeile zu haben
    const std::string dd              = Globals::dir_data_bitmap;
    void (*ssii)(std::string&, void*) = sort_string_into_internal;

    Help::find_tree(dd, Globals::mask_ext_bmp, ssii, (void*) this);
    Help::find_tree(dd, Globals::mask_ext_tga, ssii, (void*) this);
    Help::find_tree(dd, Globals::mask_ext_pcx, ssii, (void*) this);

    // Countdown-Matrix erstellen
    const Cutbit&     cb = internal[Globals::file_bitmap_lemming];
    const Lemming::XY xy = { cb.get_xl() / 2 - 1, 12 }; // Standardwert
          BITMAP*     b  = cb.get_al_bitmap();
    Lemming::countdown = Lemming::Matrix(
     cb.get_x_frames(), std::vector <Lemming::XY> (
     cb.get_y_frames(), xy));
    // fx, fy = welcher X- bzw. Y-Frame
    // x,  y  = wo in diesem Frame
    for  (int fx = 0; fx < cb.get_x_frames(); ++fx)
     for (int fy = 0; fy < cb.get_y_frames(); ++fy)
     for (int x  = 0; x  < cb.get_xl(); ++x )
     for (int y  = 0; y  < cb.get_yl(); ++y ) {
        // Pixel ansehen
        const int real_x = 1 + fx * (cb.get_xl() + 1) + x;
        const int real_y = 1 + fy * (cb.get_yl() + 1) + y;
        if (_getpixel16(b, real_x, real_y) == color[COL_LEMFILE_MARK]) {
            // Position im aktuellen Frame als Countdown-Position nutzen
            Lemming::countdown[fx][fy].x = x;
            Lemming::countdown[fx][fy].y = y;
            // Das if unterdrueckt Abstuerze, wenn einer dummes Zeug reinmalt
            if (real_x < cb.get_al_bitmap()->w - 1)
             _putpixel16(b, real_x, real_y, _getpixel16(b, real_x+1, real_y));
        }
    }
    // Alle Pixel sind abgegrast.

    // Alle Lemminge malen
    for (int loop = Lem::GARDEN; loop != Lem::STYLE_MAX; ++loop) {
        style.push_back(internal[Globals::file_bitmap_lemming]);
        BITMAP* b   = style[loop].get_al_bitmap();
        if (!b) continue; // Datei nicht gefunden oder sowas

        const int body_old = color[COL_LEMFILE_BODY];
        const int hair_old = color[COL_LEMFILE_HAIR];
        const int ruck_old = color[COL_LEMFILE_RUCK];
        const int skin_old = color[COL_LEMFILE_SKIN];

        for  (int y = 0; y < b->h; ++y)
         for (int x = 0; x < b->w; ++x) {
            const int        px = _getpixel16(b, x, y);
            const Lem::Style st = static_cast <Lem::Style> (loop);
            if      (px==body_old) _putpixel16(b,x,y,Lem::get_color(st,Lem::BODY));
            else if (px==hair_old) _putpixel16(b,x,y,Lem::get_color(st,Lem::HAIR));
            else if (px==ruck_old) _putpixel16(b,x,y,Lem::get_color(st,Lem::RUCK));
            else if (px==skin_old) _putpixel16(b,x,y,Lem::get_color(st,Lem::SKIN));
            else if (px==color[COL_BLACK]) x += style[st].get_xl();
        }
    }

    // Alle Namensersetzungen
    replacement["./bitmap/goal/goal.G.bmp"         ] = "./bitmap/goal.G.bmp";
    replacement["./bitmap/hatch/hatch.H.bmp"       ] = "./bitmap/hatch.H.bmp";
    replacement["./bitmap/hazard/trap/10tons.T.bmp"] = "./bitmap/10tons.T.bmp";
    replacement["./bitmap/hazard/trap/spike.T.bmp" ] = "./bitmap/spike.T.bmp";
    replacement["./bitmap/hazard/water/water.W.bmp"] = "./bitmap/water.W.bmp";
}



// Wir speichern alle Bilddateinamen ohne (Endung inklusive Punkt). Das hilft
// beim einfachen Austauschen des benutzten Grafikformates: Man kann einfach
// seine Grafiken konvertieren und die Endung aendern, wenn man den Datei-
// namen ansonsten konstant haelt.
void GraLib::sort_string_into_internal(std::string& s, void* v) {
    // zweites Argument: Nur Schneideversuch unternehmen, wenn mit Prae-End.
    const Cutbit c(s, Help::string_get_pre_extension(s));
    Help::string_remove_extension(s);
    ((GraLib*) v)->internal.insert(std::make_pair(s, c));
}



GraLib::~GraLib()
{
}



void GraLib::initialize()
{
    if (!singleton) singleton = new GraLib;
}



void GraLib::deinitialize()
{
    if (singleton) {
        delete singleton;
        singleton = 0;
    }
}



const Cutbit& GraLib::get(const std::string& str)
{
    return singleton->internal[str];
}



const Cutbit& GraLib::get_lemming(const Lem::Style style)
{
    return singleton->style[style];
}



const std::string& GraLib::replace_string(const std::string& str)
{
    const std::map <std::string, std::string> ::const_iterator
     itr = singleton->replacement.find(str);
    if (itr != singleton->replacement.end()) return itr->second;
    else return str;
}
