/*
 * map.cpp
 *
 * LEMSCR_Y - 60, weil das Panel 60 Pixel hoch ist
 *
 */

#include "map.h"

#include "../level/level.h"    // Mindestmasse
#include "../other/hardware.h" // Rechte Maustaste fuer Scrolling
#include "../other/globals.h"  // Scrollspeed-Einstellung
#include "../other/help.h"     // Timer-Ticks zur Scrollspeed-Berechnung

Map::Map(int w, int h, int x, int y)
:
    Torbit(w, h),
    scroll_speed(2 * Globals::user.scroll_speed
                   / Help::get_timer_ticks_per_draw()),
    zoom        (false)
{
    if (w < Level::min_xl) w = Level::min_xl;
    if (h < Level::min_yl) h = Level::min_yl;

    min_screen_x = 0;
    min_screen_y = 0;
    max_screen_x = w - Level::view_xl;
    max_screen_y = h - Level::view_yl;
    if (max_screen_x < min_screen_x) max_screen_x = min_screen_x;
    if (max_screen_y < min_screen_y) max_screen_y = min_screen_y;

    set_screen_x(x);
    set_screen_y(y);

    clear_to_color(color[COL_PINK]);
}



void Map::resize(int w, int h)
{
    if (w < Level::min_xl) w = Level::min_xl;
    if (h < Level::min_yl) h = Level::min_yl;

    Torbit::resize(w, h);

    // Nicht ueber den Rand scrollen:
    set_zoom(zoom);
}



void Map::set_torus_x(const bool b)
{
    Torbit::set_torus_x(b);
    set_screen_x(get_screen_x());
}
void Map::set_torus_y(const bool b)
{
    Torbit::set_torus_y(b);
    set_screen_y(get_screen_y());
}




bool Map::get_scrollable()
{
    return get_scrollable_up()
     ||    get_scrollable_right()
     ||    get_scrollable_down()
     ||    get_scrollable_left();
}
bool Map::get_scrollable_up()
{
    if (screen_y > min_screen_y
     || get_torus_y() && (Globals::user.scroll_torus_y == 2
     || Globals::user.scroll_torus_y == 1 && get_yl() > Level::view_yl))
     return true;
    else return false;
}
bool Map::get_scrollable_right()
{
    if (screen_x < max_screen_x
     || get_torus_x() && (Globals::user.scroll_torus_x == 2
     || Globals::user.scroll_torus_x == 1 && get_xl() > Level::view_xl))
     return true;
    else return false;
}
bool Map::get_scrollable_down()
{
    if (screen_y < max_screen_y
     || get_torus_y() && (Globals::user.scroll_torus_y == 2
     || Globals::user.scroll_torus_y == 1 && get_yl() > Level::view_yl))
     return true;
    else return false;
}
bool Map::get_scrollable_left()
{
    if (screen_x > min_screen_x
     || get_torus_x() && (Globals::user.scroll_torus_x == 2
     || Globals::user.scroll_torus_x == 1 && get_xl() > Level::view_xl))
     return true;
    else return false;
}



void Map::set_screen_x(const int x)
{
    screen_x = x;
    if (get_torus_x() && (Globals::user.scroll_torus_x == 2
     || (Globals::user.scroll_torus_x == 1 && get_xl() > Level::view_xl))) {
        screen_x = Help::mod(screen_x, get_xl());
    }
    else {
        if (screen_x < min_screen_x) screen_x = min_screen_x;
        if (screen_x > max_screen_x) screen_x = max_screen_x;
    }
}

void Map::set_screen_y(const int y)
{
    screen_y = y;
    if (get_torus_y() && (Globals::user.scroll_torus_y == 2
     || (Globals::user.scroll_torus_y == 1 && get_yl() > Level::view_yl))) {
        // If-Abfrage war: > min_h*3/2, aber der ADmiral und ich meinen
        // beide, dass es nur auf Einbildschirmkarten sinnig ist. Sonst
        // geht zu viel Vorteil verloren gegenueber der besseren Uebersicht.
        screen_y = Help::mod(screen_y, get_yl());
    }
    else {
        if (screen_y < min_screen_y) screen_y = min_screen_y;
        if (screen_y > max_screen_y) screen_y = max_screen_y;
    }
}



void Map::set_zoom(const bool b)
{
    zoom = b;
    min_screen_x =                           - zoom*Level::view_xl/4;
    min_screen_y =                           - zoom*Level::view_yl/4;
    max_screen_x = get_xl() - Level::view_xl + zoom*Level::view_xl/4;
    max_screen_y = get_yl() - Level::view_yl + zoom*Level::view_yl/4;
    if (max_screen_x < min_screen_x) max_screen_x = min_screen_x;
    if (max_screen_y < min_screen_y) max_screen_y = min_screen_y;
    // Verhindern, dass ueber den Rand gescrollt ist
    set_screen_x(screen_x);
    set_screen_y(screen_y);
}



int Map::get_mouse_x()
{
    int ret = screen_x;
    ret += (zoom ? Level::view_xl/4    : 0      );
    ret += (zoom ? hardware.get_mx()/2 : hardware.get_mx());
    if      (Level::view_xl > get_xl() * (zoom+1))
     ret -= (Level::view_xl - get_xl() * (zoom+1)) / 2; // image is centered
    if (get_torus_x()) ret = Help::mod(ret, get_xl());
    return ret;
}

int Map::get_mouse_y()
{
    int ret = screen_y;
    ret += (zoom ? Level::view_yl/4    : 0      );
    ret += (zoom ? hardware.get_my()/2 : hardware.get_my());
    if      (Level::view_yl > get_yl() * (zoom+1))
     ret -= (Level::view_yl - get_yl() * (zoom+1));
    if (get_torus_y()) ret = Help::mod(ret, get_yl());
    return ret;
}



int Map::get_mourning_border_x(const bool manual_zoom)
{
    const bool z = (zoom && !manual_zoom);
    return get_xl() * (z+1) < Level::view_xl
     ? (Level::view_xl - get_xl() * (z+1))/2 : 0;
}



int Map::get_mourning_border_y(const bool manual_zoom)
{
    const bool z = (zoom && !manual_zoom);
    return get_yl() * (z+1) < Level::view_yl
     ? Level::view_yl - get_yl() * (z+1) : 0;
}




void Map::calculate_scrolling()
{
    // Scrollen am Rand oder (immer aktiv) mit dem Numblock
    int scrd = scroll_speed;
    if (hardware.get_mrh()    || key[KEY_5_PAD])  scrd *= 2;
    if (zoom)                                    (scrd += 1) /= 2;
    if (Globals::user.scroll_border && hardware.get_my() == 0
     || key[KEY_8_PAD]
     || key[KEY_7_PAD]
     || key[KEY_9_PAD]) set_screen_y(screen_y - scrd);
    if (Globals::user.scroll_border && hardware.get_mx() == LEMSCR_X-1
     || key[KEY_6_PAD]
     || key[KEY_3_PAD]
     || key[KEY_9_PAD]) set_screen_x(screen_x + scrd);
    if (Globals::user.scroll_border && hardware.get_my() == LEMSCR_Y-1
     || key[KEY_2_PAD]
     || key[KEY_1_PAD]
     || key[KEY_3_PAD]) set_screen_y(screen_y + scrd);
    if (Globals::user.scroll_border && hardware.get_mx() == 0
     || key[KEY_4_PAD]
     || key[KEY_1_PAD]
     || key[KEY_7_PAD]) set_screen_x(screen_x - scrd);

    // Rechtsklick-Scrolling
    if (Globals::user.scroll_right) {
        if (hardware.get_mr()) {
            // Merken, auf welcher Hoehe die Maus war
            scroll_click_x = hardware.get_mx();
            scroll_click_y = hardware.get_my();
        }
        if (hardware.get_mrh()) {
            const bool xp = get_scrollable_right();
            const bool xm = get_scrollable_left();
            const bool yp = get_scrollable_down();
            const bool ym = get_scrollable_up();
            // Bildschirm tatsaechlich scrollen und ggf. Maus festhalten
            if (xm && hardware.get_mx      () <= scroll_click_x
                   && hardware.get_mickey_x() <  0
             || xp && hardware.get_mx      () >= scroll_click_x
                   && hardware.get_mickey_x() >  0) {
                set_screen_x(screen_x + hardware.get_mickey_x());
                hardware.freeze_mouse_x();
            }
            if (ym && hardware.get_my      () <= scroll_click_y
                   && hardware.get_mickey_y() <  0
             || yp && hardware.get_my      () >= scroll_click_y
                   && hardware.get_mickey_y() >  0) {
                set_screen_y(screen_y + hardware.get_mickey_y());
                hardware.freeze_mouse_y();
            }
        }
        // Ende mrh
    }
    // Ende Rechtsklick-Scrolling
}
// Ende calculate_scrolling()



void Map::draw(Torbit& target, const bool manual_zoom)
{
                       draw_at(target, manual_zoom, false, false);
    if (get_torus_x()) draw_at(target, manual_zoom, true,  false);
    if (get_torus_y()) draw_at(target, manual_zoom, false, true );
    if (get_torus_x() && get_torus_y()) draw_at(target,manual_zoom,true,true);

    // Bei kleinen Karten zusaetzlich den Aussenbereich faerben.
    // Bei der Rueckgabe der Mauskoordinaten wird dies beruecksichtigt.
    const int offx = get_mourning_border_x(manual_zoom);
    const int offy = get_mourning_border_y(manual_zoom);
    const int c = Globals::user.screen_border_colored
                ? color[COL_SCREEN_BORDER] : 0;
    if (offx) {
        rectfill(target.get_al_bitmap(), 0, 0, offx-1, Level::view_yl-1, c);
        rectfill(target.get_al_bitmap(), Level::view_xl-offx, 0,
         Level::view_xl-1, Level::view_yl-1, c);
    }
    if (offy) rectfill(target.get_al_bitmap(), offx, 0, Level::view_xl-offx-1,
               offy-1, c);
}



void Map::draw_at(
    Torbit&    target,
    const bool manual_zoom,
    const bool px,
    const bool py
) {
    const bool z = (zoom && !manual_zoom);
    const int& mapx = get_xl();
    const int& mapy = get_yl();
    const int  scrx = !z ? screen_x : screen_x + Level::view_xl/4;
    const int  scry = !z ? screen_y : screen_y + Level::view_yl/4;
    const int& lx = Level::view_xl;
    const int& ly = Level::view_yl;
    const int  offx = mapx*(z+1) < lx ? (lx - mapx*(z+1))/2 : 0; // Mal-Offset
    const int  offy = mapy*(z+1) < ly ?  ly - mapy*(z+1)    : 0; // sehr kleine
    if (!z) masked_blit(get_al_bitmap(), target.get_al_bitmap(),
     !px ? scrx : 0, // Quell-Start-xy
     !py ? scry : 0, // Woher?
     !px ? 0 + offx : mapx - scrx + offx, // Ziel-Start-xy
     !py ? 0 + offy : mapy - scry + offy, // Wohin?
     !px ? (mapx-scrx < lx ? mapx-scrx : lx) : lx - (mapx-scrx),  // Laenge:
     !py ? (mapy-scry < ly ? mapy-scry : ly) : ly - (mapy-scry)); // Wieviel?
    else masked_stretch_blit(get_al_bitmap(), target.get_al_bitmap(),
     // Quell-Start-XY
     !px ? scrx : 0,
     !py ? scry : 0,
     // Quell-Laenge, also wieviel
     !px ? (mapx-scrx < lx/2 ? mapx-scrx : lx/2) : lx/2 - (mapx-scrx),
     !py ? (mapy-scry < ly/2 ? mapy-scry : ly/2) : ly/2 - (mapy-scry),
     // Ziel-Start-XY
     !px ? 0 + offx : 2*(mapx-scrx) + offx,
     !py ? 0 + offy : 2*(mapy-scry) + offy,
     // Ziel-Wieviel
     !px ? (2*(mapx-scrx) < lx ? 2*(mapx-scrx) : lx) : lx - (2*(mapx-scrx)),
     !py ? (2*(mapy-scry) < ly ? 2*(mapy-scry) : ly) : ly - (2*(mapy-scry)));
}



void Map::load_masked_screen_rectangle(Torbit& src)
{
    if (get_xl() != src.get_xl() || get_yl() != src.get_yl()) return;
    const int&  x  = screen_x;
    const int&  y  = screen_y;
    const int&  xl = get_xl();
    const int&  yl = get_yl();
    const bool& tx = get_torus_x();
    const bool& ty = get_torus_y();
    const int min_w = Level::view_xl;
    const int min_h = Level::view_yl;
    BITMAP* s = src.get_al_bitmap();
    BITMAP* b =     get_al_bitmap();
                masked_blit(s, b, x,    y,    x,    y,    min_w, min_h);
    if (tx)     masked_blit(s, b, x-xl, y,    x-xl, y,    min_w, min_h);
    if (ty)     masked_blit(s, b, x,    y-yl, x,    y-yl, min_w, min_h);
    if (tx&&ty) masked_blit(s, b, x-xl, y-yl, x-xl, y-yl, min_w, min_h);
}



void Map::clear_screen_rectangle(int col)
{
    const int&  x1 = screen_x;
    const int&  y1 = screen_y;
    const int&  x2 = get_xl();
    const int&  y2 = get_yl();
    const bool& tx = get_torus_x();
    const bool& ty = get_torus_y();
    BITMAP* b = get_al_bitmap();
                rectfill(b, x1, y1, x2, y2, col);
    if (tx)     rectfill(b, 0,  y1, x1, y2, col);
    if (ty)     rectfill(b, x1, 0,  x2, y1, col);
    if (tx&&ty) rectfill(b, 0,  0,  x1, y1, col);
}
