/*
 * lemming/jumper.cpp
 *
 */

#include "ac.h"

void assign_jumper(Lemming& l)
{
    l.set_frame    ( 0);
    l.set_special_y(-8); // Y-Geschwindigkeit
    l.set_special_x( 6); // X-Geschwindigkeit
    l.set_ac       (Lem::JUMPER);

    for (int i = -4; i > -16; --i)
     if (l.is_solid(0, i)) {
        l.assign(Lem::STUNNER);
        return;
    }
}



// wird in tumbler.cpp definiert
bool jumper_and_tumbler_collision(Lemming&);

void update_jumper(Lemming& l, const UpdateArgs& ua)
{
    if (l.get_special_x() % 2 == 1) {
        if (l.get_special_x() > 0) l.set_special_x(l.get_special_x() + 1);
        else                       l.set_special_x(l.get_special_x() - 1);
    }
    const int       sgn = l.get_special_y() >= 0 ? 1 : -1; // sgn(Y-Spd.)
    const unsigned  abs = l.get_special_y() / sgn;         // |Y-Spd.|
    const unsigned& spe = l.get_special_x();               // X-Spd.
    // Das Signum ist auch fuer 0 gleich +1, das macht im Code allerdings
    // nix, weil in dem Fall null Schleifendurchlaeufe passieren oder
    // mit null multipliziert wird.

    // Schrittweise schraeg vorruecken und jedes Mal auf
    // Kollision mit der Landschaft testen.
    // Die groessere Laenge wird pixelweise abgearbeitet.
    for (unsigned i = 0; i < (abs >= spe ? abs : spe); ++i) {
        // 2 * (... / 2) sorgt fuer das Einhalten der geraden X-Zahlen.
        // Es wird jeweils geguckt, ob der Zaehler i weit genug vor-
        // geschritten ist, damit in die kurze Richtung ein
        // Schritt stattfinden kann - wenn nicht, ist das Argument
        // von move_ahead stets 0. In die lange Richtung geht's immer.
        if (abs >= spe) {
            l.move_ahead(2 * ((i+1)*spe/2 / abs - i*spe/2 / abs));
            l.move_down(sgn);
        }
        else {
            l.move_down(sgn * ((i+1)*abs/spe - i*abs/spe));
            l.move_ahead(i%2 * 2);
        }

        if (jumper_and_tumbler_collision(l)) {
            switch (l.get_ac()) {
            case Lem::STUNNER:  l.play_sound(ua, Sound::OUCH);  break;
            case Lem::SPLATTER: l.play_sound(ua, Sound::SPLAT); break;
            default: break;
            }
            break;
        }
    }

    // Wenn nicht beim Bewegen irgendwo angestossen...
    if (l.get_ac() == Lem::JUMPER
     || l.get_ac() == Lem::TUMBLER) {
        l.set_special_y(l.get_special_y() > 12 ?
                        l.get_special_y() +  1 :
                        l.get_special_y() +  2);
        if (l.get_special_y() > 14) {
            const int sx = l.get_special_x();
            const int sy = l.get_special_y();
            if      (l.get_floater())
                     l.assign(     Lem::FLOATER);
            else if (l.get_ac() != Lem::TUMBLER)
                     l.assign(     Lem::TUMBLER);
            l.set_special_x(sx);
            l.set_special_y(sy);
        }
        if (l.get_ac() == Lem::JUMPER && l.is_last_frame()) {
            l.set_frame(l.get_frame() - 1);
        }
        else l.next_frame();
    }
}
