/*
 * lemming/lem_enum.cpp
 *
 */

#include <map>
#include <vector>

#include "lem_enum.h"

#include "../other/myalleg.h" // makecol

namespace Lem {

std::map    <Ac,    std::string> ac_string;
std::map    <Style, std::string> style_string;
std::vector <std::vector <int> > style_color(STYLE_MAX, std::vector<int>
                                                        (BODY_PART_MAX));



void initialize_strings_this_gets_called_from_globals_load_config_file()
{
    ac_string   [NOTHING]           = "NOTHING";
    ac_string   [FALLER]            = "FALLER";
    ac_string   [TUMBLER]           = "TUMBLER";
    ac_string   [STUNNER]           = "STUNNER";
    ac_string   [LANDER]            = "LANDER";
    ac_string   [SPLATTER]          = "SPLATTER";
    ac_string   [BURNER]            = "BURNER";
    ac_string   [DROWNER]           = "DROWNER";
    ac_string   [EXITER]            = "EXITER";
    ac_string   [WALKER]            = "WALKER";

    ac_string   [CLIMBER]           = "CLIMBER";
    ac_string   [ASCENDER]          = "ASCENDER";
    ac_string   [FLOATER]           = "FLOATER";
    ac_string   [EXPLODER]          = "EXPLODER";
    ac_string   [EXPLODER2]         = "EXPLODER2";
    ac_string   [BLOCKER]           = "BLOCKER";
    ac_string   [BUILDER]           = "BUILDER";
    ac_string   [SHRUGGER]          = "SHRUGGER";
    ac_string   [PLATFORMER]        = "PLATFORMER";
    ac_string   [SHRUGGER2]         = "SHRUGGER2";
    ac_string   [BASHER]            = "BASHER";
    ac_string   [MINER]             = "MINER";
    ac_string   [DIGGER]            = "DIGGER";

    ac_string   [JUMPER]            = "JUMPER";
    ac_string   [ROPER]             = "ROPER";
    ac_string   [AC_MAX]            = "AC_MAX";

    style_string[GARDEN]            = "Garden";
    style_string[RED]               = "Red";
    style_string[YELLOW]            = "Yellow";
    style_string[GREEN]             = "Green";
    style_string[BLUE]              = "Blue";
    style_string[CYAN]              = "Cyan";
    style_string[PURPLE]            = "Purple";
    style_string[GREY]              = "Grey";
    style_string[BLACK]             = "Black";
}



void initialize_colors_this_gets_called_from_glob_gfx_cpp()
{
    style_color [GARDEN][BODY]      = makecol(0x28, 0x28, 0xFF);
    style_color [GARDEN][HAIR]      = makecol(   0, 0xC0,    0);
    style_color [GARDEN][RUCK]      = makecol(0xFF, 0x40, 0x40);
    style_color [GARDEN][SKIN]      = makecol(0xFF, 0xFF, 0xFF);

    style_color [RED][BODY]         = makecol(0xC0,    0,    0);
    style_color [RED][HAIR]         = makecol(0xFF, 0x40, 0x40);
    style_color [RED][RUCK]         = makecol(0x40, 0x60, 0xFF);
    style_color [RED][SKIN]         = style_color[GARDEN][SKIN];
    style_color [YELLOW][BODY]      = makecol(0xD0, 0xA0,    0);
    style_color [YELLOW][HAIR]      = makecol(0xFF, 0xC0,    0);
    style_color [YELLOW][RUCK]      = style_color[GARDEN][RUCK];
    style_color [YELLOW][SKIN]      = style_color[GARDEN][SKIN];
    style_color [GREEN][BODY]       = makecol(   0, 0xB0,    0);
    style_color [GREEN][HAIR]       = makecol(   0, 0xD0,    0);
    style_color [GREEN][RUCK]       = style_color[GARDEN][RUCK];
    style_color [GREEN][SKIN]       = style_color[GARDEN][SKIN];
    style_color [BLUE][BODY]        = makecol(   0,    0, 0xE0);
    style_color [BLUE][HAIR]        = makecol(0x60, 0x60, 0xFF);
    style_color [BLUE][RUCK]        = style_color[GARDEN][RUCK];
    style_color [BLUE][SKIN]        = style_color[GARDEN][SKIN];
    style_color [CYAN][BODY]        = makecol(0x00, 0xC0, 0xE0);
    style_color [CYAN][HAIR]        = makecol(0x80, 0xE0, 0xFF);
    style_color [CYAN][RUCK]        = style_color[GARDEN][RUCK];
    style_color [CYAN][SKIN]        = style_color[GARDEN][SKIN];
    style_color [PURPLE][BODY]      = makecol(0xD0,    0, 0xD0);
    style_color [PURPLE][HAIR]      = makecol(0xFF, 0x60, 0xFF);
    style_color [PURPLE][RUCK]      = makecol(   0, 0xC0,    0);
    style_color [PURPLE][SKIN]      = style_color[GARDEN][SKIN];
    style_color [GREY][BODY]        = makecol(0x80, 0x80, 0x80);
    style_color [GREY][HAIR]        = makecol(0xA0, 0xA0, 0xA0);
    style_color [GREY][RUCK]        = style_color[GARDEN][RUCK];
    style_color [GREY][SKIN]        = style_color[GARDEN][SKIN];
    style_color [BLACK][BODY]       = makecol(0x40, 0x40, 0x40);
    style_color [BLACK][HAIR]       = makecol(0x60, 0x60, 0x60);
    style_color [BLACK][RUCK]       = style_color[GARDEN][RUCK];
    style_color [BLACK][SKIN]       = style_color[GARDEN][SKIN];
}



Ac string_to_ac(const std::string& s) {
    for (std::map <Ac, std::string> ::iterator i = ac_string.begin();
         i != ac_string.end(); ++i) {
        if (i->second == s) return i->first;
    } return AC_MAX;
}



const std::string& ac_to_string(const Ac a) {
    for (std::map <Ac, std::string> ::iterator i = ac_string.begin();
         i != ac_string.end(); ++i) {
        if (i->first == a) return i->second;
    } return (--ac_string.end())->second;
}



Style string_to_style(const std::string& s) {
    for (std::map <Style, std::string> ::iterator i = style_string.begin();
         i != style_string.end(); ++i) {
        if (i->second == s) return i->first;
    } return GARDEN;
}



const std::string& style_to_string(const Style a) {
    for (std::map <Style, std::string> ::iterator i = style_string.begin();
         i != style_string.end(); ++i) {
        if (i->first == a) return i->second;
    } return style_string.begin()->second;
}



int get_color(const Style st, const BodyPart bp)
{
    if (st == (int) STYLE_MAX || bp >= (int) style_color.size()) return 0;
    else return style_color[st][bp];
}

}
// End of namespace Lem
