/*
 * lemming/lemmi_ac.cpp
 *
 * Auslagerung aller faehigkeitslastigen Memberfunktionen
 *
 */

#include "lemming.h"

#include "../gameplay/gameplay.h" // Abstand fuer Blocker zum Ziel
#include "../gameplay/goal.h"     // Ebenso Blockerabstand
#include "../other/globals.h"

unsigned Lemming::get_priority(Lem::Ac                   new_ac,
                               unsigned                  tribes_active,
                               const std::vector <Goal>& goal,
                               const bool                personal)
{
    // Argument:  Die Faehigkeit, die beim Klick gegeben wuerde.
    // Rueckgabe: 0, falls nicht anklickbar und Cursor geschlossen.
    //            1, falls nicht anklickbar und Cursor dennoch offen.
    //            2 bis 99.998, falls anklickbar. Hoehere Zahl = hoehere Prior.

    unsigned p = 0;

    // Bleibende Faehigkeiten
    if (new_ac == Lem::EXPLODER  && updates_since_bomb > 0
     || new_ac == Lem::EXPLODER2 && updates_since_bomb > 0
     || new_ac == Lem::CLIMBER   && climber
     || new_ac == Lem::FLOATER   && floater) return 1;

    // Blocker duerfen im Mehrspielermodus nicht in die Naehe von Ausgaengen
    // gepflanzt werden
    if (new_ac == Lem::BLOCKER && tribes_active > 1)
     for (Goal::CIt i = goal.begin(); i != goal.end(); ++i)
     if (get_in_trigger_area(*i, true)) return 1; // true == twice

    switch (ac) {
        // Gar keine Fhigkeiten sind erlaubt
        case Lem::NOTHING:
        case Lem::SPLATTER:
        case Lem::DROWNER:
        case Lem::BURNER:
            return 0;

        case Lem::EXITER:
        case Lem::EXPLODER:
            return 1;

        // Nur Exploder ist erlaubt, und zwar mit wesentlich hoeherer Prior.!
        // Falls der Walker zuweisbar ist, dann ist er es ebenfalls Blockern.
        case Lem::BLOCKER:
            if (new_ac == Lem::WALKER && frame < 20
             || new_ac == Lem::EXPLODER
             || new_ac == Lem::EXPLODER2) p = 5000;
            else return 1;
            break;

        // Sonstige vereinnahmende Aktivitaeten, die den Lemming mehr oder
        // weniger willentlich in Komplettbeschlag nehmen:
        // Bleibende Fhigkeiten und Exploder sind immer erlaubt
        case Lem::FALLER:
        case Lem::TUMBLER:
        case Lem::STUNNER:

        case Lem::ASCENDER:
        case Lem::CLIMBER:
        case Lem::FLOATER:
        case Lem::JUMPER:
            if (new_ac == Lem::CLIMBER
             || new_ac == Lem::FLOATER
             || new_ac == Lem::EXPLODER
             || new_ac == Lem::EXPLODER2) p = 2000;
            else return 1;
            break;

        case Lem::WALKER:
        case Lem::LANDER:
            p = 3000;
            break;

        // Builder kann selbstzuweisen, Ausnahme vom Default-Wert erforderlich
        // Mit verneinter Einstellung oder im Mehrspielermodus
        // wird's allerdings wird's wie Default behandelt.
        case Lem::BUILDER:
        case Lem::PLATFORMER:
            if      (new_ac == ac
                && (!personal || Globals::user.multiple_builders)) p = 1000;
            else if (new_ac != ac)                                 p = 4000;
            else                                              return      1;
            break;

        // Normalerweise kann alles zugewiesen werden, was der Lemming
        // gerade nicht tut.
        default:
            if (new_ac != ac) p = 4000;
            else return 1;

    }
    p += updates_since_bomb;
    p += 100 * (climber + floater);
    return p;
}



void Lemming::evaluate_click(Lem::Ac new_ac)
{
    switch (new_ac) {
    case Lem::CLIMBER:   climber = true;            break;
    case Lem::FLOATER:   floater = true;            break;
    case Lem::EXPLODER2: exploder_knockback = true; // faellt durch
    case Lem::EXPLODER:  inc_updates_since_bomb();  break;

    default:
        if (ac == new_ac)   assign(new_ac);
        else              { assign(new_ac); --frame; }
        break;
    }
}
