/*
 * level.cpp
 *
 */

#include "level.h"

#include "../graphic/gra_lib.h"
#include "../other/help.h"
#include "../other/language.h"

const int Level::min_xl (LEMSCR_X);
const int Level::min_yl (160 * 2);
const int Level::view_xl(LEMSCR_X);
const int Level::view_yl(LEMSCR_Y - 60);

const Date Level::date_before_terrain_reverse = Date("2009-08-23 00:00:00");

Level::Level(const std::string& filename)
:
    pos(Object::MAX)
{
    load_from_file(filename);
}

Level::~Level()
{
    clear();
}

void Level::clear()
{
    status = BAD_EMPTY;

    built        = "";
    author       = "";
    name_german  = "";
    name_english = "";
    size_x       = view_xl;
    size_y       = view_yl;
    start_x      =  0;
    start_y      =  0;
    torus_x      = false;
    torus_y      = false;
    seconds      =  0;
    lemmings     = 30;
    required     = 20;
    rate         = 50;

    for (int i = Object::TERRAIN; i != Object::MAX; ++i) pos[i].clear();

    for (int i = 0; i < 8; ++i) {
        skill[i].ac = Lem::NOTHING;
        skill[i].nr = 0;
    }
}



bool Level::operator == (const Level& l) const
{
    if (this->author       != l.author
     || this->name_german  != l.name_german
     || this->name_english != l.name_english

     || this->size_x       != l.size_x
     || this->size_y       != l.size_y
     || this->start_x      != l.start_x
     || this->start_y      != l.start_y
     || this->torus_x      != l.torus_x
     || this->torus_y      != l.torus_y

     || this->seconds      != l.seconds
     || this->lemmings     != l.lemmings
     || this->required     != l.required
     || this->rate         != l.rate) return false;

    for (int i = Object::TERRAIN; i != Object::MAX; ++i)
     if (pos[i]            != l.pos[i]) return false;

    for (int i = 0; i < skill_max; ++i)
     if (this->skill[i].ac != l.skill[i].ac
     ||  this->skill[i].nr != l.skill[i].nr) return false;

    return true;
}



bool Level::operator != (const Level& l) const
{
    return !(*this == l);
}



Level::Pos::Pos()
:
    ob  (0),
    x   (0),
    y   (0),
    mirr(false),
    rot (0),
    dark(false),
    noow(false)
{
}



bool Level::Pos::operator == (const Level::Pos& t) const
{
    return this->ob   == t.ob
        && this->x    == t.x
        && this->y    == t.y
        && this->mirr == t.mirr
        && this->rot  == t.rot
        && this->dark == t.dark
        && this->noow == t.noow;
}



const std::string& Level::get_name() const {
    if (Language::get() == Language::GERMAN)
         return name_german .empty() ? name_english : name_german;
    else return name_english.empty() ? name_german  : name_english;
}
