/*
 * menu/brow_big.cpp
 *
 */

#include "brow_big.h"

#include "../level/level.h"
#include "../other/language.h"

namespace Api {

BrowserBig::BrowserBig(const std::string& wintitle,
                       const std::string& basedir,
                       const std::string& lastdir,
                       const std::string& lastfile,
                       const bool checkmark_style,
                       const bool replay_style)
:
    Window(*osd, 0, 0, LEMSCR_X, LEMSCR_Y, wintitle),
    exit_with  (EXIT_WITH_NOTHING),
    info_y     (200),
    dir_list   (get_torbit(), 20, 40,
                dir_list_xl, any_list_yl, basedir, lastdir),
    lev_list   (get_torbit(), 40+dir_list_xl, 40,
                lev_list_xl, any_list_yl),
    button_play(get_torbit(), but_x, 40 + (but_yl + but_y_spacing) * 0,
                but_xl, but_yl),
    button_exit(get_torbit(), but_x, LEMSCR_Y - 60,
                but_xl, but_yl),
    preview    (get_torbit(), but_x, 20 + but_yl + 2 * but_y_spacing,
                but_xl, pre_yl)
{
    lev_list.set_activate_clicked_button();
    lev_list.set_checkmark_style(checkmark_style);
    lev_list.set_replay_style   (replay_style);
    lev_list.load_dir           (dir_list.get_current_dir());
    lev_list.highlight_file     (lastfile);
    filename = get_current_dir() + get_current_file();
    // on_level_highlight() muss von der abgeleiteten Klasse aufgerufen
    // werden, weil diese zu diesem Konstruktoraufruf-Zeitpunkt noch
    // nicht konstruiert ist.
    button_exit.set_text(Language::back);
    button_exit.set_hotkey(KEY_ESC);
}



BrowserBig::~BrowserBig()
{
}



void BrowserBig::set_current_dir_to_parent_dir()
{
    dir_list.set_current_dir_to_parent_dir();
    lev_list.load_dir(dir_list.get_current_dir());
}



void BrowserBig::calculate()
{
    if (dir_list.clicked(get_x(), get_y())) {
        lev_list.load_dir(dir_list.get_current_dir());
        filename = get_current_dir() + get_current_file();
        on_file_highlight      (filename);
        lev_list.highlight_file(get_current_file());
        // DEBUGGING: Dies ist noch nicht optimal bei mehreren Dateien, die
        // gleich heissen in verschiedenen Verzeichnissen!
    }
    else if (lev_list.clicked(get_x(), get_y())) {
        Button* b = lev_list.get_button_last_clicked();
        // Gerade zum ersten Mal angeklickt
        if (b->get_on()) {
            filename = get_current_dir() + get_current_file();
            on_file_highlight(filename);
        }
        // Button schon angeklickt?
        else {
            b->set_on(); // Damit ein nichtguter Level nicht deselektiert wird
            on_file_select(filename);
        }
    }
    else if (button_play.clicked(get_x(), get_y())) {
        // Ohne Check, ob wir im richtigen Verzeichnis sind
        filename = get_current_dir() + get_current_file();
        on_file_select(filename);
    }
    else if (button_exit.clicked(get_x(), get_y())) exit_with = EXIT_WITH_EXIT;
}



void BrowserBig::draw_self()
{
    if (get_current_dir() != get_subtitle()) set_subtitle(get_current_dir());
    dir_list.draw();
    lev_list.draw();
    preview .draw();
    button_play.draw();
    button_exit.draw();
}



void BrowserBig::reload_dir()
{
    lev_list.load_dir(lev_list.get_current_dir());
}



void BrowserBig::clear_info()
{
    rectfill(get_torbit().get_al_bitmap(), but_x, info_y, but_x + but_xl - 1,
     LEMSCR_Y - but_yl - 2*but_y_spacing, color[COL_API_M]);
}

}
// Ende des Namensraumes Api
