/*
 * menu/brow_rep.cpp
 *
 */

#include "browsers.h"

#include "../gameplay/replay.h"

namespace Api {

ReplayBrowser::ReplayBrowser()
:
    BrowserBig(Language::browser_replay_title,
               Globals::dir_replay,
               Globals::user.replay_last_dir,
               Globals::user.replay_last_file,
               false, true), // kein Checkmark-Stil, aber Replay-Stil

    extraction_possible(false),
    deletion_possible  (false),
    button_extract(get_torbit(), but_x, LEMSCR_Y - 180,
                   but_xl, but_yl),
    button_delete (get_torbit(), but_x, LEMSCR_Y - 120,
                   but_xl, but_yl),
    browser_save(0),
    box_delete  (0)
{
    set_button_play_text(Language::browser_replay);
    button_extract.set_text(Language::browser_extract);
    button_extract.set_undraw_color(color[COL_API_M]);
    button_delete .set_text(Language::browser_delete);
    button_delete .set_undraw_color(color[COL_API_M]);

    on_file_highlight(get_current_dir() + get_current_file());
}



ReplayBrowser::~ReplayBrowser()
{
    if (browser_save) delete browser_save;
    if (box_delete  ) delete box_delete;
}



void ReplayBrowser::calculate()
{
    if (browser_save) {
        browser_save->calculate();
        browser_save->set_draw_required();
        if (browser_save->get_exit()) {
            if (browser_save->get_exit_with()) {
                Globals::user.single_last_dir  = browser_save->get_current_dir();
                Globals::user.single_last_file = browser_save->get_current_file();
                // Replay-Verzeichnis und -Datei wurden aktualisiert, als
                // auf den Extraktions-Button gedrueckt wurde
                Replay r(get_current_dir() + get_current_file());
                Level  l(r.get_level_filename());
                l.save_to_file(browser_save->get_current_dir()
                             + browser_save->get_current_file());
            }
            delete browser_save;
            browser_save = 0;
        }
    }
    else if (box_delete) {
        box_delete->calculate();
        box_delete->set_draw_required(); // Damit Buttons eingedrueckt werd.
        std::string filename = get_current_dir() + get_current_file();
        switch (box_delete->get_button_clicked()) {
        case 1:
            delete_file(filename.c_str());
            reload_dir();

            clear_preview();
            clear_info();
            deletion_possible   = false;
            extraction_possible = false;
            button_delete .undraw();
            button_extract.undraw();
            // faellt durch
        case 2:
            delete box_delete;
            box_delete = 0;
        default: ;
        }
    }
    // Replaybrowser normal, also ohne Extraktionsdialog
    else {
        BrowserBig::calculate();

        if (extraction_possible && button_clicked(button_extract)) {
            Globals::user.replay_last_dir  = get_current_dir();
            Globals::user.replay_last_file = get_current_file();
            const std::string&   lf   = Globals::user.replay_last_dir
                                      + Globals::user.replay_last_file;
            browser_save = new Api::SaveBrowser(Globals::dir_levels,
                           Globals::ext_level,
                           Globals::user.single_last_dir,
                           Globals::empty_string,
                           Api::SaveBrowser::search_criterion_level,
                           Api::SaveBrowser::new_box_overwrite_level);
            browser_save->set_y(50); // mittig, nicht der Editor-Standard
            browser_save->set_info_level_name(Level::get_name(lf));
            browser_save->set_info_file_name(lf);
        }
        else if (deletion_possible && button_clicked(button_delete)) {
            std::string filename = get_current_dir() + get_current_file();
            if (exists(filename.c_str())) {
                Replay r(filename);
                std::string s1 = Language::editor_file_name
                               + ' ' + get_current_dir() + get_current_file();
                std::string s2 = Language::editor_level_name
                               + ' ' + Level::get_name(r.get_level_filename());
                box_delete = new Api::BoxMessage(500, 3,
                                          Language::browser_box_delete_title);
                box_delete->add_text(Language::browser_box_delete_question);
                box_delete->add_text(s1);
                box_delete->add_text(s2);
                box_delete->add_button(Language::yes, KEY_ENTER);
                box_delete->add_button(Language::no,  KEY_ESC);
            }
        }
        // Den normalen Dialog-Siedler-2-Rechtsklick realisieren:
        else if (hardware.get_mr()) {
            if (get_current_dir() != get_base_dir())
             set_current_dir_to_parent_dir();
            else set_exit_with(EXIT_WITH_EXIT);
        }
    }
}



void ReplayBrowser::on_file_highlight(const std::string& filename)
{
    Replay r(filename);
    Level  l(r.get_level_filename());
    clear_info();
    set_preview(l);

    if (r.get_file_not_found()) {
        deletion_possible = false;
        button_delete.undraw();
        clear_preview();
    }
    else if (l.get_status() == Level::BAD_FILE_NOT_FOUND) {
        deletion_possible = true;
        button_delete.set_draw_required();
    }
    else {
        const int y = get_info_y();
        const int c = color[COL_WHITE];

        const unsigned long over = Globals::version;
        const unsigned long omin = Globals::version_min;
        const unsigned long rmin = r.get_version_min();
        const Date&         dlev = l.built;
        const Date&         drep = r.get_built_required();
        const bool&   holdslevel = r.get_holds_level();

        std::string pl = Language::browser_info_player  + ' ';
        std::string ti = Language::browser_info_version + ' ';
        std::string bu = Language::browser_info_built   + ' ';

        pl +=               r.get_player_local_name();
        ti += rmin > over ? Language::browser_info_old
            : rmin < omin ? Language::browser_info_new
            :               Language::browser_info_same;
        bu += holdslevel  ? Language::browser_info_holds_level
            : drep > dlev ? Language::browser_info_old
            : drep < dlev ? Language::browser_info_new
            :               Language::browser_info_same;

        draw_text(pl,                             but_x, y,       c);
        draw_text(Language::browser_info_player,  but_x, y         );
        draw_text(ti,                             but_x, y +  40, c);
        draw_text(Language::browser_info_version, but_x, y +  40   );
        draw_text(bu,                             but_x, y +  60, c);
        draw_text(Language::browser_info_built,   but_x, y +  60   );

        if (r.get_holds_level()) {
            extraction_possible = true;
            button_extract.set_draw_required();
        }
        else {
            extraction_possible = false;
            button_extract.undraw();
        }
        deletion_possible = true;
        button_delete.set_draw_required();
    }
}



void ReplayBrowser::on_file_select(const std::string& filename)
{
    Replay r(filename);
    Level  l(r.get_level_filename());

    if (l.get_status() == Level::GOOD) {
        set_exit_with(EXIT_WITH_OKAY);
        Globals::user.replay_last_dir  = get_current_dir();
        Globals::user.replay_last_file = get_current_file();
    }
}



void ReplayBrowser::draw()
{
    BrowserBig::draw();
    if (browser_save) browser_save->draw();
    if (box_delete  ) box_delete  ->draw();
}



void ReplayBrowser::draw_self()
{
    BrowserBig::draw_self();
    if (extraction_possible) button_extract.draw();
    if (deletion_possible  ) button_delete.draw();
}

} // Ende Namensraum Api
