/*
 * menu/browser.cpp
 *
 */

#include "browsers.h"

#include "../level/crunch.h"

namespace Api {

SingleBrowser::SingleBrowser()
:
    BrowserBig(Language::browser_single_title,
                        Globals::dir_levels,
                        Globals::user.single_last_dir,
                        Globals::user.single_last_file,
                        true, false), // Checkmark-Stil, aber kein Replay-Stil

    button_edit(get_torbit(), but_x, 40 + (but_yl + but_y_spacing) * 1,
                but_xl, but_yl)
{
    set_button_play_text(Language::browser_play);
    button_edit.set_text(Language::browser_edit);

    set_preview_y(40 + (but_yl + but_y_spacing) * 2);
    set_info_y   (260);

    on_file_highlight(get_current_dir() + get_current_file());
}

SingleBrowser::~SingleBrowser()
{
}



void SingleBrowser::calculate()
{
    BrowserBig::calculate();

    if (button_clicked(button_edit)) {
        on_file_select(get_current_dir() + get_current_file());
    }
}



void SingleBrowser::draw_self()
{
    BrowserBig::draw_self();
    button_edit.draw();
}



void SingleBrowser::on_file_highlight(const std::string& filename)
{
    Torbit&   b = get_torbit();
    FONT*     f = font_med;
    const int x = but_x + but_xl;
    const int y = get_info_y();
    const int c = color[COL_WHITE];

    std::string cl;

    Level l(filename);
    clear_info();
    if (l.get_status() == Level::BAD_FILE_NOT_FOUND) clear_preview();
    else set_preview(l);

    if (!l.get_good()) return;

    ////////////////////////////////////
    // Allgemeine Leveldaten anzeigen //
    ////////////////////////////////////

    if (l.seconds > 0) {
        cl += l.seconds / 60;
        cl += ':';
        cl += l.seconds % 60 / 10;
        cl += l.seconds % 10;
    }
    else cl = Language::browser_info_clock_none;

    // Wenn der Level brauchbar ist, schreibe fuer diesen Modus wichtige Daten
    draw_text(Language::browser_info_lemmings, but_x, y     );
    draw_text(Language::browser_info_required, but_x, y + 20);
    draw_text(Language::browser_info_clock_1,  but_x, y + 40);

    // true: Schreibe rechtsbuendig
    Help::draw_shadow_fixed_number(b, f, l.lemmings, x, y,      c, true);
    Help::draw_shadow_fixed_number(b, f, l.required, x, y + 20, c, true);
    Help::draw_shadow_fixed_text  (b, f, cl.c_str(), x, y + 40, c, true);

    //////////////////////////////////////
    // Besten frueheren Erfolg anzeigen //
    //////////////////////////////////////

    const Result& result = Globals::user.get_level_result(filename);

    if (result.lemmings_saved >= l.required) {
        // >= wird auch in api/list_lev.cpp verwendet fuer Haken-Darstellung
        if (result.built >= l.built) {
            draw_text(Language::browser_info_result_saved, but_x, y +  80);
            draw_text(Language::browser_info_result_skills,but_x, y + 100);
            draw_text(Language::browser_info_result_time,  but_x, y + 120);

            Help::draw_shadow_fixed_number(
             b, f, result.lemmings_saved, x, y +  80, c, true);
            Help::draw_shadow_fixed_number(
             b, f, result.skills_used,    x, y + 100, c, true);
            Help::draw_shadow_fixed_updates_used(
             b, f, result.updates_used,   x, y + 120, c, true);
        }
        else {
            draw_text(Language::browser_info_result_saved, but_x, y +  80);
            draw_text(Language::browser_info_result_old_1, but_x, y + 100);
            draw_text(Language::browser_info_result_old_2, but_x, y + 120);

            Help::draw_shadow_fixed_number(
             b, f, result.lemmings_saved, x, y +  80, c, true);
        }

    }
}

void SingleBrowser::on_file_select(const std::string& filename)
{
    if (hardware.get_mrh() || button_clicked(button_edit)) {
        set_exit_with(EXIT_WITH_EDIT);
        Globals::user.single_last_dir  = get_current_dir();
        Globals::user.single_last_file = get_current_file();
    }
    else {
        Level l(filename);
        if (l.get_status() == Level::GOOD) {
            set_exit_with(EXIT_WITH_OKAY);
            Globals::user.single_last_dir  = get_current_dir();
            Globals::user.single_last_file = get_current_file();
        }
    }
}










NetworkBrowser::NetworkBrowser()
:
    BrowserBig(Language::browser_network_title,
                        Globals::dir_levels,
                        Globals::user.network_last_dir,
                        Globals::user.network_last_file)
{
    set_button_play_text(Language::ok);
    on_file_highlight(get_current_dir() + get_current_file());
}



NetworkBrowser::~NetworkBrowser()
{
}



void NetworkBrowser::calculate()
{
    BrowserBig::calculate();
    // Den normalen Dialog-Siedler-2-Rechtsklick realisieren:
    // on_file_select wird nochmal differenzieren, ob es per Rechtsklick
    // oder sonstwie aufgerufen wurde, falls der Level doof ist.
    if (hardware.get_mr()) {
        on_file_select(get_current_dir() + get_current_file());
    }
}



void NetworkBrowser::on_file_highlight(const std::string& filename)
{
    Level l(filename);
    clear_info();
    // Wir gucken dies hier nach und nicht in BrowserBig, weil dort nicht
    // sicher ist, ob ein Level da sein soll oder ein Replay, etwa. Nur auf
    // beliebige Dateien koennen wir nicht kotnrollieren, weil sonst auch
    // das Basisverzeichnis eine gueltige Datei ist.
    if (l.get_status() == Level::BAD_FILE_NOT_FOUND) clear_preview();
    else set_preview(l);

    if (!l.get_good()) return;

    Torbit&   b = get_torbit();
    FONT*     f = font_med;
    const int x = but_x + but_xl;
    const int y = get_info_y();
    const int c = color[COL_WHITE];

    int hatches = l.pos[Object::HATCH].size();
    int goals   = l.pos[Object::GOAL] .size();
    int skills  = 0;
    for (unsigned i = 0; i != 8; ++i) skills += l.skill[i].nr;

    std::string cl;
    if (l.seconds > 0) {
        cl += l.seconds / 60;
        cl += ':';
        cl += l.seconds % 60 / 10;
        cl += l.seconds % 10;
    }
    else cl = Language::browser_info_clock_none;

    draw_text(Language::browser_info_hatches,  but_x, y      );
    draw_text(Language::browser_info_goals,    but_x, y +  20);
    draw_text(Language::browser_info_lemmings, but_x, y +  60);
    draw_text(Language::browser_info_skills,   but_x, y +  80);
    draw_text(Language::browser_info_clock_2,  but_x, y + 100);

    Help::draw_shadow_fixed_number(b, f, hatches,    x, y,       c, true);
    Help::draw_shadow_fixed_number(b, f, goals,      x, y +  20, c, true);
    Help::draw_shadow_fixed_number(b, f, l.lemmings, x, y +  60, c, true);
    Help::draw_shadow_fixed_number(b, f, skills,     x, y +  80, c, true);
    Help::draw_shadow_fixed_text  (b, f, cl.c_str(), x, y + 100, c, true);
}



void NetworkBrowser::on_file_select(const std::string& filename)
{
    Level l(filename);
    if (l.get_status() == Level::GOOD) {
        set_exit_with(EXIT_WITH_OKAY);
        Globals::user.network_last_dir  = get_current_dir();
        Globals::user.network_last_file = get_current_file();
    }
    else if (hardware.get_mr()) {
        set_exit_with(EXIT_WITH_EXIT);
    }
}

} // Ende Namensraum Api
