/*
 * menu/lobby.cpp
 *
 */

#include "lobby.h"

#include "../network/network.h"
#include "../other/console.h"

namespace Api {

const unsigned Lobby::dir_list_xl  = 120;
const unsigned Lobby::lev_list_xl  = 300;
const unsigned Lobby::any_list_yl  = 160;
const unsigned Lobby::preview_xl   = 380;
const unsigned Lobby::medium_row_y = 220;

const unsigned Lobby::console_y    = 260;
const unsigned Lobby::console_xl   = 600;
const unsigned Lobby::console_yl   = 160;
const unsigned Lobby::chat_type_x  =  80;
const unsigned Lobby::chat_type_y  = console_y + console_yl + 20;

const unsigned Lobby::start_but_x  = 160;
const unsigned Lobby::start_but_xl = LEMSCR_X - 2*start_but_x;
const unsigned Lobby::start_but_y  =  80;
const unsigned Lobby::start_but_ys =  30;

const unsigned Lobby::button_xl    = 140;

Lobby::Lobby()
:
    Window(*osd, 0, 0, LEMSCR_X, LEMSCR_Y, Language::win_lobby_title),
    exit_with   (EXIT_WITH_NOTHING),
    browser     (0),

    preview     (get_torbit(), LEMSCR_X - 20 - preview_xl, 40,
                               preview_xl, 160), // 160 ist loblist_y_len
    room_list   (get_torbit(), LEMSCR_X - 20 - preview_xl, 40,
                               preview_xl, 160),
    button_level(get_torbit(), LEMSCR_X - 20 - button_xl,
                               medium_row_y,   button_xl),
    button_ready(get_torbit(), 20,
                               medium_row_y,   button_xl),
    button_exit (get_torbit(), LEMSCR_X - 20 - button_xl,
                               chat_type_y,    button_xl),
    start_central(get_torbit(),start_but_x, start_but_y,                  start_but_xl),
    start_server(get_torbit(), start_but_x, start_but_y + 1*start_but_ys, start_but_xl),
    start_client(get_torbit(), start_but_x, start_but_y + 2*start_but_ys, start_but_xl/2),
    start_ip    (get_torbit(), LEMSCR_X/2,  start_but_y + 2*start_but_ys, start_but_xl/2),

    chat_frame  (get_torbit(), 20, console_y, console_xl, console_yl),
    chat_type   (get_torbit(), chat_type_x, chat_type_y,
                               LEMSCR_X - 40 - chat_type_x - button_xl),
    lobby_list  (get_torbit(), 20, 40)
{
    button_level .set_text(Language::win_lobby_select_level);
    button_ready .set_text(Language::win_lobby_ready);
    start_central.set_text(Language::win_lobby_start_central);
    start_server .set_text(Language::win_lobby_start_server);
    start_client .set_text(Language::win_lobby_start_client);

    button_exit  .set_hotkey(KEY_ESC);
    chat_type    .set_hotkey(Globals::user.key_chat);
    // button_ready has no hotkey; it checks for KEY_SPACE manually to avoid
    // firing while chat_type is active. The GUI code doesn't support such
    // a thing by itself yet.

    button_level .set_undraw_color(color[COL_API_M]);
    button_ready .set_undraw_color(color[COL_API_M]);
    start_central.set_undraw_color(color[COL_API_M]);
    start_server .set_undraw_color(color[COL_API_M]);
    start_client .set_undraw_color(color[COL_API_M]);
    start_ip     .set_undraw_color(color[COL_API_M]);
    start_ip     .set_scroll();
    start_ip     .set_text(Globals::ip_last_used);
    chat_frame   .set_undraw_color(color[COL_API_M]);
    chat_type    .set_undraw_color(color[COL_API_M]);
    chat_type    .set_scroll();

    // Don't show the GARDEN style by placing it off the screen
    for (unsigned i = Lem::GARDEN; i < Lem::STYLE_MAX; ++i) {
        button_color.push_back(LobbyColorButton(get_torbit(),
         (i <= Lem::GARDEN ? 10000 :
         180 + (i-1) % 2 * LobbyColorButton::this_xl),
          40 + (i-1) / 2 * LobbyColorButton::this_yl,
         static_cast <Lem::Style> (i)));
    }

    // Coming back into the menu after a game
    if (Network::get_started()) {
        lobby_list.set_data(Network::get_player_data());
        if (Network::get_room() == 0)
         room_list.set_room_data(Network::get_room_data());
        else preview.set_level(Network::get_level());

        button_ready.set_on(Network::get_ready());
        for (unsigned i = 0; i < button_color.size(); ++i) {
            if (i == (unsigned)Network::get_style()) button_color[i].set_on();
            else                                     button_color[i].set_off();
        }
    }
    // Or just about to start a network session
    else {
        Console::clear();
        Console::push_back(Language::net_chat_welcome_1);
        Console::push_back(Language::net_chat_welcome_2);
    }
}



Lobby::~Lobby()
{
    if (browser) delete browser;
    // Nicht Shutdown, das waere Unfug bei Spielstart!
}





void Lobby::calculate()
{
    Network::calculate();

    // Unterfenster aktiv?
    if (browser) {
        browser->calculate();
        browser->set_draw_required(); // Damit Buttons eingedrueckt werden
        if (browser    ->get_exit_with() == NetworkBrowser::EXIT_WITH_OKAY
         || browser    ->get_exit_with() == NetworkBrowser::EXIT_WITH_EXIT) {
            if (browser->get_exit_with() == NetworkBrowser::EXIT_WITH_OKAY) {
                std::string s = Globals::user.network_last_dir
                              + Globals::user.network_last_file;
                Network::set_level(s);
                // Und diesen Level auch sofort anzeigen, selbst wenn er
                // noch nicht offiziell angekommen ist. So wird das Flackern
                // bzw. das verzoegerte Akzeptieren der Vorschau unterbunden,
                // die sonst erst den Lag abwarten muesste.
                // Wir nehmen dabei den (nicht vorhandenen ;-)
                // Geschwindigkeitsnachteil in Kauf, der beim Neuladen des
                // Levels aus der Datei entsteht.
                Level l(s);
                preview.set_level(l);
            }
            delete browser;
            browser = 0;
        }
        return;
    }



    // Wir sind in der Lobby, nicht in der Levelauswahl - also ganz normal
    // die verschiedenen Faelle abarbeiten, je nach Netzwerkzustand
    if (!Network::get_started()) {
        if (button_clicked(start_central)) {
            Network::start_as_client(Globals::ip_central_server);
            button_exit.set_text(Language::cancel);
        }
        else if (button_clicked(start_server)) {
            Network::start_as_server();
            button_exit.set_text(Language::cancel);
        }
        else if (button_clicked(start_client)) {
            Network::start_as_client(start_ip.get_text());
            button_exit.set_text(Language::cancel);
        }
        else if (button_clicked(button_exit)
         || hardware.get_mr()) {
            exit_with = EXIT_WITH_EXIT;
        }
        else if (start_ip.get_on() && hardware.key_enter_once()) {
            start_ip.set_off();
            // Do the same as the button "start as client"
            Network::start_as_client(start_ip.get_text());
            button_exit.set_text(Language::cancel);
        }
        start_ip.calculate();
    }
    else if (button_clicked(button_exit)
     || (button_exit.get_hotkey() == KEY_ESC && hardware.get_mr())) {
        if (Network::get_room() == 0) Network::shutdown();
        else Network::set_room(0);
    }



    // Inside any room
    if (Network::get_people_in_room() > 0) {
        button_exit.set_hotkey(KEY_ESC);

        for (unsigned i = 0; i < button_color.size(); ++i)
         if (button_color[i].clicked()) {
            Network::set_style((Lem::Style) i);
            // Already select newest button to be pressed in, so that
            // there is no flicker because the packet takes time
            for (std::vector <LobbyColorButton> ::iterator
             itr = button_color.begin(); itr != button_color.end(); ++itr)
             itr->set_off();
            button_color[i].set_on();
        }
    }

    // Inside any room with other people
    if (Network::get_people_in_room() > 1) {
        button_exit.set_hotkey();

        // Chatnachricht senden
        if (chat_type.get_on() && !chat_type.get_text().empty()
         && hardware.key_enter_once()) {
            Network::send_chat_message(chat_type.get_text());
            chat_type.set_text(Globals::empty_string);
            chat_type.calculate();
            // Wieder anschalten, da calculate() es ausschalten kann
            chat_type.set_on();
        }
        else chat_type.calculate();
    }

    // Inside the lobby
    if (Network::get_people_in_room() > 0 && Network::get_room() == 0) {
        set_subtitle(Globals::empty_string);
        button_exit.set_text(Language::win_lobby_exit);
        if (room_list.clicked(get_x(), get_y())) {
            if (room_list.get_room_clicked() == 0)
                 Network::create_room();
            else Network::set_room(room_list.get_room_clicked());
        }
        if (!Network::get_server()) Globals::ip_last_used=start_ip.get_text();
    }

    // Inside a game room
    if (Network::get_people_in_room() > 0 && Network::get_room() > 0) {
        set_subtitle(Language::win_lobby_room_number + Network::get_room());
        button_exit.set_text(Language::win_lobby_room_leave);

        if (button_clicked(button_level)) {
            if (Network::get_ready()) {
                button_ready.set_off();
                Network::set_ready(false);
            }
            browser = new NetworkBrowser;
        }
        if (Network::get_level_change()) {
            preview.set_level(Network::get_level());
        }
        // Additionally: Inside a game room with other people
        if (Network::get_people_in_room() > 1) {
            if (button_clicked(button_ready) ||
             hardware.key_once(KEY_SPACE) && !chat_type.get_on()) {
                button_ready.set_on(!button_ready.get_on()
                                  && preview.get_status() == Level::GOOD);
                Network::set_ready(  button_ready.get_on());
            }
            if (Network::get_game_start()) exit_with = EXIT_WITH_PLAY;
        }
    }

    // Alone in any room
    if (Network::get_people_in_room() < 2) {
        button_exit.set_hotkey(KEY_ESC);
    }

    // Do this always
    if (Network::get_player_data_change()) {
        lobby_list.set_data(Network::get_player_data());
        button_ready.set_on(Network::get_ready());
        for (unsigned i = 0; i < button_color.size(); ++i) {
            if (i == (unsigned)Network::get_style()) button_color[i].set_on();
            else                                     button_color[i].set_off();
        }
    }
    if (Network::get_room_data_change()) {
        room_list.set_room_data(Network::get_room_data());
    }
}



void Lobby::draw()
{
    if (browser) {
        browser->draw();
    }
    else Window::draw();
}



void Lobby::draw_self()
{
    const unsigned peers = Network::get_people_in_room();

    if (peers > 0) {
        if (Network::get_room() <= 0) {
            room_list.draw();
        }
        else {
            preview     .draw();
            button_level.draw();
        }
        for (unsigned i = 0; i < button_color.size(); ++i) {
            button_color[i].draw();
        }
    }
    else {
        // This is a kludge.
        // I don't know why the player list remains there if the user
        // quits the network.
        rectfill(get_torbit().get_al_bitmap(), 2, 20, LEMSCR_X - 3,
         chat_frame.get_y() - 1, color[COL_API_M]);
    }

    if (!Network::get_started()) {
        button_exit.set_text(Language::back);
        start_central.draw();
        start_server .draw();
        start_client .draw();
        start_ip     .draw();
    }
    // Verbunden und es sind noch andere Leute da
    else if (peers > 1) {
        if (Network::get_room() != 0) button_ready.draw();
        draw_text(Language::win_lobby_chat, 23, chat_type_y);
        chat_type.draw();
    }

    button_exit.draw();
    chat_frame.undraw();
    chat_frame.draw();
    lobby_list.draw(); // Die regelt anhand der Spielerzahlen alles selbst.

    // Den Text in den Chat zeichnen
    int count = 0;
    const std::list <Console::Line>& li = Console::get_lines_all();
    for (Console::LiIt i = li.begin(); i != li.end(); ++i)
     // + 3 sind's auch in api/button_t.cpp
     draw_text(i->text, chat_frame.get_x() + 3,
               chat_frame.get_y() + 3 + 19*count++,
               i->white ? color[COL_TEXT_ON] : color[COL_TEXT]);
}



void Lobby::undraw_self()
{
    if (browser) {
        browser->undraw();
        return;
    }

    for (unsigned i = 0; i<button_color.size(); ++i) button_color[i].undraw();
    preview      .undraw();
    room_list    .undraw();
    lobby_list   .undraw();
    button_level .undraw();
    button_ready .undraw();
    start_central.undraw();
    start_server .undraw();
    start_client .undraw();
    start_ip     .undraw();
    chat_type    .undraw();

    // Text "Chat:" entfernen, weil diese Leere unten auch dann
    // angezeigt wird, wenn alle Clients gehen und man wieder als Server
    // alleine ist
    rectfill(get_torbit().get_al_bitmap(), 2, chat_type_y, chat_type_x - 1,
     LEMSCR_Y - 2, color[COL_API_M]);
}

}
// Ende Namensraum Api
