/*
 * menu/lobby_li.cpp
 *
 */

#include "lobby_li.h"

#include "../graphic/gra_lib.h"
#include "../network/network.h"

namespace Api {

const unsigned LobbyList::LobbyDataButton::this_xl(140);
const unsigned LobbyList::LobbyDataButton::this_yl( 20);
const unsigned LobbyList::LobbyDataButton::text_xl(100);

LobbyList::LobbyDataButton::LobbyDataButton(Torbit& b, const int x,const int y)
:
    Button(b, x, y, this_xl, this_yl),
    filled(false),
    spec  (false),
    ready (false),
    name  (),
    style (Lem::GARDEN),
    cutbit(&GraLib::get_lemming(Lem::GARDEN)),
    cutchk(&GraLib::get(Globals::file_bitmap_checkbox))
{
}

LobbyList::LobbyDataButton::~LobbyDataButton()
{
}

void LobbyList::LobbyDataButton::read_from_pldatcit(
    const Network::PlDatCIt& itr
) {
    filled = true;
    spec   = itr->spec;
    ready  = itr->ready;
    style  = static_cast <const Lem::Style> (itr->style);
    cutbit = &GraLib::get_lemming(style);
    name   = itr->name;
    while ((unsigned) text_length(font_med, name.c_str()) > text_xl)
     name.erase(--name.end());
    set_draw_required();
}

void LobbyList::LobbyDataButton::draw_self()
{
    if (!filled) return;
    Button::draw_self();
    cutbit->draw(get_ground(), get_x(), get_y(), 12, Lem::WALKER-1);
    Help::draw_shadow_centered_text(get_ground(), font_med, name.c_str(),
     get_x() + this_xl/2, get_y(), color[COL_TEXT]);
    cutchk->draw(get_ground(), get_x()+get_xl()-20, get_y(), ready, 0);
}

void LobbyList::LobbyDataButton::undraw_self()
{
    if (filled) set_undraw_color(0);
    else        set_undraw_color(color[COL_API_M]);
}



// ############################################################################
// ############################################################################
// ############################################################################



LobbyList::LobbyList(Torbit& b, const int lx, const int ly)
:
    Frame(b, lx, ly, LobbyDataButton::this_xl,
       buttons_max * LobbyDataButton::this_yl)
{
    for (unsigned y = 0; y < buttons_max; ++y)
    button.push_back(LobbyDataButton(b, lx,
     ly + y * LobbyDataButton::this_yl));
}

LobbyList::~LobbyList()
{
}

void LobbyList::set_data(const std::list <PlayerData>& pdli)
{
    for (size_t i = 0; i < button.size(); ++i) {
        button[i].filled = false;
        button[i].set_draw_required();
    }
    unsigned count = 0;
    // Active players
    for (Network::PlDatCIt itr = pdli.begin(); itr != pdli.end(); ++itr)
     if (!itr->spec && count < buttons_max)
     button[count++].read_from_pldatcit(itr);
    // Spectators
    for (Network::PlDatCIt itr = pdli.begin(); itr != pdli.end(); ++itr)
     if (itr->spec && count < buttons_max)
     button[count++].read_from_pldatcit(itr);

    if (count > 0) set_undraw_color(0);
    else           set_undraw_color(color[COL_API_M]);
    set_draw_required();
}

void LobbyList::draw_self()
{
    if (Network::get_people_in_room() > 0) Frame::draw_self();
    for (std::vector <LobbyDataButton> ::iterator i = button.begin();
     i != button.end(); ++i) i->draw();
}

void LobbyList::undraw_self()
{
    for (std::vector <LobbyDataButton> ::iterator i = button.begin();
     i != button.end(); ++i) i->undraw();
    Frame::undraw_self();
}



// ############################################################################
// ############################################################################
// ############################################################################



const unsigned LobbyColorButton::this_xl(20);
const unsigned LobbyColorButton::this_yl(32);

LobbyColorButton::LobbyColorButton(
    Torbit&   b,
    const int nx,
    const int ny,
    const Lem::Style st)
:
    Button(b, nx, ny, this_xl, this_yl),
    lem   (GraLib::get_lemming(st), b, get_x() - 7, get_y() - 3)
{
    lem.set_y_frame(Lem::WALKER - 1);
    set_undraw_color(color[COL_API_M]);
}

LobbyColorButton::~LobbyColorButton()
{
}

void LobbyColorButton::draw_self()
{
    Button::draw_self();
    lem.set_x_frame(13);
    lem.draw();
    lem.set_x_frame(3);
    lem.draw();
}






// Ende Namensraum Api
}
