/*
 * menu/lobby_li.h
 *
 * Die Liste mit den derzeit im Netzwerk anwesenden Spielern
 *
 * Derzeit ist sie fuer acht Spieler angelegt, mehr geht nicht. Die Liste
 * versorgt sich auch nicht selbst aus dem Netzwerkobjekt, sondern muss von
 * der Lobby bei Bedarf mit neuen Daten gefuettert werden.
 *
 */

#pragma once

#include <string>

#include "../api/frame.h"
#include "../api/button.h"
#include "../graphic/graphic.h"
#include "../lemming/lem_enum.h"
#include "../network/network.h"

namespace Api {
class LobbyList : public Frame {

private:

    static const unsigned buttons_max = 8;

    struct LobbyDataButton : public Button {
        static const unsigned this_xl;
        static const unsigned this_yl;
        static const unsigned text_xl;
        bool           filled;
        bool           spec;
        bool           ready;
        std::string    name;
        Lem::Style     style;
        const Cutbit*  cutbit;
        const Cutbit*  cutchk;
        LobbyDataButton(Torbit& = *osd, const int = 0, const int = 0);
        virtual ~LobbyDataButton();
        void read_from_pldatcit(const Network::PlDatCIt&);
    protected:
        virtual void   draw_self();
        virtual void undraw_self();
    };

    std::vector <LobbyDataButton> button;

    // Kopierverbot
    LobbyList       (const LobbyList&);
    void operator = (const LobbyList&);

public:

             LobbyList(Torbit& = *osd, const int = 0, const int = 0);
    virtual ~LobbyList();

    void set_data(const std::list <PlayerData>&);

protected:

    virtual void draw_self();
    virtual void undraw_self();

};



class LobbyColorButton : public Button {
    Graphic lem;
public:
    static const unsigned this_xl;
    static const unsigned this_yl;
    LobbyColorButton(Torbit&, const int = 0, const int = 0,
                     const Lem::Style = Lem::GARDEN);
    virtual ~LobbyColorButton();
protected:
    virtual void draw_self();
};

}
