/*
 * menu/lobby_ro.cpp
 *
 */

#include "lobby_ro.h"

#include "../other/language.h"

namespace Api {

const int RoomList::button_yl(20);



RoomList::RoomList(
    Torbit& ground,
    int x,
    int y,
    int xl,
    int yl
) :
    Frame  (ground, x, y, xl, yl),
    last_button_is_create(false),
    room_clicked(0)
{
}



void RoomList::set_room_data(const std::set <NetClient::RoomData>& rd)
{
    buttons.clear();
    button_rooms.clear();
    last_button_is_create = false;
    // Make one button for each room
    for (Network::RoomCIt itr = rd.begin(); itr != rd.end(); ++itr) {
        // don't display the lobby as a room to switch into
        if (itr->number == 0) continue;
        if (((int) buttons.size() + 1) * button_yl > get_yl()) break;
        button_rooms.push_back(itr->number);
        buttons.push_back(TextButton(get_ground(),
         get_x(), get_y() + buttons.size() * button_yl, get_xl(), button_yl));
        std::string str = Language::win_lobby_room_number;
        str += (int) itr->number;
        str += ", ";
        str += (int) itr->members;
        str += " ";
        str += itr->members == 1 ? Language::win_lobby_room_player
                                 : Language::win_lobby_room_players;
        buttons.back().set_text(str);
        buttons.back().set_undraw_color(color[COL_API_M]);
    }
    // Room creation button
    if (((int) buttons.size() + 1) * button_yl <= (int) get_yl()) {
        button_rooms.push_back(0);
        buttons.push_back(TextButton(get_ground(),
         get_x(), get_y() + buttons.size() * button_yl, get_xl(), button_yl));
        buttons.back().set_text(Language::win_lobby_room_create);
        buttons.back().set_undraw_color(color[COL_API_M]);
        last_button_is_create = true;
    }
    set_draw_required();
}



void RoomList::clear()
{
    buttons.clear();
    set_draw_required();
}



bool RoomList::clicked(int x_offset, int y_offset)
{
    // This call here is a kludge. I might consider to rewrite the Api code.
    // It's not suited to hide elements and draw others in the same place.
    set_draw_required();

    for (unsigned i = 0; i < buttons.size(); ++i) {
        if (buttons[i].clicked(x_offset, y_offset)) {
            room_clicked = button_rooms[i];
            return true;
        }
    }
    // no button got clicked on
    return false;
}



void RoomList::draw_self()
{
    Frame::draw_self();

    for (std::vector <TextButton> ::iterator
     itr = buttons.begin(); itr != buttons.end(); ++itr) {
        itr->draw();
    }

}

}
// end of namespace Api
