/*
 * menu/main_men.cpp
 *
 */

#include "main_men.h"

#include "../other/language.h"



MainMenu::MainMenu()
:
    window(new Api::MainMenuWindow())
{
    osd->clear_to_color(color[COL_BLACK]);
}

MainMenu::~MainMenu()
{
    delete window;
}



void MainMenu::undraw()
{
    window->undraw();
}

void MainMenu::calculate()
{
    window->calculate();
}

void MainMenu::draw()
{
    window->draw();
}



// ############################################################################
// ############################################################################
// ############################################################################



namespace Api {

const unsigned MainMenuWindow::button_xl  = 200;
const unsigned MainMenuWindow::button_yl  =  40;
const unsigned MainMenuWindow::button_y_spacing =  20;

const unsigned MainMenuWindow::window_xl  = button_xl
                                                  + 2 * button_y_spacing;
const unsigned MainMenuWindow::window_yl  =   4 * button_yl
                                                  + 6 * button_y_spacing
                                                  + 60; // Fuer Infotexte

MainMenuWindow::MainMenuWindow()
:
    Window(*osd, LEMSCR_X/2 - window_xl/2, LEMSCR_Y/2 - window_yl/2,
     window_xl, window_yl, Language::lemmings_plus_plus),

    exit_with(SUBMENU_NOTHING),

    single (get_torbit(), 20, 40 + 0 * (button_yl + button_y_spacing),
     button_xl, button_yl, Language::browser_single_title),
    network(get_torbit(), 20, 40 + 1 * (button_yl + button_y_spacing),
     button_xl, button_yl, Language::win_lobby_title),
    replay (get_torbit(), 20, 40 + 2 * (button_yl + button_y_spacing),
     button_xl/2 - 10, button_yl, Language::browser_replay_title),
    options(get_torbit(),130, 40 + 2 * (button_yl + button_y_spacing),
     button_xl/2 - 10, button_yl, Language::option_title),
    exit   (get_torbit(), 20, 40 + 3 * (button_yl + button_y_spacing),
     button_xl, button_yl, Language::exit)
{
    exit.set_hotkey(KEY_ESC);
    const std::string  v = Language::lemmings_version + " "
                         + Help::version_to_string(Globals::version);
    const std::string& w = Language::lemmings_website;
    draw_centered_text(v.c_str(), window_xl / 2, window_yl - 60);
    draw_centered_text(w.c_str(), window_xl / 2, window_yl - 40);
}

MainMenuWindow::~MainMenuWindow()
{
}



void MainMenuWindow::calculate()
{
    if      (single .clicked(get_x(), get_y())) exit_with = SUBMENU_SINGLE;
    else if (network.clicked(get_x(), get_y())) exit_with = SUBMENU_NETWORK;
    else if (replay .clicked(get_x(), get_y())) exit_with = SUBMENU_REPLAY;
    else if (options.clicked(get_x(), get_y())) exit_with = SUBMENU_OPTIONS;
    else if (exit   .clicked(get_x(), get_y())) exit_with = SUBMENU_EXIT;
}



void MainMenuWindow::draw_self()
{
    single .draw();
    network.draw();
    replay .draw();
    options.draw();
    exit   .draw();
}

} // Ende Namensraum Api
