/*
 * menu.cpp
 *
 */

#include "menu.h"

#include "../graphic/gra_lib.h"

Menu::Menu(SubMenu sm)
:
    exit_with          (NOTHING),
    main               (0),
    single             (sm == SUBMENU_SINGLE  ? new Api::SingleBrowser() : 0),
    lobby              (sm == SUBMENU_NETWORK ? new Api::Lobby()         : 0),
    replay             (sm == SUBMENU_REPLAY  ? new Api::ReplayBrowser() : 0),
    options            (0),
    new_player         (0),
    language           (0),

    mouse_cursor_offset(15),
    mouse_cursor(GraLib::get(Globals::file_bitmap_mouse), *osd)
{
}

Menu::~Menu() {
    if (main)       delete main;
    if (single)     delete single;
    if (lobby)      delete lobby;
    if (replay)     delete replay;
    if (options)    delete options;
    if (new_player) delete new_player;
    if (language)   delete language;
}



// ############################################################################
// ############################################################################
// ############################################################################



void Menu::undraw()
{
    mouse_cursor.undraw();
    if (main)       main      ->undraw();
    if (single)     single    ->undraw();
    if (lobby)      lobby     ->undraw();
    if (replay)     replay    ->undraw();
    if (options)    options   ->undraw();
    if (new_player) new_player->undraw();
    if (language)   language  ->undraw();
}



void Menu::calculate()
{
    if (main) {
        main->calculate();
        switch (main->get_exit_with()) {

        case SUBMENU_NOTHING:
            break;

        case SUBMENU_SINGLE:
            delete main;
            main = 0;
            single = new Api::SingleBrowser();
            break;

        case SUBMENU_NETWORK:
            delete main;
            main = 0;
            lobby = new Api::Lobby();
            break;

        case SUBMENU_REPLAY:
            delete main;
            main = 0;
            replay = new Api::ReplayBrowser();
            break;

        case SUBMENU_OPTIONS:
            delete main;
            main = 0;
            options = new Api::OptionMenu();
            break;

        case SUBMENU_EXIT:
            delete main;
            main = 0;
            exit_with = EXIT_PROGRAM;
            break;
        }
    }

    else if (single) {
        single->calculate();
        if (single->get_exit_with() != Api::SingleBrowser::EXIT_WITH_NOTHING) {
            switch (single->get_exit_with()) {
            case Api::SingleBrowser::EXIT_WITH_NOTHING:
                break;

            case Api::SingleBrowser::EXIT_WITH_OKAY:
                exit_with = START_LEVEL;
                break;

            case Api::SingleBrowser::EXIT_WITH_EDIT:
                exit_with = START_EDITOR;
                break;

            case Api::SingleBrowser::EXIT_WITH_EXIT:
                delete single;
                single = 0;
                main = new MainMenu();
                break;
            }
        }
    }

    else if (lobby) {
        lobby->calculate();
        if (lobby->get_exit_with() == Api::Lobby::EXIT_WITH_PLAY) {
            delete lobby;
            lobby = 0;
            exit_with = START_LEVEL;
        }
        else if (lobby->get_exit_with() == Api::Lobby::EXIT_WITH_EXIT) {
            delete lobby;
            lobby = 0;
            main = new MainMenu();
        }
    }

    else if (replay) {
        replay->calculate();
        if (replay->get_exit_with() == Api::ReplayBrowser::EXIT_WITH_OKAY) {
            delete replay;
            replay = 0;
            exit_with = START_REPLAY;
            // Replay-Last-Dir und -File hat der Replaybrowser gesetzt
        }
        else if (replay->get_exit_with()==Api::ReplayBrowser::EXIT_WITH_EXIT) {
            delete replay;
            replay = 0;
            main = new MainMenu();
        }
    }

    else if (options) {
        options->calculate();
        if (options->get_exit()) {
            delete options;
            options = 0;
            main = new MainMenu();
        }
    }

    else if (new_player) {
        new_player->calculate();
        if (new_player->get_exit()) {
            delete new_player;
            new_player = 0;
            main = new MainMenu();
        }
    }

    else if (language) {
        language->calculate();
        if (language->get_exit()) {
            Language::set(language->get_exit_with());
            delete language;
            language = 0;
            if (Globals::user_name_ask
             || Globals::user.get_name().empty()) {
                 new_player = new Api::NewPlayerMenu();
            }
            else main = new MainMenu();
        }
    }



    // Kein Fenster offen?
    else {
        if (Globals::user.language == Language::NONE
         || Globals::user.language >= Language::MAX) {
            language   = new Api::LanguageMenu();
        }
        else if (Globals::user_name_ask
         ||      Globals::user.get_name().empty()) {
            new_player = new Api::NewPlayerMenu();
        }
        else main      = new MainMenu();
    }
}



void Menu::draw()
{
    if (main)       main      ->draw();
    if (single)     single    ->draw();
    if (lobby)      lobby     ->draw();
    if (replay)     replay    ->draw();
    if (options)    options   ->draw();
    if (new_player) new_player->draw();
    if (language)   language  ->draw();

    if (!new_player) {
        mouse_cursor.set_x(hardware.get_mx()-mouse_cursor_offset);
        mouse_cursor.set_y(hardware.get_my()-mouse_cursor_offset);
        mouse_cursor.draw();
    }

    blit_to_screen(osd->get_al_bitmap());
}
