/*
 * menu/options.cpp
 *
 */

#include "options.h"

#include "../other/language.h"

namespace Api {

const unsigned OptionMenu::check_x   =  20;
const unsigned OptionMenu::check_nx  =  60;
const unsigned OptionMenu::other_x   = 300;
const unsigned OptionMenu::other_nx  = 460;

const unsigned OptionMenu::button_xl = 140;
const unsigned OptionMenu::key_xl    =  70;
const unsigned OptionMenu::frame_y   = 320;
const unsigned OptionMenu::frame_yl  = 100;

const unsigned OptionMenu::key_b1    =  20;
const unsigned OptionMenu::key_t1    =  40 + OptionMenu::key_xl;
const unsigned OptionMenu::key_b2    = 220;
const unsigned OptionMenu::key_t2    = 240 + OptionMenu::key_xl;
const unsigned OptionMenu::key_b3    = 420;
const unsigned OptionMenu::key_t3    = 440 + OptionMenu::key_xl;



void OptionMenu::add_to_pointers_hotkeys(KeyButton* tb)
{
    tb->set_x(hotkey_add_counter < 10 ? key_b2 : key_b3);
    tb->set_y(100 + (hotkey_add_counter % 10) * 30);
    pointers[GROUP_HOTKEYS].push_back(tb);
    ++hotkey_add_counter;
}


OptionMenu::OptionMenu()
:
    Window(*osd, 0, 0, LEMSCR_X, LEMSCR_Y, Language::option_title),
    exit                  (false),
    hotkey_add_counter    (0),

    button_okay           (get_torbit(), LEMSCR_X/2 - 150, 440, button_xl),
    button_cancel         (get_torbit(), LEMSCR_X/2 +  10, 440, button_xl),
    pointers              (GROUP_MAX),

    user_name             (get_torbit(), other_x, 100, button_xl),
    user_name_ask         (get_torbit(), check_x, 100),
    language              (get_torbit(), other_x, 130, button_xl),
    language_nr           (Globals::user.language),

    mouse_speed           (get_torbit(), other_x, 100, button_xl, 2, 1, 30),
    scroll_speed          (get_torbit(), other_x, 130, button_xl, 2, 1, 20),
    scroll_border         (get_torbit(), check_x, 100),
    scroll_right          (get_torbit(), check_x, 130),
    scroll_torus_x        (get_torbit(), other_x, 160, button_xl),
    scroll_torus_y        (get_torbit(), other_x, 190, button_xl),
    multiple_builders     (get_torbit(), check_x, 190),

    key_force_left          (get_torbit(), key_b1,  100, key_xl),
    key_force_right         (get_torbit(), key_b1,  120, key_xl),
    key_rate_minus        (get_torbit(), key_b1,  150, key_xl),
    key_rate_plus         (get_torbit(), key_b1,  170, key_xl),
    key_pause             (get_torbit(), key_b1,  200, key_xl),
    key_speed_slow        (get_torbit(), key_b1,  220, key_xl),
    key_speed_fast        (get_torbit(), key_b1,  240, key_xl),
    key_speed_turbo       (get_torbit(), key_b1,  260, key_xl),
    key_restart           (get_torbit(), key_b1,  290, key_xl),
    key_state_load        (get_torbit(), key_b1,  310, key_xl),
    key_state_save        (get_torbit(), key_b1,  330, key_xl),
    key_zoom              (get_torbit(), key_b1,  360, key_xl),
    key_nuke              (get_torbit(), key_b1,  390, key_xl), // placed later
    key_chat              (get_torbit(), key_b1,  390, key_xl),
    key_skill             (Lem::AC_MAX,
     KeyButton            (get_torbit(), 9999, 9999, key_xl)), // out of way

    screen_resolution_x   (get_torbit(), other_x, 100, button_xl/2),
    screen_resolution_y   (get_torbit(), 370,     100, button_xl/2),
    screen_windowed_x     (get_torbit(), other_x, 130, button_xl/2),
    screen_windowed_y     (get_torbit(), 370,     130, button_xl/2),
    screen_windowed       (get_torbit(), check_x, 130),
    screen_scaling        (get_torbit(), other_x, 190, button_xl),
    screen_border_colored (get_torbit(), check_x, 190),
    screen_vsync          (get_torbit(), check_x, 100),
    arrows_replay         (get_torbit(), check_x, 250),
    arrows_network        (get_torbit(), check_x, 280),
    debris_amount         (get_torbit(), other_x, 250, button_xl),
    debris_type           (get_torbit(), other_x, 280, button_xl),

    sound_load_driver     (get_torbit(), check_x, 100),
    sound_volume          (get_torbit(), other_x, 100, button_xl, 2, 0, 20)
{
    // Standardwerte fuer alle Checkboxes und Buttons
    button_okay           .set_text(Language::ok);
    button_cancel         .set_text(Language::cancel);
    button_okay           .set_hotkey(KEY_ENTER);
    button_cancel         .set_hotkey(KEY_ESC);

    for (size_t i = 0; i < GROUP_MAX; ++i) {
        TextButton temp(get_torbit(), i*124 + 20, 40, 104);
        temp.set_text(Language::option_group[i]);
        button_group.push_back(temp);
    }

    pointers[GROUP_GENERAL] .push_back(&user_name);
    pointers[GROUP_GENERAL] .push_back(&user_name_ask);
    pointers[GROUP_GENERAL] .push_back(&language);

    scroll_torus_x          .add_text   (Language::option_scroll_torus_never);
    scroll_torus_x          .add_text   (Language::option_scroll_torus_big);
    scroll_torus_x          .add_text   (Language::option_scroll_torus_always);
    scroll_torus_y          .add_text   (Language::option_scroll_torus_never);
    scroll_torus_y          .add_text   (Language::option_scroll_torus_big);
    scroll_torus_y          .add_text   (Language::option_scroll_torus_always);
    pointers[GROUP_CONTROLS].push_back(&scroll_border);
    pointers[GROUP_CONTROLS].push_back(&scroll_right);
    pointers[GROUP_CONTROLS].push_back(&mouse_speed);
    pointers[GROUP_CONTROLS].push_back(&scroll_speed);
    pointers[GROUP_CONTROLS].push_back(&scroll_torus_x);
    pointers[GROUP_CONTROLS].push_back(&scroll_torus_y);
    pointers[GROUP_CONTROLS].push_back(&multiple_builders);

    pointers[GROUP_HOTKEYS ].push_back(&key_force_left);
    pointers[GROUP_HOTKEYS ].push_back(&key_force_right);
    pointers[GROUP_HOTKEYS ].push_back(&key_rate_minus);
    pointers[GROUP_HOTKEYS ].push_back(&key_rate_plus);
    pointers[GROUP_HOTKEYS ].push_back(&key_pause);
    pointers[GROUP_HOTKEYS ].push_back(&key_speed_slow);
    pointers[GROUP_HOTKEYS ].push_back(&key_speed_fast);
    pointers[GROUP_HOTKEYS ].push_back(&key_speed_turbo);
    pointers[GROUP_HOTKEYS ].push_back(&key_restart);
    pointers[GROUP_HOTKEYS ].push_back(&key_state_load);
    pointers[GROUP_HOTKEYS ].push_back(&key_state_save);
    pointers[GROUP_HOTKEYS ].push_back(&key_zoom);
    pointers[GROUP_HOTKEYS ].push_back(&key_chat);
    add_to_pointers_hotkeys(&key_skill[Lem::WALKER]);
    add_to_pointers_hotkeys(&key_skill[Lem::JUMPER]);
    add_to_pointers_hotkeys(&key_skill[Lem::CLIMBER]);
    add_to_pointers_hotkeys(&key_skill[Lem::FLOATER]);
    add_to_pointers_hotkeys(&key_skill[Lem::EXPLODER]);
    add_to_pointers_hotkeys(&key_skill[Lem::BLOCKER]);
    add_to_pointers_hotkeys(&key_skill[Lem::BUILDER]);
    add_to_pointers_hotkeys(&key_skill[Lem::PLATFORMER]);
    add_to_pointers_hotkeys(&key_skill[Lem::BASHER]);
    add_to_pointers_hotkeys(&key_skill[Lem::MINER]);
    add_to_pointers_hotkeys(&key_skill[Lem::DIGGER]);
    // add_to_pointers_hotkeys(&key_skill[Lem::ROPER]);
    add_to_pointers_hotkeys(&key_nuke);

    screen_scaling          .add_text (Language::option_screen_scaling_stretch);
    screen_scaling          .add_text (Language::option_screen_scaling_aspect);
    screen_scaling          .add_text (Language::option_screen_scaling_integer);
    debris_amount           .add_text (Language::option_debris_amount_none);
    debris_amount           .add_text (Language::option_debris_amount_own);
    debris_amount           .add_text (Language::option_debris_amount_all);
    debris_type             .add_text (Language::option_debris_type_stars);
    debris_type             .add_text (Language::option_debris_type_pixels);
    pointers[GROUP_GRAPHICS].push_back(&screen_resolution_x);
    pointers[GROUP_GRAPHICS].push_back(&screen_resolution_y);
    pointers[GROUP_GRAPHICS].push_back(&screen_windowed_x);
    pointers[GROUP_GRAPHICS].push_back(&screen_windowed_y);
    pointers[GROUP_GRAPHICS].push_back(&screen_windowed);
    pointers[GROUP_GRAPHICS].push_back(&screen_scaling);
    pointers[GROUP_GRAPHICS].push_back(&screen_border_colored);
    pointers[GROUP_GRAPHICS].push_back(&screen_vsync);
    pointers[GROUP_GRAPHICS].push_back(&arrows_replay);
    pointers[GROUP_GRAPHICS].push_back(&arrows_network);
    pointers[GROUP_GRAPHICS].push_back(&debris_amount);
    pointers[GROUP_GRAPHICS].push_back(&debris_type);

    pointers[GROUP_SOUND]   .push_back(&sound_load_driver);
    pointers[GROUP_SOUND]   .push_back(&sound_volume);

    reset_elements();
    show_group(static_cast <OptionGroup> (Globals::user.option_group));
}



OptionMenu::~OptionMenu()
{
}



// Alle angezeigten Elemente auf den derzeit vom Spiel benutzten Wert setzen
// Dies ist der Wert aus Globals::? oder Globals::user.?, wobei
void OptionMenu::reset_elements()
{
    user_name            .set_text   (Globals::user.get_name());
    user_name_ask        .set_checked(Globals::user_name_ask);
    language             .set_text   (Language::language_name[language_nr]);

    scroll_border        .set_checked(Globals::user.scroll_border);
    scroll_right         .set_checked(Globals::user.scroll_right);
    mouse_speed          .set_number (Globals::user.mouse_speed);
    scroll_speed         .set_number (Globals::user.scroll_speed);
    scroll_torus_x       .set_number (Globals::user.scroll_torus_x);
    scroll_torus_y       .set_number (Globals::user.scroll_torus_y);
    multiple_builders    .set_checked(Globals::user.multiple_builders);

    key_force_left         .set_scancode(Globals::user.key_force_left);
    key_force_right        .set_scancode(Globals::user.key_force_right);
    key_rate_minus       .set_scancode(Globals::user.key_rate_minus);
    key_rate_plus        .set_scancode(Globals::user.key_rate_plus);
    key_pause            .set_scancode(Globals::user.key_pause);
    key_speed_slow       .set_scancode(Globals::user.key_speed_slow);
    key_speed_fast       .set_scancode(Globals::user.key_speed_fast);
    key_speed_turbo      .set_scancode(Globals::user.key_speed_turbo);
    key_restart          .set_scancode(Globals::user.key_restart);
    key_state_load       .set_scancode(Globals::user.key_state_load);
    key_state_save       .set_scancode(Globals::user.key_state_save);
    key_zoom             .set_scancode(Globals::user.key_zoom);
    key_nuke             .set_scancode(Globals::user.key_nuke);
    key_chat             .set_scancode(Globals::user.key_chat);
    for (size_t i = 0; i < key_skill.size(); ++i)
     key_skill[i]        .set_scancode(Globals::user.key_skill[i]);

    std::string s1; s1 += Globals::screen_resolution_x;
    std::string s2; s2 += Globals::screen_resolution_y;
    screen_resolution_x  .set_text(s1);
    screen_resolution_y  .set_text(s2);
    s1.clear();     s1 += Globals::screen_windowed_x;
    s2.clear();     s2 += Globals::screen_windowed_y;
    screen_windowed_x    .set_text(s1);
    screen_windowed_y    .set_text(s2);

    screen_windowed      .set_checked(Globals::user.screen_windowed);
    screen_scaling       .set_number (Globals::user.screen_scaling);
    screen_border_colored.set_checked(Globals::user.screen_border_colored);
    screen_vsync         .set_checked(Globals::screen_vsync);
    arrows_replay        .set_checked(Globals::user.arrows_replay);
    arrows_network       .set_checked(Globals::user.arrows_network);
    debris_amount        .set_number (Globals::user.debris_amount);
    debris_type          .set_number (Globals::user.debris_type);

    sound_load_driver    .set_checked(Globals::sound_load_driver);
    sound_volume         .set_number (Globals::user.sound_volume);
}



// Show Group: U.a. Texte fuer die gewaehlte Seite schreiben.
// Enthaelt etwas Zeichencode, der einmalig in der Hauptschleife,
// nicht in der Zeichenschleife arbeitet. Ist aber gaengig so.
void OptionMenu::show_group(const OptionGroup grp)
{
    if (grp == GROUP_MAX) return;
    Globals::user.option_group = grp;

    // Korrekten Button gedrueckt darstellen
    for (std::vector <TextButton> ::iterator itr = button_group.begin();
     itr != button_group.end(); ++itr) itr->set_off();
    button_group[grp].set_on();

    for (PtrVec::iterator itr = pointers[grp].begin();
     itr != pointers[grp].end(); ++itr)
     (*itr)->set_draw_required();

    rectfill(get_torbit().get_al_bitmap(),
     20, 60, LEMSCR_X-3, LEMSCR_Y-40-1, color[COL_API_M]);

    switch (grp) {
    case GROUP_GENERAL:
        draw_text(Language::option_user_name,              other_nx, user_name.get_y());
        draw_text(Language::option_user_name_ask,          check_nx, user_name_ask.get_y());
        draw_text(Language::option_language,               other_nx, language.get_y());
        break;

    case GROUP_CONTROLS:
        draw_text(Language::option_mouse_speed,            other_nx, mouse_speed.get_y());
        draw_text(Language::option_scroll_speed,           other_nx, scroll_speed.get_y());
        draw_text(Language::option_scroll_border,          check_nx, scroll_border.get_y());
        draw_text(Language::option_scroll_right,           check_nx, scroll_right.get_y());
        draw_text(Language::option_scroll_torus_x,         other_nx, scroll_torus_x.get_y());
        draw_text(Language::option_scroll_torus_y,         other_nx, scroll_torus_y.get_y());
        draw_text(Language::option_multiple_builders,      check_nx, multiple_builders.get_y());
        break;

    case GROUP_HOTKEYS:
        draw_text(Language::option_key_force_left,           key_t1, key_force_left.get_y());
        draw_text(Language::option_key_force_right,          key_t1, key_force_right.get_y());
        draw_text(Language::option_key_rate_minus,         key_t1, key_rate_minus.get_y());
        draw_text(Language::option_key_rate_plus,          key_t1, key_rate_plus.get_y());
        draw_text(Language::option_key_pause,              key_t1, key_pause.get_y());
        draw_text(Language::option_key_speed_slow,         key_t1, key_speed_slow.get_y());
        draw_text(Language::option_key_speed_fast,         key_t1, key_speed_fast.get_y());
        draw_text(Language::option_key_speed_turbo,        key_t1, key_speed_turbo.get_y());
        draw_text(Language::option_key_restart,            key_t1, key_restart.get_y());
        draw_text(Language::option_key_state_load,         key_t1, key_state_load.get_y());
        draw_text(Language::option_key_state_save,         key_t1, key_state_save.get_y());
        draw_text(Language::option_key_zoom,               key_t1, key_zoom.get_y());
        draw_text(Language::option_key_chat,               key_t1, key_chat.get_y());
        for (size_t i = 0; i < key_skill.size(); ++i) {
            std::string s = Lem::ac_to_string(static_cast <Lem::Ac> (i));
            Help::string_to_nice_case(s);
            draw_text(s, key_skill[i].get_x() + key_xl + 20,
                         key_skill[i].get_y());
        }
        draw_text(Language::option_key_nuke, key_nuke.get_x() + key_xl + 20,
                                             key_nuke.get_y());
        draw_text(Language::option_key_info_1, key_b3, 330);
        draw_text(Language::option_key_info_2, key_b3, 350);
        draw_text(Language::option_key_info_3, key_b3, 370);
        break;

    case GROUP_GRAPHICS:
        draw_text(Language::option_screen_resolution,      other_nx, screen_resolution_x.get_y());
        draw_text(Language::option_screen_windowed_res,    other_nx, screen_windowed_x.get_y());
        draw_text(Language::option_screen_windowed,        check_nx, screen_windowed.get_y());
        draw_text(Language::option_screen_scaling,         other_nx, screen_scaling.get_y());
        draw_text(Language::option_screen_border_colored,  check_nx, screen_border_colored.get_y());
        draw_text(Language::option_screen_vsync,           check_nx, screen_vsync.get_y());
        draw_text(Language::option_arrows_replay,          check_nx, arrows_replay.get_y());
        draw_text(Language::option_arrows_network,         check_nx, arrows_network.get_y());
        draw_text(Language::option_debris_amount,          other_nx, debris_amount.get_y());
        draw_text(Language::option_debris_type,            other_nx, debris_type.get_y());
        draw_text(Language::option_gfx_zero, 20, button_okay.get_y() - 40);
        draw_text(Language::option_info,     20, button_okay.get_y() - 60);
        break;

    case GROUP_SOUND:
        draw_text(Language::option_sound_load_driver,      check_nx, sound_load_driver.get_y());
        draw_text(Language::option_sound_volume,           other_nx, sound_volume.get_y());
        draw_text(Language::option_info, 20, button_okay.get_y() - 40);
        break;

    default:
        break;
    }
}



void OptionMenu::calculate()
{
    // Bei geaendertem Spielernamen die neue Userdatei laden.
    // Die alte wird jedoch nicht gespeichert. Dazu muss man erst auf OK
    // klicken und das Optionsmenue erneut aufrufen.
    // Ob der Name zu lang ist, prueft User::get_name().
    if (!Texttype::get_selection()
     && user_name.get_text() != Globals::user.get_name()) {
        Globals::user_name_last_used = user_name.get_text();
        Globals::user.load(            user_name.get_text());
        Globals::user.option_group = GROUP_GENERAL; // da sind wir ja
        reset_elements();
    }

    /////////////////////////
    // Angeklickte Buttons //
    /////////////////////////

    if ((button_clicked(button_okay) || hardware.get_mr())
     && !Texttype::get_selection() && !KeyButton::get_selection())
    {
        const int res_fx = atoi(screen_resolution_x.get_text().c_str());
        const int res_fy = atoi(screen_resolution_y.get_text().c_str());
        const int res_wx = atoi(screen_windowed_x.get_text().c_str());
        const int res_wy = atoi(screen_windowed_y.get_text().c_str());
        const bool full  = Globals::screen_fullscreen_now;

        // Only call set_screen_mode() if it's necessary. The main menu
        // flickers for a very short time. A different screen_scaling option
        // should also call the function, because the function sets
        // glob_gfx.h's clear_screen_at_next_blit to true, to clear option
        // menu remainders on the screen borders.
        bool call_ssm = false;
        if (        Globals::user.screen_scaling != screen_scaling.get_number()
         ||   full && Globals::screen_resolution_x != res_fx
         ||   full && Globals::screen_resolution_y != res_fy
         || ! full && Globals::screen_windowed_x   != res_wx
         || ! full && Globals::screen_windowed_y   != res_wy) call_ssm = true;

        // Die Werte aller Checkboxen und Buttons in die Optionen schreiben
        // Die Konfigurationsdatei wird gegen eventuelle Abstuerze oder
        // Allegro-Abschuesse schon am Ende dieser Abarbeitung gespeichert.
        Globals::user_name_ask          = user_name_ask.get_checked();

        if (Globals::user.language != language_nr)
         Language::set(Globals::user.language = language_nr);

        Globals::user.mouse_speed       = mouse_speed      .get_number();
        Globals::user.scroll_speed      = scroll_speed     .get_number();
        Globals::user.scroll_border     = scroll_border    .get_checked();
        Globals::user.scroll_right      = scroll_right     .get_checked();
        Globals::user.scroll_torus_x    = scroll_torus_x   .get_number();
        Globals::user.scroll_torus_y    = scroll_torus_y   .get_number();
        Globals::user.multiple_builders = multiple_builders.get_checked();

        Globals::user.key_force_left    = key_force_left   .get_scancode();
        Globals::user.key_force_right   = key_force_right  .get_scancode();
        Globals::user.key_rate_minus  = key_rate_minus .get_scancode();
        Globals::user.key_rate_plus   = key_rate_plus  .get_scancode();
        Globals::user.key_pause       = key_pause      .get_scancode();
        Globals::user.key_speed_slow  = key_speed_slow .get_scancode();
        Globals::user.key_speed_fast  = key_speed_fast .get_scancode();
        Globals::user.key_speed_turbo = key_speed_turbo.get_scancode();
        Globals::user.key_restart     = key_restart    .get_scancode();
        Globals::user.key_state_load  = key_state_load .get_scancode();
        Globals::user.key_state_save  = key_state_save .get_scancode();
        Globals::user.key_zoom        = key_zoom       .get_scancode();
        Globals::user.key_nuke        = key_nuke       .get_scancode();
        Globals::user.key_chat        = key_chat       .get_scancode();
        for (size_t i = 0; i < key_skill.size(); ++i)
         Globals::user.key_skill[i]   = key_skill[i]   .get_scancode();
        // There's only the Exploder hotkey in the options menu, but
        // it shall control both hotkeys.
        Globals::user.key_skill[Lem::EXPLODER2]
         = key_skill[Lem::EXPLODER].get_scancode();

        Globals::user.screen_windowed       = screen_windowed      .get_checked();
        Globals::user.screen_scaling        = screen_scaling       .get_number();
        Globals::user.screen_border_colored = screen_border_colored.get_checked();
        Globals::user.arrows_replay         = arrows_replay        .get_checked();
        Globals::user.arrows_network        = arrows_network       .get_checked();
        Globals::user.debris_amount         = debris_amount        .get_number();
        Globals::user.debris_type           = debris_type          .get_number();

        Globals::screen_resolution_x = atoi(screen_resolution_x.get_text().c_str());
        Globals::screen_resolution_y = atoi(screen_resolution_y.get_text().c_str());
        Globals::screen_windowed_x   = atoi(screen_windowed_x  .get_text().c_str());
        Globals::screen_windowed_y   = atoi(screen_windowed_y  .get_text().c_str());
        Globals::screen_vsync        = screen_vsync            .get_checked();

        Globals::sound_load_driver = sound_load_driver.get_checked();
        Globals::user.sound_volume = sound_volume     .get_number();

        // Use new resolution that's already written to globals
        if (call_ssm) set_screen_mode(Globals::screen_fullscreen_now);

        Globals::save_config_file();
        exit = true;
    }
    else if (button_clicked(button_cancel)
     && !Texttype::get_selection() && !KeyButton::get_selection()) {
        exit = true;
    }

    for (size_t i = 0; i < GROUP_MAX; ++i)
     if (button_clicked(button_group[i]))
     show_group(static_cast <OptionGroup> (i));

    // Eigentliche Schalter einer ausgewaehlten Optionsgruppe
    switch (Globals::user.option_group) {

    case GROUP_GENERAL:
        user_name.calculate(get_x(), get_y());
        button_clicked(user_name_ask);
        if (button_clicked(language)) {
            if (++language_nr == Language::MAX)
             language_nr       = Language::ENGLISH;
            language.set_text(Language::language_name[language_nr]);
        }
        break;

    case GROUP_CONTROLS:
        mouse_speed.calculate (get_x(), get_y());
        scroll_speed.calculate(get_x(), get_y());
        if (button_clicked(scroll_border) && !scroll_border && !scroll_right)
                           scroll_right.set_checked(true);
        if (button_clicked(scroll_right)  && !scroll_border && !scroll_right)
                           scroll_border.set_checked(true);
        button_clicked(scroll_torus_x);
        button_clicked(scroll_torus_y);
        button_clicked(multiple_builders);
        break;

    case GROUP_HOTKEYS:
        key_force_left   .calculate(get_x(), get_y());
        key_force_right  .calculate(get_x(), get_y());
        key_rate_minus .calculate(get_x(), get_y());
        key_rate_plus  .calculate(get_x(), get_y());
        key_pause      .calculate(get_x(), get_y());
        key_speed_slow .calculate(get_x(), get_y());
        key_speed_fast .calculate(get_x(), get_y());
        key_speed_turbo.calculate(get_x(), get_y());
        key_restart    .calculate(get_x(), get_y());
        key_state_load .calculate(get_x(), get_y());
        key_state_save .calculate(get_x(), get_y());
        key_zoom       .calculate(get_x(), get_y());
        key_nuke       .calculate(get_x(), get_y());
        key_chat       .calculate(get_x(), get_y());
        for (std::vector <KeyButton> ::iterator itr = key_skill.begin();
         itr != key_skill.end(); ++itr) itr->calculate(get_x(), get_y());
        break;

    case GROUP_GRAPHICS:
        screen_resolution_x.calculate();
        screen_resolution_y.calculate();
        screen_windowed_x.calculate();
        screen_windowed_y.calculate();
        button_clicked(screen_windowed);
        button_clicked(screen_scaling);
        button_clicked(screen_border_colored);
        button_clicked(screen_vsync);
        button_clicked(arrows_replay);
        button_clicked(arrows_network);
        button_clicked(debris_amount);
        button_clicked(debris_type);
        break;

    case GROUP_SOUND:
        button_clicked(sound_load_driver);
        sound_volume.calculate(get_x(), get_y());
        break;

    default:
        break;
    }
}



void OptionMenu::draw_self()
{
    button_okay  .draw();
    button_cancel.draw();

    // Vier Gruppenwahl-Buttons zeichnen
    for (std::vector <TextButton> ::iterator
     itr = button_group.begin(); itr != button_group.end(); ++itr) itr->draw();

    // Alle Buttons der aktuellen Seite zeichnen
    for (PtrVec::iterator itr = pointers[Globals::user.option_group].begin();
     itr != pointers[Globals::user.option_group].end(); ++itr) (*itr)->draw();
}

}
// Ende Namensraum Api
