/*
 * menu/options.h
 *
 * Das Optionsmenue, in dem viele Einstellungen der Konfigurationsdatei
 * bequem geaendert werden koennen.
 *
 * void reset_elements()
 *
 *   Ruft nochamls die globalen und die User-Variablen ab (aber ohne sie zuvor
 *   aus den entsprechenden Dateien neu zu laden) und setzt alle API-Elemente
 *   auf diese Werte.
 *
 *   Nach Aenderung des Spielernamens kann man beispielsweise die Spieler-
 *   datei von Hand laden und anschliessend reset_elements() aufrufen.
 *
 */

#pragma once

#include "../api/window.h"

namespace Api {
class OptionMenu : public Window {

private:

    static const unsigned check_x;  // X-Position von Checkoxen einfach 20
    static const unsigned check_nx; // X-Pos. fuer Checkbox-Beschriftungen, 60
    static const unsigned other_x;  // LEMSCR_X - 160, da Buttons 140 breit
    static const unsigned other_nx; // Irgendwas in der Mitte, dass es passt

    static const unsigned button_xl; // 140
    static const unsigned key_xl;
    static const unsigned frame_y;
    static const unsigned frame_yl;

    static const unsigned key_b1;    // 1st row's buttons
    static const unsigned key_t1;    // 1st row's text
    static const unsigned key_b2;
    static const unsigned key_t2;
    static const unsigned key_b3;
    static const unsigned key_t3;

    bool       exit;
    int        hotkey_add_counter;

    enum OptionGroup {
        GROUP_GENERAL,
        GROUP_CONTROLS,
        GROUP_HOTKEYS,
        GROUP_GRAPHICS,
        GROUP_SOUND,
        GROUP_MAX
    };

    TextButton button_okay;
    TextButton button_cancel;

    std::vector <TextButton> button_group;

    // Nimmt einen Zeiger zu jedem erstellten Button auf. Alle Zeiger zu
    // generellen Einstellungen landen im Vektor pointers[GROUP_GENERAL] etc.
    // Man kann so rasch ueber alle notwendigen Buttons iterieren.
    typedef std::vector <Element*> PtrVec;
    std::vector <PtrVec> pointers;

    Texttype   user_name;
    Checkbox   user_name_ask;
    TextButton language;
    int        language_nr;

    Number     mouse_speed;
    Number     scroll_speed;
    Checkbox   scroll_border;
    Checkbox   scroll_right;
    EnumButton scroll_torus_x;
    EnumButton scroll_torus_y;
    Checkbox   multiple_builders;

    KeyButton  key_force_left;
    KeyButton  key_force_right;
    KeyButton  key_rate_minus;
    KeyButton  key_rate_plus;
    KeyButton  key_pause;
    KeyButton  key_speed_slow;
    KeyButton  key_speed_fast;
    KeyButton  key_speed_turbo;
    KeyButton  key_restart;
    KeyButton  key_state_load;
    KeyButton  key_state_save;
    KeyButton  key_zoom;
    KeyButton  key_nuke;
    KeyButton  key_chat;
    std::vector <KeyButton> key_skill;

    Texttype   screen_resolution_x;
    Texttype   screen_resolution_y;
    Texttype   screen_windowed_x;
    Texttype   screen_windowed_y;
    Checkbox   screen_windowed;
    EnumButton screen_scaling;
    Checkbox   screen_border_colored;
    Checkbox   screen_vsync;
    Checkbox   arrows_replay;
    Checkbox   arrows_network;
    EnumButton debris_amount;
    EnumButton debris_type;

    Checkbox   sound_load_driver;
    Number     sound_volume;

    void add_to_pointers_hotkeys(KeyButton*);
    void reset_elements();
    void show_group (const OptionGroup);

    // Kopierverbot
    OptionMenu      (const OptionMenu&);
    void operator = (const OptionMenu&);

public:

                OptionMenu();
    virtual    ~OptionMenu();

    void        calculate();
    inline bool get_exit() { return exit; }

protected:

    void        draw_self();

}; // Ende Klasse OptionMenu
}  // Ende Namensraum Api
