/*
 * other/globals.cpp
 *
 */

#include <fstream>

#include "io.h"
#include "language.h"
#include "myalleg.h"

#include "../lemming/lem_enum.h" // initialize the strings in there

#define CONCAT(a, b, c, d) (a ## b ## c ## d)

namespace Globals {

const unsigned long version      CONCAT(2010, 03, 11, 00);
const unsigned long version_min  CONCAT(2010, 03, 09, 01);

User        user                 ("");
std::string user_name_last_used  ("");

bool        user_name_ask        (false);

int         screen_resolution_x  (   0);
int         screen_resolution_y  (   0);
int         screen_windowed_x    ( 640);
int         screen_windowed_y    ( 480);
bool        screen_vsync         (false);
bool        screen_fullscreen_now(false);

bool        sound_load_driver    (true);

std::string ip_last_used         (net_ip_localhost);
std::string ip_central_server    ("flux.camanis.net");
unsigned    server_port          (22934);



// Konfigurationsdatei korrekt verarbeiten
void load_config_file()
{
    // Sprache ruecksetzen, weil sie zu Spielbeginn Englisch ist, damit
    // korrekte Log-Meldungen geschrieben werden
    user.language = Language::NONE;

    // Needed for loading the user file later, as it contains skill hotkeys
    Lem::initialize_strings_this_gets_called_from_globals_load_config_file();

    // Jetzt aber die Datei laden.
    std::vector <IO::Line> lines;
    IO::fill_vector_from_file(lines, Globals::file_config);

    for (IO::LineIt i = lines.begin(); i != lines.end(); ++i) switch(i->type) {
    // String setzen
    case '$':
        if      (i->text1 == config_user_name_last_used) user.load(            i->text2);
        else if (i->text1 == config_ip_last_used       ) ip_last_used        = i->text2;
        else if (i->text1 == config_ip_central_server  ) ip_central_server   = i->text2;
        break;

    // Ganzzahlwert setzen
    case '#':
        if      (i->text1 == config_user_name_ask      ) user_name_ask       = i->nr1;

        else if (i->text1 == config_screen_resolution_x) screen_resolution_x = i->nr1;
        else if (i->text1 == config_screen_resolution_y) screen_resolution_y = i->nr1;
        else if (i->text1 == config_screen_windowed_x  ) screen_windowed_x   = i->nr1;
        else if (i->text1 == config_screen_windowed_y  ) screen_windowed_y   = i->nr1;
        else if (i->text1 == config_screen_vsync       ) screen_vsync        = i->nr1;

        else if (i->text1 == config_sound_load_driver  ) sound_load_driver   = i->nr1;

        else if (i->text1 == config_server_port        ) server_port         = i->nr1;
        break;
    }

    Language::set(user.language);
}



void save_config_file()
{
    std::ofstream file(file_config.c_str());

    file
     << IO::LineDollar(config_user_name_last_used,     user.get_name())
     << IO::LineHash  (config_user_name_ask,           user_name_ask)
     << std::endl

     << IO::LineHash  (config_screen_resolution_x,     screen_resolution_x)
     << IO::LineHash  (config_screen_resolution_y,     screen_resolution_y)
     << IO::LineHash  (config_screen_windowed_x,       screen_windowed_x)
     << IO::LineHash  (config_screen_windowed_y,       screen_windowed_y)
     << IO::LineHash  (config_screen_vsync,            screen_vsync)
     << std::endl

     << IO::LineHash  (config_sound_load_driver,       sound_load_driver)
     << std::endl

     << IO::LineDollar(config_ip_last_used,            ip_last_used)
     << IO::LineDollar(config_ip_central_server,       ip_central_server)
     << IO::LineHash  (config_server_port,             server_port);

    file.close();
    user.save();
}
// Ende save_config_file

}
// Ende Namensraum Globals
