/*
 * other/globals.h
 *
 * Konstanten, Optionen und andere globale Variablen, die fuer mehr als ein
 * Programmmodul von Wichtigkeit sind
 *
 * void initialize()
 *
 *   Wird aufgerunfen von Funktionen, die schon sehr fruehzeitig gesetzte
 *   globale Konstanten, etwa die sprachunabhaengigen Strings, benoetigen.
 *
 */

#pragma once

#include <string>

#include "user.h"



namespace Globals {

void load_config_file();
void save_config_file();

// Konstanten
// version_min legt die aelteste Version fest, mit der wir kommunizieren
// duerfen. Der Client prueft die Server-Version und der Server die
// Client-Version. Wenn einem der jeweils andere zu alt ist, wird
// die Verbindung abgebrochen.
extern const unsigned long version;
extern const unsigned long version_min;

static const size_t skill_max        = 8;
static const int    scroll_torus_max = 2;

extern User        user;
extern std::string user_name_last_used;
extern bool        user_name_ask;

extern int         screen_resolution_x;
extern int         screen_resolution_y;
extern int         screen_windowed_x;
extern int         screen_windowed_y;
extern bool        screen_vsync;
extern bool        screen_fullscreen_now;

extern bool        sound_load_driver;

extern std::string ip_last_used;
extern std::string ip_central_server;
extern unsigned    server_port;



static const std::string

// Diverses
empty_string                 = "",
net_ip_localhost             = "127.0.0.1",

// Dateien laden
ext_level                    = ".txt",
ext_level_orig               = ".lvl",
ext_replay                   = ".txt",
ext_user                     = ".txt",
mask_anything                = "*.*",
mask_ext_level               = "*.txt",
mask_ext_level_orig          = "*.lvl",
mask_ext_replay              = "*.txt",
mask_ext_user                = "*.txt",
mask_ext_bmp                 = "*.bmp",
mask_ext_tga                 = "*.tga",
mask_ext_pcx                 = "*.pcx",

// Unterteilung der Bilddateien nach Prae-Endung
pre_ext_null                 = ":",
pre_ext_internal             = "I",
pre_ext_steel                = "S",
pre_ext_deco                 = "D",
pre_ext_hatch                = "H",
pre_ext_goal                 = "G",
pre_ext_trap                 = "T",
pre_ext_fire                 = "F",
pre_ext_water                = "W",
pre_ext_oneway_left          = "L",
pre_ext_oneway_right         = "R",

// Wichtige Verzeichnisse
dir_levels                   = "./levels/",
dir_levels_single            = "./levels/",
dir_levels_network           = "./levels/network/",
dir_replay                   = "./replay/",
dir_data                     = "./data/",
dir_data_bitmap              = "./data/bitmap/",
dir_data_sound               = "./data/sound/",
dir_data_user                = "./data/user/",
dir_bitmap                   = "./bitmap/",
dir_bitmap_orig              = "./bitmap/orig/",
dir_bitmap_orig_l1           = dir_bitmap_orig + "L1/",
dir_bitmap_orig_l2           = dir_bitmap_orig + "L2/",

// Wichtige Dateien
file_config                  = "./data/config.txt",
file_log                     = "./data/log.txt",
file_level_network           = "./data/netlevel.txt",

file_bitmap_api_number       = dir_data_bitmap + "api_numb.I",
file_bitmap_checkbox         = dir_data_bitmap + "checkbox.I",
file_bitmap_debris           = dir_data_bitmap + "debris.I",
file_bitmap_edit_flip        = dir_data_bitmap + "edit_flp.I",
file_bitmap_edit_hatch       = dir_data_bitmap + "edit_hat.I",
file_bitmap_edit_panel       = dir_data_bitmap + "edit_pan.I",
file_bitmap_edit_big         = dir_data_bitmap + "edit_big.I",
file_bitmap_explosion        = dir_data_bitmap + "explode.I",
file_bitmap_fuse_flame       = dir_data_bitmap + "fuse_fla.I",
file_bitmap_game_arrow       = dir_data_bitmap + "game_arr.I",
file_bitmap_game_panel       = dir_data_bitmap + "game_pan.I",
file_bitmap_game_pause       = dir_data_bitmap + "game_pau.I",
file_bitmap_game_replay      = dir_data_bitmap + "game_rep.I",
file_bitmap_lemming          = dir_data_bitmap + "lemming.I",
file_bitmap_mouse            = dir_data_bitmap + "mouse.I",
file_bitmap_preview_icon     = dir_data_bitmap + "prev_ico.I",

file_bitmap_font_big         = dir_data_bitmap + "font_big.I.bmp",
file_bitmap_font_med         = dir_data_bitmap + "font_med.I.bmp",
file_bitmap_font_sml         = dir_data_bitmap + "font_sml.I.bmp",

// Config-Schluessel
config_user_name_last_used   = "USER_NAME",
config_user_name_ask         = "USER_NAME_ASK",

config_screen_resolution_x   = "SCREEN_RESOLUTION_X",
config_screen_resolution_y   = "SCREEN_RESOLUTION_Y",
config_screen_windowed_x     = "SCREEN_WINDOWED_X",
config_screen_windowed_y     = "SCREEN_WINDOWED_Y",
config_screen_vsync          = "SCREEN_VSYNC",

config_sound_load_driver     = "SOUND_LOAD_DRIVER",

config_ip_last_used          = "IP_LAST_USED",
config_ip_central_server     = "IP_CENTRAL_SERVER",
config_server_port           = "SERVER_PORT",

// Leveldatei-Schluessel
level_built                  = "BUILT",
level_author                 = "AUTHOR",
level_name_german            = "GERMAN",
level_name_english           = "ENGLISH",
level_size_x                 = "SIZE_X",
level_size_y                 = "SIZE_Y",
level_start_x                = "START_X",
level_start_y                = "START_Y",
level_torus_x                = "TORUS_X",
level_torus_y                = "TORUS_Y",
level_seconds                = "SECONDS",
level_lemmings               = "LEMMINGS",
level_required               = "REQUIRED",
level_rate                   = "RATE",

// Replaydatei-Schluessel
replay_version_min           = "VERSION_REQUIRED",
replay_built_required        = "BUILT_REQUIRED",
replay_level_filename        = "FILENAME",
replay_friend                = "FRIEND",
replay_player                = "PLAYER",
replay_permu                 = "PERMUTATION",
replay_update                = "UPDATE",
replay_rate                  = "RATE",
replay_skill                 = "SKILL",
replay_aim                   = "AIM",
replay_nuke                  = "NUKE",
replay_lemming               = "LEMMING",

// Userdatei-Schluessel
user_language                = "LANGUAGE",
user_option_group            = "OPTION_GROUP",

user_mouse_speed             = "MOUSE_SPEED",
user_scroll_speed            = "SCROLL_SPEED",
user_scroll_border           = "SCROLL_BORDER",
user_scroll_right            = "SCROLL_RIGHT",
user_scroll_torus_x          = "SCROLL_TORUS_X",
user_scroll_torus_y          = "SCROLL_TORUS_Y",
user_multiple_builders       = "MULTIPLE_BUILDERS",

user_screen_scaling          = "SCREEN_SCALING",
user_screen_border_colored   = "SCREEN_BORDER_COLORED",
user_screen_windowed         = "SCREEN_WINDOWED",
user_arrows_replay           = "ARROWS_REPLAY",
user_arrows_network          = "ARROWS_NETWORK",
user_debris_amount           = "DEBRIS_AMOUNT",
user_debris_type             = "DEBRIS_TYPE",

user_sound_volume            = "SOUND_VOLUME",

user_single_last_dir         = "SINGLE_LAST_DIR",
user_single_last_file        = "SINGLE_LAST_FILE",
user_network_last_dir        = "NETWORK_LAST_DIR",
user_network_last_file       = "NETWORK_LAST_FILE",
user_replay_last_dir         = "REPLAY_LAST_DIR",
user_replay_last_file        = "REPLAY_LAST_FILE",

user_editor_last_dir_terrain = "EDITOR_LAST_DIR_TERRAIN",
user_editor_last_dir_steel   = "EDITOR_LAST_DIR_STEEL",
user_editor_last_dir_goal    = "EDITOR_LAST_DIR_GOAL",
user_editor_last_dir_hatch   = "EDITOR_LAST_DIR_HATCH",
user_editor_last_dir_deco    = "EDITOR_LAST_DIR_DECO",
user_editor_last_dir_hazard  = "EDITOR_LAST_DIR_HAZARD",

user_key_force_left          = "KEY_FORCE_LEFT",
user_key_force_right         = "KEY_FORCE_RIGHT",
user_key_rate_minus          = "KEY_RATE_MINUS",
user_key_rate_plus           = "KEY_RATE_PLUS",
user_key_pause               = "KEY_PAUSE",
user_key_speed_slow          = "KEY_SPEED_SLOW",
user_key_speed_fast          = "KEY_SPEED_FAST",
user_key_speed_turbo         = "KEY_SPEED_TURBO",
user_key_restart             = "KEY_RESTART",
user_key_state_load          = "KEY_STATE_LOAD",
user_key_state_save          = "KEY_STATE_SAVE",
user_key_zoom                = "KEY_ZOOM",
user_key_nuke                = "KEY_NUKE",
user_key_chat                = "KEY_CHAT";

}
// Ende Namensraum Globals
