/*
 * hardware.cpp
 *
 */

#include "hardware.h"
#include "help.h" // Timer fuer Doppelklick-Geschw.
#include "log.h"

const unsigned int Hardware::doubleclick_speed(Help::timer_ticks_per_second/3);
const unsigned int Hardware::doubleclick_for60(20);

Hardware::Hardware()
:
    mouse_own_x   (LEMSCR_X/2),
    mouse_own_y   (LEMSCR_Y/2),
    mickey_x(0),
    mickey_y(0)
{
    mouse_buffer[0]            = false;
    mouse_buffer[1]            = false;
    mouse_since [0]            = 0;
    mouse_since [1]            = 0;
    key_from_buffer            = -1;
    key_from_buffer_ascii      = -1;
    clear_keybuf();
}

Hardware::~Hardware() {}



void Hardware::main_loop() {

    //////////
    // Maus //
    //////////

    mickey_x = mouse_x - LEMSCR_X/2;
    mickey_y = mouse_y - LEMSCR_Y/2;
    if (Globals::screen_fullscreen_now) {
        mickey_x = mickey_x * (int) Globals::user.mouse_speed / 20;
        mickey_y = mickey_y * (int) Globals::user.mouse_speed / 20;
    }
    if (mickey_x || mickey_y) position_mouse(LEMSCR_X/2, LEMSCR_Y/2);

    mouse_own_x += mickey_x;
    mouse_own_y += mickey_y;
    if (mouse_own_x < 0)         mouse_own_x = 0;
    if (mouse_own_y < 0)         mouse_own_y = 0;
    if (mouse_own_x >= LEMSCR_X) mouse_own_x = LEMSCR_X - 1;
    if (mouse_own_y >= LEMSCR_Y) mouse_own_y = LEMSCR_Y - 1;

    // Buttons
    this->poll_mouse(true);

    //////////////
    // Tastatur //
    //////////////

    ctrl  = (key[KEY_LCONTROL] || key[KEY_RCONTROL]);
    shift = (key[KEY_LSHIFT  ] || key[KEY_RSHIFT  ]);
    alt   = (key[KEY_ALT     ] || key[KEY_ALTGR   ]);

    if (keypressed()) {
        key_from_buffer       = readkey();
        key_from_buffer_ascii = key_from_buffer & 0xff;
        key_from_buffer       = key_from_buffer >> 8;
    }
    else {
        key_from_buffer = -1;
        key_from_buffer_ascii = -1;
    }

}
// Ende Hardware::main_loop()



void Hardware::poll_mouse(const bool actual_tick)
{
    // Moegliche Mausaktivitaet puffern
    // Dies immer tun, egal ob ein Tick ansteht oder nicht]
    for (unsigned b = 0; b < 2; ++b)
     mouse_buffer[b] = mouse_buffer[b] || mouse_b & (b+1);

    // Wenn wirklich ein Tick ansteht
    if (actual_tick) for (unsigned b = 0; b < 2; ++b) {
        ++mouse_since[0];
        // Wurde seit dem letzten Tick die Maus gedrueckt?
        if (mouse_buffer[b]) {
            // Schon in den vorherigen Ticks gedrueckt?
            if (mouse_hold[b]) {
                mouse_click[b]  = false;
                mouse_double[b] = false;
            }
            // Jetzt im ersten Tick geklickt?
            else {
                mouse_click[b] = true;
                // Doppelklick?
                if (mouse_since [b] > doubleclick_speed) {
                    mouse_double[b] = false;
                    mouse_since [b] = 0;
                } else {
                    mouse_double[b] = true;
                    mouse_since [b] = doubleclick_speed; // Gegen Dreifachklick
                }
            }
            ++mouse_hold[b];
            mouse_release[b] = false;
        }
        // Maus nicht gedrueckt, also Puffer war == false
        else {
            mouse_click  [b] = false;
            mouse_double [b] = false;
            mouse_release[b] = mouse_hold[b];
            mouse_hold   [b] = 0;
        }
        // Egal, ob Maus gedrueckt war oder nicht:
        // Puffer hat seine Dienste getan, fuer naechsten Tick neu sammeln
        mouse_buffer[b] = false;
    }
}



// ############################################################################
// ############################################################################
// ############################################################################



bool Hardware::key_release(unsigned int scancode) {
    if      (!key_buffer[scancode] && !key[scancode]) return false;
    else if ( key_buffer[scancode] && !key[scancode]) {
        key_buffer[scancode] = false;
        return true; // Das ist, was wir wollen.
    }
    else {
        key_buffer[scancode] = true;
        return false;
    }
}
// Ende von key_release



bool Hardware::key_enter_once()
{
    // Don't trigger on fullscreen/windowed switch
    return !get_alt() && (key_once(KEY_ENTER) || key_once(KEY_ENTER_PAD));
}



bool Hardware::key_enter_release()
{
    return key_release(KEY_ENTER) || key_release(KEY_ENTER_PAD);
}



void Hardware::freeze_mouse_x()
{
    mouse_own_x -= mickey_x;
    if (mouse_own_x < 0)         mouse_own_x = 0;
    if (mouse_own_x >= LEMSCR_X) mouse_own_x = LEMSCR_X - 1;
    mickey_x = 0;
}
void Hardware::freeze_mouse_y()
{
    mouse_own_y -= mickey_y;
    if (mouse_own_y < 0)         mouse_own_y = 0;
    if (mouse_own_y >= LEMSCR_Y) mouse_own_y = LEMSCR_Y - 1;
    mickey_y = 0;
}



// Und das einzelne Objekt erschaffen
Hardware hardware;
