/*
 * help.h
 *
 * Hilfsfunktionen fuer L++
 *
 * "Was will ich mit std, zwei Doppelpunkten, "cout", und dann dem
 * bitweisen Shift-Operator? printf kann ALLES!" -- Malte Dehling
 *
 */

#pragma once

#include <cstdlib> // RAND_MAX
#include <string>
#include <vector>

#include "globals.h"
#include "../network/net_t.h"    // Uint32

#include "../graphic/glob_gfx.h" // Standardfarbe fuer Schattentext
#include "../graphic/torbit.h"   // Textzeichnen auf Torus statt BITMAP*

namespace Help {

    // Timer-Funktionen
    extern volatile Uint32 timer_ticks;
    extern const    int    timer_ticks_per_second;

    void     timer_start();
    unsigned get_timer_ticks_per_draw();

    double hypot        (const int,    const int,    const int, const int);
    double random_double(const double, const double); // zw. 1. und 2. Argument
    int    mod          (const int,    const int); // besser als % im Negativen

    struct MainArgs {
        int  scr_f;
        int  scr_x;
        int  scr_y;
        bool sound_load_driver;
    };
    MainArgs parse_main_arguments(int, char*[]);
}

    // Stringfunktionen
    std::string  operator +          (const std::string&, const int);
    std::string& operator +=         (      std::string&, const int);
    std::string  operator +          (const std::string&, const long);
    std::string& operator +=         (      std::string&, const long);
    std::string  operator +          (const std::string&, const unsigned);
    std::string& operator +=         (      std::string&, const unsigned);
    std::string  operator +          (const std::string&, const unsigned long);
    std::string& operator +=         (      std::string&, const unsigned long);

namespace Help {

    std::string version_to_string   (const unsigned long);

    void string_to_nice_case    (std::string&); // Alle ausser 1. Bch.
    void string_remove_extension(std::string&);
    void string_remove_dir      (std::string&);
    void string_cut_to_dir      (std::string&);
    void string_shorten         (std::string&, const FONT*, const int);
    std::string
         string_get_extension    (const std::string&);
    char string_get_pre_extension(const std::string&); // 0, wenn keine
    bool string_ends_with        (const std::string&, const std::string&);

    void draw_shaded_text         (Torbit&, FONT*, const char*,
                                   int, int, int, int, int);
    void draw_shadow_text         (Torbit&, FONT*, const char*,
                                // x    y    Farbe, Schattenfarbe
                                   int, int, int = color[COL_TEXT],
                                   int = color[COL_API_SHADOW]);

    void draw_shaded_centered_text(Torbit&, FONT*, const char*,
                                   int, int, int, int, int);
    void draw_shadow_centered_text(Torbit&, FONT*, const char*,
                                // x    y    Farbe, Schattenfarbe
                                   int, int, int = color[COL_TEXT],
                                   int = color[COL_API_SHADOW]);

    void draw_shadow_fixed_number (Torbit&, FONT*,
                                   int, int, int, int=color[COL_TEXT],
                                   bool = false, int=color[COL_API_SHADOW]);
    void draw_shadow_fixed_text   (Torbit&, FONT*, const std::string&,
                                   int, int, int=color[COL_TEXT], bool = false,
                                   int=color[COL_API_SHADOW]);
    void draw_shadow_fixed_updates_used(Torbit&, FONT*,
                                   int, int, int, int=color[COL_TEXT],
                                   bool = false,  int=color[COL_API_SHADOW]);



    // Funktionszeigertypendefinition fr die kommenden Suchfunktionen
    typedef void (*DoStr)(std::string&, void*);

    void find_files(const std::string&, const std::string&, DoStr,void*);
    // Die Funktion durchsucht das mit dem ersten Argument angegebene Verzeich-
    // nis nach Dateien, die dem zweiten Argument "*.abc" entsprechen. Es
    // wird ein Pfad/Dateiname relativ zur Lemmings-Executable gefunden.
    // DoStr ist ein Funktionszeiger. Die entsprechende Funktion wird fr jeden
    // String aufgerufen, der gefunden wird.

    void find_dirs(std::string, DoStr, void*);
    // Wie find_files, allerdings wird hier im angegebenen Ordner nach Unter-
    // verzeichnissen gesucht mit Ausnahme von "." und "..". Diese Funktion
    // bentigt keine Suchmaske und somit kein drittes Argument.

    void find_tree(std::string, const std::string&, DoStr, void*);
    // Wie find_files, schliet allerdings auch alle Unterverzeichnisse in der
    // Suche nach passenden Dateien ein.

    bool dir_exists(const std::string&);
}
