/*
 * log.cpp
 *
 */

#include "log.h"
#include "globals.h"
#include "language.h" // Error and Info text
#include "help.h"     // timer_ticks

#include <cstdio>     // sprintf

Log::Log()
:
    file(Globals::file_log.c_str(), std::ios::app)
{
    priority = INFO;
    something_was_logged_already_this_session = false;
}

Log::~Log() {
    file.close();
}

// Create the singleton
Log Log::singleton;



void Log::log(Priority p, const std::string& s) {
    if ((singleton.priority == ERROR && p == ERROR)
     || (singleton.priority == INFO && p == ERROR)
     || (singleton.priority == INFO && p == INFO)) {

        char time[10]; // Time string, will be written later
        sprintf(time, "%9.2f", (float) Help::timer_ticks
                                     / Help::timer_ticks_per_second);
        // Free line between to L++ programm sessions
        if (!singleton.something_was_logged_already_this_session) {
             singleton.something_was_logged_already_this_session = true;
             singleton.file << std::endl;
        }
        // Output the line
        singleton.file << time << ' ';
        if (p == ERROR) singleton.file << Language::log_error;
        else            singleton.file << Language::log_info;
        singleton.file << ' ' << s << std::endl;
    }
}

void Log::log(Priority p, int i) {
    if ((singleton.priority == ERROR && p == ERROR)
     || (singleton.priority == INFO  && p == ERROR)
     || (singleton.priority == INFO  && p == INFO)) {

        char time[10];
        sprintf(time, "%9.2f", (float) Help::timer_ticks
                                     / Help::timer_ticks_per_second);
        if (!singleton.something_was_logged_already_this_session) {
             singleton.something_was_logged_already_this_session = true;
             singleton.file << std::endl;
        }
        singleton.file << time << ' ';
        if (p == ERROR) singleton.file << Language::log_error;
        else            singleton.file << Language::log_info;
        singleton.file << ' ' << i << std::endl;
    }
}

void Log::log(Priority p, const std::string& s, int i) {
    if ((singleton.priority == ERROR && p == ERROR)
     || (singleton.priority == INFO  && p == ERROR)
     || (singleton.priority == INFO  && p == INFO)) {

        char time[10];
        sprintf(time, "%9.2f", (float) Help::timer_ticks
                                     / Help::timer_ticks_per_second);
        if (!singleton.something_was_logged_already_this_session) {
             singleton.something_was_logged_already_this_session = true;
             singleton.file << std::endl;
        }
        singleton.file << time << ' ';
        if (p == ERROR) singleton.file << Language::log_error;
        else            singleton.file << Language::log_info;
        singleton.file << ' ' << s << ' ' << i << std::endl;
    }
}
