/*
 * other/main.cpp
 *
 *       XXXX    XXXX
 *       XXXX    XXXX
 *   XXXXXXXXXXXX
 *   XXXXXXXXXXXX
 *   XXXXXXXXOOOO
 *   XXXXXXXXOOOO             _______
 *       OOOOOOOOOOOO         \     /              ___            ___
 *       OOOOOOOOOOOO          |   |              |   |          |   |
 *       OOOO####              |   |           ___|   |___    ___|   |___
 *       OOOO####              |   |          |           |  |           |
 *   OOOO########              |   |    ___   |___     ___|  |___     ___|
 *   OOOO########              |   |___|   |      |   |          |   |
 *   OOOO########    OOOO      |           |      |___|          |___|
 *   OOOO########    OOOO     /____________|
 *       ########    OOOO
 *       ########    OOOO
 *   ########    OOOO
 *   ########    OOOO
 *   OOOOOOOO
 *   OOOOOOOO
 *
 *
 *
 * Introduction
 *
 *   This is the main file for L++. The game is written in C++ and uses
 *   two libraries:
 *
 *   - Allegro (www.allegro.cc)  for graphics, sound and input,
 *   - enet    (enet.bespin.org) for networking.
 *
 * License
 *
 *   L++ is free software.
 *
 *   You may do anything you'd like with this game. Download, play, use, alter
 *   and spread L++ as you wish. There are no restrictions. I am just not
 *   responsible for any bugs, loss of data, or the like.
 *
 *   I believe that programming should be an entertaining hobby, not fiddling
 *   around with license restrictions. I've had and still have a lot of fun
 *   with the development and playtesting of this game. You should have as
 *   as possible, too.
 *
 * About the source code itself
 *
 *   The source code consists of about 850 KB. It consists of roughly 27,000
 *   lines. 18,000 of those are code lines, the rest are comments and empty
 *   lines.
 *
 *   The core of the program is an object of class LMain. Usually I have
 *   commented each class in its header file. For LMain, this is other/lmain.h.
 *
 * Have fun going through the souce code!
 *
 * Simon
 *
 */

#include "lmain.h" // Main object, as commented upon above

int main(int argc, char* argv[])
{
    // Initialize Allegro. I need this for loading the config file, as
    // I use the exists() function from Allegro.
    allegro_init();

    Language::set(Language::GERMAN);
    Help::timer_start();
    Globals::load_config_file();
    Help::MainArgs margs = Help::parse_main_arguments(argc, argv);

    // Allegro preparations, no graphics function are called yet
    install_keyboard();
    install_mouse();
    if (margs.sound_load_driver) Sound::initialize();

    // Allegro graphics
    set_color_depth(16);
    set_screen_mode(margs.scr_f, margs.scr_x, margs.scr_y); // in glob_gfx.h
    set_window_title(Language::lemmings_plus_plus.c_str());

    load_all_bitmaps();
    Network::initialize();

    // Main loop. See other/lmain.cpp for this.
    LMain* l_main = new LMain;
    l_main->main_loop();
    delete l_main;

    // Clean up
    Globals::save_config_file();

    Network::deinitialize();
    Sound::deinitialize();
    destroy_all_bitmaps();

    // don't call allegro_exit(), doing that causes the program
    // to not terminate in rare cases
    return 0;
}
END_OF_MAIN()
