/*
 * server/s_main.cpp
 *
 * This is the main source file for the L++ server.
 *
 * This program runs non-interactively and realizes the functionalities a
 * hosting normal L++ player does to his clients.
 *
 * The L++ server shares some source files with L++ itself. The only distinct
 * sources are those in the src/server/ subdirectory like this file.
 *
 * Simon
 *
 */

#include <iostream> // initial msg: this is the server, end it with keypress

#include "../network/server.h"
#include "../network/net_t.h"

#include "../other/myalleg.h"

// Timer
volatile Uint32 timer_ticks = 0;

void timer_increment_ticks() {
    ++timer_ticks;
}
void timer_start() {
    LOCK_VARIABLE (timer_ticks);
    LOCK_FUNCTION (timer_increment_ticks);
    install_int_ex(timer_increment_ticks,
                   BPS_TO_TIMER(NETWORK_UPDATES_PER_SECOND));
}



int main()
{
    allegro_init();
    install_keyboard(); // ESC terminates the server
    set_window_title("L++Serv, [ESC] to quit");

    enet_initialize();

    timer_start();

    // Create server. Note that we use the number of updates per second as
    // the number of ticks per second: Our timer is slower than L++'s.
    NetServer* server = new NetServer(true, 22934,
     NETWORK_UPDATES_PER_SECOND, &timer_ticks);
    std::cout << "This is the dedicated L++ server. Press [ESC] to exit."
              << std::endl;

    // Main loop
    while (!key[KEY_ESC]) {
        server->calculate();
        rest(1);
    }
    delete server;

    enet_deinitialize();
    allegro_exit();

    return 0;
}
END_OF_MAIN()
